(*
  Object Pascal translation of the Altap Salamander SDK library.
  Original source: spl_arc.h

  Portions created by ALTAP are Copyright (c) ALTAP, spol. s r.o.
  All rights reserved.

  Portions created by LostGenius Posse are Copyright (c) 2007, LostGenius Posse (Timo 'jomanto' Boll)
  All rights reserved.

  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

      * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
      * Redistributions in binary form may reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
      * Neither the name of the LostGenius Posse nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*)
(*
  History:

  2007-06-01: initial version
*)
unit spl_arc;

interface

uses
  winliblt,
  spl_com;

type
//
// ****************************************************************************
// CPluginInterfaceForArchiverAbstract
//

  TPluginInterfaceForArchiverAbstract = class
    // funkce pro "panel archiver view"; vola se pro nacteni obsahu archivu 'fileName';
    // obsah se plni do objektu 'dir'; Salamander zjisti obsah
    // pluginem pridanych sloupcu pomoci interfacu 'pluginData' (pokud plugin sloupce
    // nepridava, vraci 'pluginData'==NULL); vraci TRUE pri uspesnem nacteni obsahu archivu,
    // pokud vrati FALSE, navratova hodnota 'pluginData' se ignoruje (data v 'dir' je potreba
    // uvolnit pomoci 'dir.Clear(pluginData)', jinak se uvolni jen Salamanderovska cast dat);
    // 'salamander' je sada uzitecnych metod vyvezenych ze Salamandera,
    // POZOR: soubor fileName take nemusi existovat (pokud je otevren v panelu a odjinud smazan),
    // ListArchive neni volan pro soubory nulove delky, ty maji automaticky prazdny obsah,
    // pri pakovani do takovych souboru se soubor pred volanim PackToArchive smaze (pro
    // kompatibilitu s externimi pakovaci)
    function ListArchive( salamander : TSalamanderForOperationsAbstract; fileName : PChar;
                          dir : TSalamanderDirectoryAbstract; var pluginData : TPluginDataInterfaceAbstract ) : BOOL; virtual; stdcall; abstract;

    // funkce pro "panel archiver view", vola se pri pozadavku na rozpakovani souboru/adresaru
    // z archivu 'fileName' do adresare 'targetDir' z cesty v archivu 'archiveRoot'; 'pluginData'
    // je interface pro praci s informacemi o souborech/adresarich, ktere jsou specificke pluginu
    // (napr. data z pridanych sloupcu; jde o stejny interface, ktery vraci metoda ListArchive
    // v parametru 'pluginData' - takze muze byt i NULL); soubory/adresare jsou zadany enumeracni
    // funkci 'next' jejimz parametrem je 'nextParam'; vraci TRUE pri uspesnem rozpakovani (nebyl
    // pouzit Cancel, mohl byt pouzit Skip) - zdroj operace v panelu je odznacen, jinak vraci
    // FALSE (neprovede se odznaceni); 'salamander' je sada uzitecnych metod vyvezenych ze
    // Salamandera
    function UnpackArchive( salamander : TSalamanderForOperationsAbstract; fileName : PChar;
                            pluginData : TPluginDataInterfaceAbstract; targetDir : PChar;
                            archiveRoot : PChar; next : TSalEnumSelectionFunc;
                            nextParam : Pointer ) : BOOL; virtual; stdcall; abstract;

    // funkce pro "panel archiver view", vola se pri pozadavku na rozpakovani jednoho souboru pro view/edit
    // z archivu 'fileName' do adresare 'targetDir'; jmeno souboru v archivu je 'nameInArchive';
    // 'pluginData' je interface pro praci s informacemi o souboru, ktere jsou specificke pluginu
    // (napr. data z pridanych sloupcu; jde o stejny interface, ktery vraci metoda ListArchive
    // v parametru 'pluginData' - takze muze byt i NULL); 'fileData' je ukazatel na strukturu CFileData
    // vypakovavaneho souboru (strukturu sestavil plugin pri listovani archivu); 'newFileName' (neni-li
    // NULL) je nove jmeno pro rozbalovany soubor (pouziva se pokud puvodni jmeno z archivu neni mozne
    // vybalit na disk (napr. "aux", "prn", atd.)); do 'renamingNotSupported' (jen neni-li 'newFileName'
    // NULL) zapsat TRUE pokud plugin nepodporuje prejmenovani pri vybalovani (standardni chybova hlaska
    // "renaming not supported" se zobrazi ze Salamandera); vraci TRUE pri uspesnem rozpakovani souboru
    // (soubor je na zadane ceste, nebyl pouzit Cancel ani Skip), 'salamander' je sada uzitecnych metod
    // vyvezenych ze Salamandera
    function UnpackOneFile( salamander : TSalamanderForOperationsAbstract; fileName : PChar;
                            pluginData : TPluginDataInterfaceAbstract; nameInArchive : PChar;
                            fileData : PFileData; targetDir : PChar; newFileName : PChar;
                            renamingNotSupported : PBOOL ) : BOOL; virtual; stdcall; abstract;

    // funkce pro "panel archiver edit" a "custom archiver pack", vola se pri pozadavku na zapakovani
    // souboru/adresaru do archivu 'fileName' na cestu 'archiveRoot', soubory/adresare jsou zadany
    // zdrojovou cestou 'sourcePath' a enumeracni funkci 'next' s parametrem 'nextParam',
    // je-li 'move' TRUE, zapakovane soubory/adresare by mely byt odstranene z disku, vraci TRUE
    // pokud se podari zapakovat/odstranit vsechny soubory/adresare (nebyl pouzit Cancel, mohl byt
    // pouzit Skip) - zdroj operace v panelu je odznacen, jinak vraci FALSE (neprovede se odznaceni),
    // 'salamander' je sada uzitecnych metod vyvezenych ze Salamandera
    function PackToArchive( salamander : TSalamanderForOperationsAbstract; fileName : PChar;
                            archiveRoot : PChar; move : BOOL; sourcePath : PChar;
                            next : TSalEnumSelection2Func; nextParam : Pointer ) : BOOL; virtual; stdcall; abstract;

    // funkce pro "panel archiver edit", vola se pri pozadavku na smazani souboru/adresaru z archivu
    // 'fileName'; soubory/adresare jsou zadany cestou 'archiveRoot' a enumeracni funkci 'next'
    // s parametrem 'nextParam'; 'pluginData' je interface pro praci s informacemi o souborech/adresarich,
    // ktere jsou specificke pluginu (napr. data z pridanych sloupcu; jde o stejny interface, ktery vraci
    // metoda ListArchive v parametru 'pluginData' - takze muze byt i NULL); vraci TRUE pokud se
    // podari smazat vsechny soubory/adresare (nebyl pouzit Cancel, mohl byt pouzit Skip) - zdroj
    // operace v panelu je odznacen, jinak vraci FALSE (neprovede se odznaceni); 'salamander' je sada
    // uzitecnych metod vyvezenych ze Salamandera
    function DeleteFromArchive( salamander : TSalamanderForOperationsAbstract; fileName : PChar;
                                pluginData : TPluginDataInterfaceAbstract; archiveRoot : PChar;
                                next : TSalEnumSelectionFunc; nextParam : Pointer ) : BOOL; virtual; stdcall; abstract;

    // funkce pro "custom archiver unpack", vola se pri pozadavku na rozbaleni souboru/adresaru z archivu
    // 'fileName' do adresare 'targetDir', soubory/adresare jsou zadany maskou 'mask', vraci TRUE pokud
    // se podari rozbalit vsechny soubory/adresare (nebyl pouzit Cancel, mohl byt pouzit Skip),
    // 'salamander' je sada uzitecnych metod vyvezenych ze Salamandera
    function UnpackWholeArchive( salamander : TSalamanderForOperationsAbstract; fileName : PChar;
                                 mask : PChar; targetDir : PChar ) : BOOL; virtual; stdcall; abstract;

    // funkce pro "panel archiver view/edit", vola se pred zavrenim panelu s archivem
    // POZOR: pokud se nepodari otevrit novou cestu, archiv muze v panelu zustat (nezavisle na tom,
    //        co vrati CanCloseArchive); metodu tedy nelze pouzit pro destrukci kontextu;
    //        je urcena napriklad pro optimalizaci operace Delete z archivu, kdy muze pri
    //        jeho opousteni nabidnou "setreseni" archivu
    //        pro destrukci kontextu lze pouzit metodu CPluginInterfaceAbstract::ReleasePluginDataInterface,
    //        viz dokument archivatory.txt
    // 'fileName' je jmeno archivu; 'salamander' je sada uzitecnych metod vyvezenych ze Salamandera;
    // 'panel' oznacuje panel, ve kterem je archiv otevreny (PANEL_LEFT nebo PANEL_RIGHT);
    // vraci TRUE pokud je zavreni mozne, je-li 'force' TRUE, vraci TRUE vzdy
    function CanCloseArchive( salamander : TSalamanderForOperationsAbstract; fileName : PChar;
                              force : BOOL; panel : Integer ) : BOOL; virtual; stdcall; abstract;

    // zjisti pozadovane nastaveni disk-cache (disk-cache se pouziva pro docasne kopie
    // souboru pri otevirani souboru z archivu ve viewerech, editorech a pres systemove
    // asociace); normalne (podari-li se po volani naalokovat kopii 'tempPath') se vola
    // jen jednou a to pred prvnim pouzitim disk-cache (napr. pred prvnim otevrenim
    // souboru z archivu ve vieweru/editoru); pokud vraci FALSE, pouziva
    // se standardni nastaveni (soubory v TEMP adresari, kopie se mazou pomoci Win32
    // API funkce DeleteFile() az pri prekroceni limitni velikosti cache nebo pri zavreni archivu)
    // a vsechny ostatni navratove hodnoty se ignoruji; vraci-li TRUE, pouzivaji se nasledujici
    // navratove hodnoty: neni-li 'tempPath' (buffer o velikosti MAX_PATH) prazdny retezec, budou
    // se vsechny docasne kopie vypakovane pluginem z archivu ukladat do podadresaru teto cesty
    // (tyto podadresare zrusi disk-cache pri ukonceni Salamandera, ale pluginu nic nebrani
    // v jejich smazani drive, napr. pri svem unloadu; zaroven je vhodne pri loadu prvni instance
    // pluginu (nejen v ramci jednoho spusteneho Salamandera) promazat podadresare "SAL*.tmp" na teto
    // ceste - resi problemy vznikle zamknutymi soubory a pady softu) + je-li 'ownDelete' TRUE,
    // bude se pro mazani kopii volat metoda DeleteTmpCopy a PrematureDeleteTmpCopy + je-li
    // 'cacheCopies' FALSE, budou se kopie mazat hned jakmile se uvolni (napr. jakmile se zavre
    // viewer), je-li 'cacheCopies' TRUE, budou se kopie mazat az pri prekroceni limitni velikosti
    // cache nebo pri zavreni archivu
    function GetCacheInfo( tempPath : PChar; ownDelete : PBOOL; cacheCopies : PBOOL ) : BOOL; virtual; stdcall; abstract;

    // pouziva se jen pokud metoda GetCacheInfo vraci v parametru 'ownDelete' TRUE:
    // smaze docasnou kopii vypakovanou z tohoto archivu (pozor na read-only soubory,
    // u nich je treba nejprve zmenit atributy, a pak jdou teprve smazat), pokud mozno
    // by nemelo zobrazovat zadna okna (uzivatel cinnost primo nevyvolal, muze ho rusit
    // v jine cinnosti), pri delsich akcich se hodi vyuziti wait-okenka (viz
    // CSalamanderGeneralAbstract::CreateSafeWaitWindow); 'fileName' je jmeno souboru
    // s kopii; pokud se maze vic souboru najednou (muze nastat napr. po zavreni
    // editovaneho archivu), je 'firstFile' TRUE pro prvni soubor a FALSE pro ostatni
    // soubory (pouziva se ke korektnimu zobrazeni wait-okenka - viz DEMOPLUG)
    //
    // POZOR: vola se v hl. threadu na zaklade doruceni zpravy z disk-cache hl. oknu - posila se
    // zprava o potrebe uvolnit docasnou kopii (typicky v okamziku zavreni vieweru nebo
    // "rozeditovaneho" archivu v panelu), tedy muze dojit k opakovanemu vstupu do pluginu
    // (pokud zpravu distribuuje message-loopa uvnitr pluginu), dalsi vstup do DeleteTmpCopy
    // je vylouceny, protoze do ukonceni volani DeleteTmpCopy disk-cache zadne dalsi zpravy
    // neposila
    procedure DeleteTmpCopy( fileName : PChar; firstFile : BOOL ); virtual; stdcall; abstract;

    // pouziva se jen pokud metoda GetCacheInfo vraci v parametru 'ownDelete' TRUE:
    // pri unloadu pluginu zjisti, jestli se ma volat DeleteTmpCopy pro kopie, ktere jsou
    // jeste pouzivane (napr. otevrene ve vieweru) - vola se jen pokud takove kopie
    // existuji; 'parent' je parent pripadneho messageboxu s dotazem uzivateli (pripadne
    // doporuceni, aby user zavrel vsechny soubory z archivu, aby je plugin mohl smazat);
    // 'copiesCount' je pocet pouzivanych kopii souboru z archivu; vraci TRUE pokud se
    // ma volat DeleteTmpCopy, pokud vrati FALSE, kopie zustanou na disku;
    // POZNAMKA: behem provadeni PrematureDeleteTmpCopy je zajisteno, ze nedojde
    // k volani DeleteTmpCopy
    function PrematureDeleteTmpCopy( parent : HWND; copiesCount : Integer ) : BOOL; virtual; stdcall; abstract;
  end;

implementation

end.

