(*
  Object Pascal translation of the Altap Salamander SDK library.
  Original source: spl_base.h

  Portions created by ALTAP are Copyright (c) ALTAP, spol. s r.o.
  All rights reserved.

  Portions created by LostGenius Posse are Copyright (c) 2007, LostGenius Posse (Timo 'jomanto' Boll)
  All rights reserved.

  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

      * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
      * Redistributions in binary form may reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
      * Neither the name of the LostGenius Posse nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*)
(*
  History:
  
  2007-05-29: initial version
*)
unit spl_base;

interface

uses
  winliblt,
  spl_com,
  spl_arc,
  spl_menu,
  spl_thum,
  spl_gen,
  spl_view,
  spl_fs,
  spl_gui,
  spl_xpth,
  spl_file;

// v debug verzi budeme testovat, jestli se neprekryvaji zdrojova a cilova pamet (pro memcpy se nesmi prekryvat)
{$ifdef _DEBUG}
  procedure memcpy _sal_safe_memcpy( dest : Pointer; src : Pointer; count : size_t );
{$endif} // _DEBUG

// puvodni SDK, ktere bylo soucasti VC6 melo hodnotu definovanou jako 0x00000040 (rok 1998, kdy se atribut jeste nepouzival, nastoupil az s W2K)
{$HPPEMIT '#if (FILE_ATTRIBUTE_ENCRYPTED != 0x00004000)'}
{$HPPEMIT '#pragma message(__FILE__ " PROBLEM: FILE_ATTRIBUTE_ENCRYPTED != 0x00004000. You have to install latest version of Microsoft SDK. This value has changed!")'}
{$HPPEMIT '#endif'}

const
//
// ****************************************************************************
// CSalamanderConnectAbstract
//
// sada metod Salamandera pro navazani pluginu do Salamandera
// (custom pack/unpack + panel archiver view/edit + file viewer + menu-items)

  // konstanty pro CSalamanderConnectAbstract::AddMenuItem
  MENU_EVENT_TRUE                    = $0001;                                   // nastane vzdy
  MENU_EVENT_DISK                    = $0002;                                   // zdroj je windows adresar ("c:\path" nebo UNC)
  MENU_EVENT_THIS_PLUGIN_ARCH        = $0004;                                   // zdroj je archiv tohoto pluginu
  MENU_EVENT_THIS_PLUGIN_FS          = $0008;                                   // zdroj je file-system tohoto pluginu
  MENU_EVENT_FILE_FOCUSED            = $0010;                                   // focus je na souboru
  MENU_EVENT_DIR_FOCUSED             = $0020;                                   // focus je na adresari
  MENU_EVENT_UPDIR_FOCUSED           = $0040;                                   // focus je na ".."
  MENU_EVENT_FILES_SELECTED          = $0080;                                   // jsou oznaceny soubory
  MENU_EVENT_DIRS_SELECTED           = $0100;                                   // jsou oznaceny adresare
  MENU_EVENT_TARGET_DISK             = $0200;                                   // cil je windows adresar ("c:\path" nebo UNC)
  MENU_EVENT_TARGET_THIS_PLUGIN_ARCH = $0400;                                   // cil je archiv tohoto pluginu
  MENU_EVENT_TARGET_THIS_PLUGIN_FS   = $0800;                                   // cil je file-system tohoto pluginu
  MENU_EVENT_SUBDIR                  = $1000;                                   // adresar neni root (obsahuje "..")
  // fokus je na souboru, pro ktery tento plugin zajistuje "panel archiver view" nebo "panel archiver edit"
  MENU_EVENT_ARCHIVE_FOCUSED         = $2000;
  // uz je k dispozici jen 0x4000 (masky se skladaji obe do DWORDu a predtim se maskuji 0x7FFF)

  // urcuje pro ktereho uzivatele je polozka urcena
  MENU_SKILLLEVEL_BEGINNER           = $0001;                                   // urceno pro nejdulezitejsi polozky menu, urcene pro zacatecniky
  MENU_SKILLLEVEL_INTERMEDIATE       = $0002;                                   // nastavovat i mene frekventovanym prikazum; pro pokrocilejsi uzivatele
  MENU_SKILLLEVEL_ADVANCED           = $0004;                                   // nastavovat vsem prikazum (profici by meli mit v menu vse)
  MENU_SKILLLEVEL_ALL                = $0007;                                   // pomocna konstanta slucujici vsechny predesle

//
// ****************************************************************************
// CPluginInterfaceAbstract
//
// sada metod pluginu, ktere potrebuje Salamander pro praci s pluginem
//
// Pro vetsi prehlednost jsou oddelene casti pro:
// archivatory - viz CPluginInterfaceForArchiverAbstract,
// viewry - viz CPluginInterfaceForViewerAbstract,
// rozsireni menu - viz CPluginInterfaceForMenuExtAbstract,
// file-systemy - viz CPluginInterfaceForFSAbstract,
// nacitace thumbnailu - viz CPluginInterfaceForThumbLoaderAbstract.
// Casti jsou pripojeny k CPluginInterfaceAbstract pres CPluginInterfaceAbstract::GetInterfaceForXXX

  // kody ruznych udalosti (a vyznam parametru 'param'), prijima metoda CPluginInterfaceAbstract::Event():
  // doslo ke zmene barev (diky zmene systemovych barev nebo diky zmene v konfiguraci); plugin si
  // muze vytahnout nove verze salamanderovskych barev pres CSalamanderGeneralAbstract::GetCurrentColor;
  // pokud ma plugin file-system s ikonami typu pitFromPlugin, mel by prebarvit pozadi image-listu
  // s jednoduchymi ikonami na barvu SALCOL_ITEM_BK_NORMAL; 'param' se zde ignoruje
  PLUGINEVENT_COLORSCHANGED        = 0;

  // doslo ke zmene konfigurace Salamandera; plugin si muze vytahnout nove verze salamanderovskych
  // konfiguracnich parametru pres CSalamanderGeneralAbstract::GetConfigParameter;
  // 'param' se zde ignoruje
  PLUGINEVENT_CONFIGURATIONCHANGED = 1;

  // doslo k prohozeni leveho a praveho panelu (Swap Panels - Ctrl+U)
  // 'param' se zde ignoruje
  PLUGINEVENT_PANELSSWAPPED        = 2;

  // doslo ke zmene aktivniho panelu (prepnuti mezi panely)
  // 'param' je PANEL_LEFT nebo PANEL_RIGHT - oznacuje aktivovany panel
  PLUGINEVENT_PANELACTIVATED       = 3;

//
// ****************************************************************************
// CSalamanderPluginEntryAbstract
//
// sada metod ze Salamandera, ktere se pouzivaji v SalamanderPluginEntry

  // flagy oznacujici, ktere funkce plugin poskytuje - jake metody CPluginInterface jsou definovany):
  FUNCTION_PANELARCHIVERVIEW     = $0001;                                       // metody pro "panel archiver view"
  FUNCTION_PANELARCHIVEREDIT     = $0002;                                       // metody pro "panel archiver edit"
  FUNCTION_CUSTOMARCHIVERPACK    = $0004;                                       // metody pro "custom archiver pack"
  FUNCTION_CUSTOMARCHIVERUNPACK  = $0008;                                       // metody pro "custom archiver unpack"
  FUNCTION_CONFIGURATION         = $0010;                                       // metoda Configuration
  FUNCTION_LOADSAVECONFIGURATION = $0020;                                       // metody pro "load/save configuration"
  FUNCTION_VIEWER                = $0040;                                       // metody pro "file viewer"
  FUNCTION_FILESYSTEM            = $0080;                                       // metody pro "file system"

  // flagy informujici o duvodu loadu pluginu (viz metoda CSalamanderPluginEntryAbstract::GetLoadInformation)
  LOADINFO_INSTALL               = $0001;                                       // prvni load pluginu (instalace do Salamandera)
  LOADINFO_NEWSALAMANDERVER      = $0002;                                       // nova verze Salamandera (doinstalace vsech pluginu z
                                                                                // podadresare plugins), loadi vsechny pluginy (mozny
                                                                                // upgrade vsech)
  LOADINFO_NEWPLUGINSVER         = $0004;                                       // zmena v souboru plugins.ver (instalace/upgrade pluginu),
                                                                                // pro jednoduchost loadi vsechny pluginy (mozny upgrade
                                                                                // vsech)
  LOADINFO_LOADONSTART           = $0008;                                       // load probehl, protoze byl nalezen flag "load on start"

// makro pro pripravu 'hotKey' pro AddMenuItem()
// rika Salamanderu, ze polozky menu bude obsahovat horkou klavesu (oddelenou znakem '\t')
// Salamander nebude v tomto pripade kricet pomoci TRACE_E a horkou klavesu zobrazi v menu Plugins
// POZOR: nejedna se o horkou klavesu, kterou by Salamander dorucil pluginu, jde skutecne pouze o napis
// pokud uzivatel priradi v Plugin Manageru tomuto commandu vlastni horkou klavesu, bude hint potlacen
  SALHOTKEY_HINT = $00020000;

type
//
// ****************************************************************************
// CSalamanderDebugAbstract
//
// sada metod ze Salamandera, ktere se pouzivaji pro hledani chyb v debug i release verzi

// makro CALLSTK_MEASURETIMES - zapne mereni casu straveneho pri priprave call-stack hlaseni (meri se pomer proti
//                              celkovemu casu behu funkci)
//                              POZOR: nutne zapnout tez pro kazdy plugin zvlast
// makro CALLSTK_DISABLEMEASURETIMES - potlaci mereni casu straveneho pri priprave call-stack hlaseni v DEBUG verzi

{$if (defined(_DEBUG) or defined(CALLSTK_MEASURETIMES)) and not defined(CALLSTK_DISABLEMEASURETIMES) }
  TCallStackMsgContext = packed record
    PushesCounterStart              : DWORD;                                    // start-stav pocitadla Pushu volanych v tomto threadu
    PushPerfTimeCounterStart        : LARGE_INTEGER;                            // start-stav pocitadla casu straveneho v metodach Push volanych v tomto threadu
    IgnoredPushPerfTimeCounterStart : LARGE_INTEGER;                            // start-stav pocitadla casu straveneho v nemerenych (ignorovanych) metodach Push volanych v tomto threadu
    StartTime                       : LARGE_INTEGER;                            // "cas" Pushe tohoto call-stack makra
    PushCallerAddress               : DWORD;                                    // adresa makra CALL_STACK_MESSAGE (adresa Pushnuti)
  end;
{$else} // (defined(_DEBUG) or defined(CALLSTK_MEASURETIMES)) and not defined(CALLSTK_DISABLEMEASURETIMES)
  TCallStackMsgContext = packed record
  end;
{$ifend} // (defined(_DEBUG) or defined(CALLSTK_MEASURETIMES)) and not defined(CALLSTK_DISABLEMEASURETIMES)
  PCallStackMsgContext = ^TCallStackMsgContext;

  TCallStackFunc = function( param : Pointer ) : Cardinal; stdcall;

  TSalamanderDebugAbstract = class
    // vypise 'file'+'line'+'str' TRACE_I na TRACE SERVER - pouze pri DEBUG verzi Salamandera
    procedure TraceI( _file : PChar; line : Integer; str : PChar ); virtual; stdcall; abstract;

    // vypise 'file'+'line'+'str' TRACE_E na TRACE SERVER - pouze pri DEBUG verzi Salamandera
    procedure TraceE( _file : PChar; line : Integer; str : PChar ); virtual; stdcall; abstract;

    // zaregistruje novy thread u TRACE (prideli Unique ID), 'thread'+'tid' vraci
    // _beginthreadex i CreateThread, nepovine (UID je pak -1)
    procedure TraceAttachThread( thread : THandle; tid : Cardinal ); virtual; stdcall; abstract;

    // nastavi jmeno aktivniho threadu pro TRACE, nepovine (thread je oznacen jako "unknown")
    procedure TraceSetThreadName( name : PChar ); virtual; stdcall; abstract;

    // zavede do threadu veci potrebne pro CALL-STACK metody (viz Push a Pop nize),
    // ve vsech volanych metodach pluginu je mozne CALL_STACK metody pouzit primo,
    // tato metoda se pouziva pouze pro nove thready pluginu,
    // spousti funkci 'threadBody' s parametrem 'param', vraci vysledek funkce 'threadBody'
    function CallWithCallStack( threadBody : TCallStackFunc; param : Pointer ) : Cardinal; virtual; stdcall; abstract;

    // uklada na CALL-STACK zpravu ('format'+'args' viz vsprintf), pri padu aplikace je
    // obsah CALL-STACKU vypsan do okna Bug Report ohlasujiciho pad aplikace
    procedure Push( format : PChar; args : va_list; callStackMsgContext : PCallStackMsgContext;
                    doNotMeasureTimes : BOOL ); virtual; stdcall; abstract;

    // odstranuje z CALL-STACKU posledni zpravu, volani musi parovat s Push
    procedure Pop( callStackMsgContext : PCallStackMsgContext ); virtual; stdcall; abstract;
  end;

//
// ****************************************************************************
// CSalamanderConnectAbstract
//
// sada metod Salamandera pro navazani pluginu do Salamandera
// (custom pack/unpack + panel archiver view/edit + file viewer + menu-items)

  TSalamanderConnectAbstract = class
    // pridani pluginu do seznamu pro "custom archiver pack",
    // 'title' je nazev custom packeru pro uzivatele, 'defaultExtension' je standardni pripona
    // pro nove archivy, pokud nejde o upgrade "custom pack" (nebo pridani celeho pluginu) je
    // volani ignorovano, vyjimkou je zmena (ne pridani) 'title' a 'defaultExtension'
    // pokud je 'update' TRUE - nutna prevence proti opakovanemu TRUE (neustalemu obnovovani)
    procedure AddCustomPacker( title : PChar; defaultExtension : PChar; update : BOOL ); virtual; stdcall; abstract;

    // pridani pluginu do seznamu pro "custom archiver unpack",
    // 'title' je nazev custom unpackeru pro uzivatele, 'masks' jsou masky souboru archivu (hleda
    // se podle nich cim rozpakovavat dany archiv, oddelovac je ';' (escape sekvence pro ';' je
    // ";;") a pouzivaji se klasicky wildcards '*' a '?' plus '#' pro '0'..'9'), pokud nejde o upgrade
    // "custom unpack" (nebo pridani celeho pluginu) je volani ignorovano, vyjimkou je zmena
    // (ne pridani) 'title' a 'masks' pokud je 'update' TRUE - nutna prevence proti opakovanemu
    // TRUE (neustalemu obnovovani)
    procedure AddCustomUnpacker( title : PChar; masks : PChar; update : BOOL ); virtual; stdcall; abstract;

    // pridani pluginu do seznamu pro "panel archiver view/edit",
    // 'extensions' jsou pripony archivu, ktere se timto pluginem maji zpracovavat
    // (oddelovac je ';' (zde nema ';' zadnou escape sekvenci) a pouziva se wildcard '#' pro
    // '0'..'9'), pokud je 'edit' TRUE, resi tento plugin "panel archiver view/edit", jinak jen
    // "panel archiver view", pokud nejde o upgrade "panel archiver view/edit" (nebo pridani
    // celeho pluginu) je volani ignorovano, vyjimkou je pridani novych pripon archivu pokud
    // 'updateExts' je TRUE (zajisteni pritomnosti vsech pripon z 'extensions') - nutna
    // prevence proti opakovanemu TRUE (neustalemu ozivovani pripon z 'extensions')
    procedure AddPanelArchiver( extensions : PChar; edit : BOOL; updateExts : BOOL ); virtual; stdcall; abstract;

    // pridani pluginu do seznamu pro "file viewer",
    // 'masks' jsou pripony vieweru, ktere se timto pluginem maji zpracovavat
    // (oddelovac je ';' (escape sekvence pro ';' je ";;") a pouzivaji se wildcardy '*' a '?',
    // pokud mozno nepouzivat mezery + znak '|' je zakazany (inverzni masky nejsou povolene)),
    // pokud nejde o upgrade "file viewer" (nebo pridani celeho pluginu) je volani ignorovano,
    // vyjimkou je volani s 'force' == TRUE - prida 'masks' (pokud jiz nejsou na seznamu) - nutna
    // prevence proti opakovanemu TRUE (neustalemu pridavani 'masks')
    procedure AddViewer( masks : PChar; force : BOOL ); virtual; stdcall; abstract;

    // vyhozeni masky ze seznamu pro "file viewer" (jen z polozek tykajicich se tohoto pluginu),
    // 'mask' je pripona viewru (jedina; pouzivaji se wildcardy '*' a '?'), nutna prevence
    // proti opakovanemu volani (neustalemu mazani 'mask')
    procedure ForceRemoveViewer( mask : PChar ); virtual; stdcall; abstract;

    // pridava polozky do menu Plugins/"jmeno pluginu" v Salamanderu, 'iconIndex' je index
    // ikony polozky (-1=zadna ikona; zadani bitmapy s ikonami viz
    // CPluginInterfaceAbstract::SetBitmapWithIcons; u separatoru se ignoruje), 'name' je
    // jmeno polozky (max. MAX_PATH - 1 znaku) nebo NULL jde-li o separator (parametry
    // 'state_or'+'state_and' v tomto pripade nemaji vyznam), 'hotKey' je horka klavesa polozky
    // ziskana pomoci makra SALHOTKEY; 'id' je unikatni
    //
    // 'name' muze obsahovat hint pro horkou klavesu, oddeleny znakem '\t'; 
    //  v promenne 'hotKey' musi byt v tomto pripade prirazena konstanta SALHOTKEY_HINT, 
    //  vice viz komentar k SALHOTKEY_HINT
    //
    // identifikacni cislo polozky v ramci pluginu (u separatoru ma vyznam jen je-li
    // 'callGetState' TRUE), pokud je 'callGetState' TRUE, vola se pro zjisteni stavu polozky
    // metoda CPluginInterfaceAbstract::GetMenuItemState (u separatoru ma vyznam jen stav
    // MENU_ITEM_STATE_HIDDEN, ostatni se ignoruji), jinak se k vypoctu stavu polozky (enabled/disabled)
    // pouziji 'state_or'+'state_and' - pri vypoctu stavu polozky se nejprve sestavi maska
    // ('eventMask') tak, ze se logicky sectou vsechny udalosti, ktere nastaly (udalosti viz
    // MENU_EVENT_XXX), polozka bude "enable" pokud bude nasl. vyraz TRUE:
    //   ('eventMask' & 'state_or') != 0 && ('eventMask' & 'state_and') == 'state_and',
    // parametr 'skillLevel' urcuje kterym urovnim uzivatele bude polozka (nebo separator)
    // zobrazena; hodnota obsahuje jednu nebo vice (ORovane) konstant MENU_SKILLLEVEL_XXX;
    // polozky v menu se updatuji pri kazdem loadu pluginu (mozna zmena polozek dle konfigurace)
    procedure AddMenuItem( iconIndex : Integer; name : PChar; hotKey : DWORD; id : Integer;
                           callGetState : BOOL; state_or : DWORD; state_and : DWORD;
                           skillLevel : DWORD ); virtual; stdcall; abstract;

    // prida submenu do menu Plugins/"jmeno pluginu" v Salamanderu, 'iconIndex'
    // je index ikony submenu (-1=zadna ikona; zadani bitmapy s ikonami
    // viz CPluginInterfaceAbstract::SetBitmapWithIcons), 'name' je jmeno
    // submenu (max. MAX_PATH - 1 znaku), 'id' je unikatni identifikacni cislo polozky
    // menu v ramci pluginu (u submenu ma vyznam jen je-li 'callGetState' TRUE),
    // pokud je 'callGetState' TRUE, vola se pro zjisteni stavu submenu metoda
    // CPluginInterfaceAbstract::GetMenuItemState (vyznam maji jen stavy
    // MENU_ITEM_STATE_ENABLED a MENU_ITEM_STATE_HIDDEN, ostatni se ignoruji), jinak se
    // k vypoctu stavu polozky (enabled/disabled) pouziji 'state_or'+'state_and' - vypocet
    // stavu polozky viz CSalamanderConnectAbstract::AddMenuItem(), parametr 'skillLevel'
    // urcuje kterym urovnim uzivatele bude submenu zobrazeno, hodnota obsahuje jednu nebo
    // vice (ORovanych) konstant MENU_SKILLLEVEL_XXX, submenu se ukoncuje volanim
    // CSalamanderConnectAbstract::AddSubmenuEnd();
    // polozky v menu se updatuji pri kazdem loadu pluginu (mozna zmena polozek dle konfigurace)
    procedure AddSubmenuStart( iconIndex : Integer; name : PChar; id : Integer;
                               callGetState : BOOL; state_or : DWORD; state_and : DWORD;
                               skillLevel : DWORD ); virtual; stdcall; abstract;

    // ukonci submenu v menu Plugins/"jmeno pluginu" v Salamanderu, dalsi polozky budou
    // pridavany do vyssi (rodicovske) urovne menu;
    // polozky v menu se updatuji pri kazdem loadu pluginu (mozna zmena polozek dle konfigurace)
    procedure AddSubmenuEnd; virtual; stdcall; abstract;

    // nastavuje polozku pro FS v Change Drive menu; 'title' je jeji text, 'iconIndex'
    // je index jeji ikony (-1=zadna ikona; zadani bitmapy s ikonami viz
    // CPluginInterfaceAbstract::SetBitmapWithIcons), 'title' muze obsahovat az tri sloupce
    // vzajemne oddelene '\t' (viz Alt+F1/F2 menu); viditelnost polozky je mozne nastavit
    // z Plugins Manageru nebo primo z pluginu pomoci metody
    // CSalamanderGeneralAbstract::SetChangeDriveMenuItemVisibility
    procedure SetChangeDriveMenuItem( title : PChar; iconIndex : Integer ); virtual; stdcall; abstract;

    // informuje Salamandera, ze plugin umi nacitat thumbnaily ze souboru odpovidajicich
    // skupinove masce 'masks' (oddelovac je ';' (escape sekvence pro ';' je ";;") a pouzivaji
    // se wildcardy '*' a '?'); pro nacteni thumbnailu se vola CPluginInterfaceAbstract::LoadThumbnail
    procedure SetThumbnailLoader( masks : PChar ); virtual; stdcall; abstract;

    // nastavi bitmapu s ikonami pluginu; Salamander si obsah bitmapy kopiruje do internich
    // struktur, plugin je zodpovedny za destrukci bitmapy (ze strany Salamanadera se
    // bitmapa pouzije pouze behem teto funkce); pocet ikon se odvozuje ze
    // sirky bitmapy, ikony jsou vzdy 16x16 bodu; transparentni cast ikon tvori fialova
    // barva (RGB(255,0,255)), barevna hloubka bitmapy muze byt 4 nebo 8 bitu (16 nebo 256
    // barev), idealni je mit pripravene obe barevne varianty a vybirat z nich podle
    // vysledku metody CSalamanderGeneralAbstract::CanUse256ColorsBitmap()
    procedure SetBitmapWithIcons( bitmap : HBITMAP ); virtual; stdcall; abstract;

    // nastavi index ikony pluginu, ktera se pro plugin pouziva v okne Plugins/Plugins,
    // v menu Help/About Plugin a pripadne i pro submenu pluginu v menu Plugins (detaily
    // viz CSalamanderConnectAbstract::SetPluginMenuAndToolbarIcon()); pokud plugin tuto
    // metodu nezavola, pouzije se standardni Salamanderovska ikona pro plugin; 'iconIndex'
    // je nastavovany index ikony (zadani bitmapy s ikonami viz
    // CPluginInterfaceAbstract::SetBitmapWithIcons)
    procedure SetPluginIcon( iconIndex : Integer ); virtual; stdcall; abstract;

    // nastavi index ikony pro submenu pluginu, ktera se pouziva pro submenu pluginu
    // v menu Plugins a pripadne i v horni toolbare pro drop-down button slouzici
    // k zobrazeni submenu pluginu; pokud plugin tuto metodu nezavola, pouzije se
    // pro submenu pluginu v menu Plugins ikona pluginu (nastaveni viz
    // CSalamanderConnectAbstract::SetPluginIcon) a v horni toolbare se neobjevi
    // button pro plugin; 'iconIndex' je nastavovany index ikony (-1=ma se pouzit ikona
    // pluginu, viz CSalamanderConnectAbstract::SetPluginIcon(); zadani bitmapy
    // s ikonami viz CPluginInterfaceAbstract::SetBitmapWithIcons);
    procedure SetPluginMenuAndToolbarIcon( iconIndex : Integer ); virtual; stdcall; abstract;
  end;

//
// ****************************************************************************
// CPluginInterfaceAbstract
//
// sada metod pluginu, ktere potrebuje Salamander pro praci s pluginem
//
// Pro vetsi prehlednost jsou oddelene casti pro:
// archivatory - viz CPluginInterfaceForArchiverAbstract,
// viewry - viz CPluginInterfaceForViewerAbstract,
// rozsireni menu - viz CPluginInterfaceForMenuExtAbstract,
// file-systemy - viz CPluginInterfaceForFSAbstract,
// nacitace thumbnailu - viz CPluginInterfaceForThumbLoaderAbstract.
// Casti jsou pripojeny k CPluginInterfaceAbstract pres CPluginInterfaceAbstract::GetInterfaceForXXX

  TPluginInterfaceAbstract = class
    // vraci verzi Salamandera, pro kterou je plugin staven:
    // viz spl_vers.h, konstanty LAST_VERSION_OF_SALAMANDER a REQUIRE_LAST_VERSION_OF_SALAMANDER
    // POZOR: tuto metodu od verze 2.5 beta 2 Salamandera nahradila exportovana funkce
    //        "SalamanderPluginGetReqVer"; tuto metodu muzou zavolat jen starsi verze
    //        Salamandera (pokud nedojde k padu behem provadeni entry pointu pluginu)
    function GetVersion : Integer; virtual; stdcall; abstract;

    // vola se jako reakce na tlacitko About v okne Plugins nebo prikaz z menu Help/About Plugins
    procedure About( parent : HWND ); virtual; stdcall; abstract;

    // vola se pred unloadem pluginu (prirozene jen pokud vratil SalamanderPluginEntry
    // tento objekt a ne NULL), vraci TRUE pokud unload muze probehnout,
    // 'parent' je parent messageboxu, 'force' je TRUE pokud se nebere ohled na navratovou
    // hodnotu, pokud vraci TRUE, nebude se tento objekt a vsechny ostatni z nej ziskane
    // dale pouzivat a doje k unloadu pluginu
    // POZOR!!! Je nutne ukoncit vsechny thready pluginu (pokud Release vrati TRUE, vola se
    // FreeLibrary na pluginove .SPL => kod pluginu se odmapuje z pameti => thready pak uz
    // nemaji co spoustet => obvykle nevypadne ani bug-report, ani Windows exception info)
    function Release( parent : HWND; force : BOOL ) : BOOL; virtual; stdcall; abstract;

    // funkce pro load defaultni konfigurace a pro "load/save configuration" (load ze soukromeho klice
    // pluginu v registry), 'parent' je parent messageboxu, je-li 'regKey' == NULL, jde o
    // defaultni konfiguraci, 'registry' je objekt pro praci s registry, tato metoda se vola vzdy
    // po SalamanderPluginEntry a pred ostatnimi volanimi (vola se load ze soukromeho klice, je-li
    // tato funkce pluginem poskytovana a klic v registry existuje, jinak vola jen load defaultni
    // konfigurace)
    procedure LoadConfiguration( parent : HWND; regKey : HKEY;
                                 registry : TSalamanderRegistryAbstract ); virtual; stdcall; abstract;

    // funkce pro "load/save configuration", vola se pro ulozeni konfigurace pluginu do jeho soukromeho
    // klice v registry, 'parent' je parent messageboxu, 'registry' je objekt pro praci s registry,
    // uklada-li Salamander konfiguraci, vola take tuto metodu (je-li pluginem poskytovana)
    procedure SaveConfiguration( parent : HWND; regKey : HKEY;
                                 registry : TSalamanderRegistryAbstract ); virtual; stdcall; abstract;

    // vola se jako reakce na tlacitko Configurate v okne Plugins
    procedure Configuration( parent : HWND ); virtual; stdcall; abstract;

    // vola se pro navazani pluginu do Salamandera, vola se az po LoadConfiguration,
    // 'parent' je parent messageboxu, 'salamander' je sada metod pro navazovani pluginu

  (*  PRAVIDLA PRO IMPLEMENTACI METODY CONNECT
        (pluginy musi mit ulozenou verzi konfigurace - viz DEMOPLUGin,
         promenna ConfigVersion a konstanta CURRENT_CONFIG_VERSION; nize je
         nazorny PRIKLAD pridani pripony "DMP2" do DEMOPLUGinu):

      -s kazdou zmenou je potreba zvysit cislo verze konfigurace - CURRENT_CONFIG_VERSION
       (v prvni verzi metody Connect je CURRENT_CONFIG_VERSION=1)
      -do zakladni casti (pred podminky "if (ConfigVersion < YYY)"):
        -se napise kod pro instalaci pluginu (uplne prvni load pluginu):
         viz metody CSalamanderConnectAbstract
        -pri upgradech je nutne doplnit seznamy pripon pro "custom archiver
         unpack" (AddCustomUnpacker), "panel archiver view/edit" (AddPanelArchiver),
         "file viewer" (AddViewer), polozky menu (AddMenuItem), atd.
        -u volani AddCustomUnpacker dame do parametru 'update' podminku
         "ConfigVersion < XXX", kde XXX je cislo posledni verze, kde se menily
         pripony pro custom unpackery
        -AddMenuItem, SetChangeDriveMenuItem a SetThumbnailLoader funguje pri kazdem loadu
         pluginu stejne (instalace/upgrady se nelisi - vzdy se zacina na zelene louce)
      -jen pri upgradech: do casti pro upgrady (za zakladni casti):
        -pridame podminku "if (ConfigVersion < XXX)", kde XXX je nova hodnota
         konstanty CURRENT_CONFIG_VERSION + pridame komentar od teto verze;
         v tele teto podminky zavolame:
          -pokud pribyly pripony pro "panel archiver", zavolame
           "AddPanelArchiver(PPP, EEE, TRUE)", kde PPP jsou jen nove pripony oddelene
           strednikem a EEE je TRUE/FALSE ("panel view+edit"/"jen panel view")
          -pokud pribyly pripony pro "viewer", zavolame "AddViewer(PPP, TRUE)",
           kde PPP jsou jen nove pripony oddelene strednikem
          -pokud se maji smazat nejake stare pripony pro "viewer", zavolame
           pro kazdou takovou priponu PPP "ForceRemoveViewer(PPP)"
          -pokud se maji smazat nejake pripony pro "panel archiver", dejte vedet
           Petrovi, zatim to nikdo nepotreboval, takze to neni implementovane

      KONTROLA: po techto upravach doporucuji vyzkouset jestli to funguje jak ma,
                staci nakompilovat plugin a zkusit ho naloadit do Salama, melo by
                dojit k automatickemu upgradu z predesle verze (bez potreby
                vyhozeni a pridani pluginu):
                -viz menu Options/Configuration:
                  -Viewery jsou na strance Viewers: najdete pridane pripony,
                   zkontrolujte, ze odebrane pripony jiz neexistuji
                  -Panel Archivers jsou na strance Archives Associations in Panels:
                   najdete pridane pripony
                  -Custom Unpackers jsou na strance Unackers in Unpack Dialog Box:
                   najdete vas plugin a zkontrolujte, jestli je seznam masek OK
                -zkontrolovat novou podobu submenu pluginu (v menu Plugins)
                -zkontrolovat novou podobu Change Drive menu (Alt+F1/F2)
                -zkontrolovat v Plugins Manageru (v menu Plugins) masky thumbnaileru:
                 fokusnout vas plugin, pak zkontrolovat editbox "Thumbnails"
              +nakonec muzete jeste taky zkusit vyhodit a pridat plugin, jestli
               funguje "instalace" pluginu: kontrola viz vsechny predesle body

      POZNAMKA: pri pridani pripon pro "panel archiver" je tez potreba doplnit
                seznam pripon v parametru 'extensions' metody SetBasicPluginData2

      PRIKLAD PRIDANI PRIPONY "DMP2" VIEWERU A ARCHIVERU:
        (radky zacinajici na "-" byly odstraneny, radky zacinajici na "+" pridany,
         symbol "=====" na zacatku radky znaci preruseni souvisleho useku kodu)
        Prehled zmen:
          -zvysila se verze konfigurace z 2 na 3:
            -pridany komentar k verzi 3
            -zvyseni CURRENT_CONFIG_VERSION na 3
          -pridani pripony "DMP2" do parametru 'extensions' SetBasicPluginData2
           (protoze pridavame priponu "DMP2" pro "panel archiver")
          -pridani masky "*.DMP2" do AddCustomUnpacker + zvyseni verze z 1 na 3
           v podmince (protoze pridavame priponu "DMP2" pro "custom unpacker")
          -pridani pripony "DMP2" do AddPanelArchiver (protoze pridavame priponu
           "DMP2" pro "panel archiver")
          -pridani masky "*.DMP2" do AddViewer (protoze pridavame priponu "DMP2"
           pro "viewer")
          -pridani podminky pro upgrade na verzi 3 + komentar tohoto upgradu,
           telo podminky:
            -volani AddPanelArchiver pro priponu "DMP2" s 'updateExts' TRUE
             (protoze pridavame priponu "DMP2" pro "panel archiver")
            -volani AddViewer pro masku "*.DMP2" s 'force' TRUE (protoze
             pridavame priponu "DMP2" pro "viewer")
=====
  // ConfigVersion: 0 - zadna konfigurace se z Registry nenacetla (jde o instalaci pluginu),
  //                1 - prvni verze konfigurace
  //                2 - druha verze konfigurace (pridane nejake hodnoty do konfigurace)
+ //                3 - treti verze konfigurace (pridani pripony "DMP2")

  int ConfigVersion = 0;
- #define CURRENT_CONFIG_VERSION 2
+ #define CURRENT_CONFIG_VERSION 3
  const char *CONFIG_VERSION = "Version";
=====
  // nastavime zakladni informace o pluginu
  salamander->SetBasicPluginData2("Salamander Demo Plugin",
                                  FUNCTION_PANELARCHIVERVIEW | FUNCTION_PANELARCHIVEREDIT |
                                  FUNCTION_CUSTOMARCHIVERPACK | FUNCTION_CUSTOMARCHIVERUNPACK |
                                  FUNCTION_CONFIGURATION | FUNCTION_LOADSAVECONFIGURATION |
                                  FUNCTION_VIEWER | FUNCTION_FILESYSTEM,
                                  "2.0",
                                  "Copyright  1999-2006 ALTAP",
                                  "This plugin should help you to make your own plugins.",
-                                 "DEMOPLUG", "DMP", "DFS");
+                                 "DEMOPLUG", "DMP;DMP2", "DFS");
=====
  void WINAPI
  CPluginInterface::Connect(HWND parent, CSalamanderConnectAbstract *salamander)
  {
    CALL_STACK_MESSAGE1("CPluginInterface::Connect(,)");

    // zakladni cast:
    salamander->AddCustomPacker("DEMOPLUG (Plugin)", "DMP", FALSE);
-   salamander->AddCustomUnpacker("DEMOPLUG (Plugin)", "*.DMP", ConfigVersion < 1);
+   salamander->AddCustomUnpacker("DEMOPLUG (Plugin)", "*.DMP;*.DMP2", ConfigVersion < 3);
-   salamander->AddPanelArchiver("DMP", TRUE, FALSE);
+   salamander->AddPanelArchiver("DMP;DMP2", TRUE, FALSE);
-   salamander->AddViewer("*.DMP", FALSE);
+   salamander->AddViewer("*.DMP;*.DMP2", FALSE);
===== (vynechal jsem pridavani polozek do menu, nastavovani ikon a masek thumbnailu)
    // cast pro upgrady:
+   if (ConfigVersion < 3)   // verze 3: pridani pripony "DMP2"
+   {
+     salamander->AddPanelArchiver("DMP2", TRUE, TRUE);
+     salamander->AddViewer("*.DMP2", TRUE);
+   }
  }
=====
    *)
    procedure Connect( parent : HWND; salamander : TSalamanderConnectAbstract ); virtual; stdcall; abstract;

    // uvolni interface 'pluginData', ktery Salamander ziskal z pluginu pomoci volani
    // CPluginInterfaceForArchiverAbstract::ListArchive nebo
    // CPluginFSInterfaceAbstract::ListCurrentPath; pred timto volanim jeste
    // dojde k uvolneni dat souboru a adresaru (CFileData::PluginData) pomoci metod
    // CPluginDataInterfaceAbstract
    procedure ReleasePluginDataInterface( pluginData : TPluginDataInterfaceAbstract ); virtual; stdcall; abstract;

    // vraci interface archivatoru; plugin musi vracet tento interface, pokud ma
    // alespon jednu z nasledujicich funkci (viz SetBasicPluginData): FUNCTION_PANELARCHIVERVIEW,
    // FUNCTION_PANELARCHIVEREDIT, FUNCTION_CUSTOMARCHIVERPACK a/nebo FUNCTION_CUSTOMARCHIVERUNPACK;
    // pokud plugin archivator neobsahuje, vraci NULL
    function GetInterfaceForArchiver : TPluginInterfaceForArchiverAbstract; virtual; stdcall; abstract;

    // vraci interface vieweru; plugin musi vracet tento interface, pokud ma funkci
    // (viz SetBasicPluginData) FUNCTION_VIEWER; pokud plugin viewer neobsahuje, vraci NULL
    function GetInterfaceForViewer : TPluginInterfaceForViewerAbstract; virtual; stdcall; abstract;

    // vraci interface rozsireni menu; plugin musi vracet tento interface, pokud pridava
    // polozky do menu (viz CSalamanderConnectAbstract::AddMenuItem); pokud plugin
    // nepridava zadne polozky do menu, vraci NULL
    function GetInterfaceForMenuExt : TPluginInterfaceForMenuExtAbstract; virtual; stdcall; abstract;

    // vraci interface file-systemu; plugin musi vracet tento interface, pokud ma funkci
    // (viz SetBasicPluginData) FUNCTION_FILESYSTEM; pokud plugin file-system neobsahuje, vraci NULL
    function GetInterfaceForFS : TPluginInterfaceForFSAbstract; virtual; stdcall; abstract;

    // vraci interface nacitace thumbnailu; plugin musi vracet tento interface, pokud informoval
    // Salamandera, ze umi nacitat thumbnaily (viz CSalamanderConnectAbstract::SetThumbnailLoader);
    // pokud plugin neumi nacitat thumbnaily, vraci NULL
    function GetInterfaceForThumbLoader : TPluginInterfaceForThumbLoaderAbstract; virtual; stdcall; abstract;

    // prijem ruznych udalosti, viz kody udalosti PLUGINEVENT_XXX; vola se jen pokud je plugin
    // nacteny; 'param' je parametr udalosti
    // POZOR: muze se zavolat kdykoliv po dokonceni entry-pointu pluginu (SalamanderPluginEntry)
    procedure Event( event : Integer; param : DWORD ); virtual; stdcall; abstract;

    // uzivatel si preje, aby byly smazany vsechny historie (spustil Clear History z konfigurace
    // ze stranky History); historii se tu mysli vse, co vznika samocinne z uzivatelem zadanych
    // hodnot (napr. seznam textu spustenych v prikazove radce, seznam aktualnich cest na
    // jednotlivych drivech, atd.); nepatri sem seznamy tvorene userem - napr. hot-paths, user-menu,
    // atd.; 'parent' je parent pripadnych messageboxu; po ulozeni konfigurace nesmi historie
    // zustat v registry; pokud ma plugin otevrena okna obsahujici historie (comboboxy), musi
    // promazat historie take tam
    procedure ClearHistory( parent : HWND ); virtual; stdcall; abstract;

    // prijem informace o zmene na ceste 'path' (je-li 'includingSubdirs' TRUE, tak
    // zahrnuje i zmenu v podadresarich cesty 'path'); teto metody je mozne vyuzit napr.
    // k invalidovani/cisteni cache souboru/adresaru; POZNAMKA: pro pluginove file-systemy (FS)
    // existuje metoda CPluginFSInterfaceAbstract::AcceptChangeOnPathNotification()
    procedure AcceptChangeOnPathNotification( path : PChar; includingSubdirs : BOOL ); virtual; stdcall; abstract;
  end;

//
// ****************************************************************************
// CSalamanderPluginEntryAbstract
//
// sada metod ze Salamandera, ktere se pouzivaji v SalamanderPluginEntry

  TSalamanderPluginEntryAbstract = class
    // vraci verzi Salamandera, viz spl_vers.h, konstanty LAST_VERSION_OF_SALAMANDER a REQUIRE_LAST_VERSION_OF_SALAMANDER
    function GetVersion : Integer; virtual; stdcall; abstract;

    // vraci "parent" okno Salamandera (parent pro messageboxy)
    function GetParentWindow : HWND; virtual; stdcall; abstract;

    // JEN PRO USPESNOU DETEKCI VERZE PLUGINU, AKTUALNI VERZE JE SetBasicPluginData2
    function SetBasicPluginData( pluginName           : PChar;
                                 supportPanelView     : BOOL;
                                 supportPanelEdit     : BOOL;
                                 supportCustomPack    : BOOL;
                                 supportCustomUnpack  : BOOL;
                                 supportConfiguration : BOOL;
                                 supportLoadSave      : BOOL;
                                 supportViewer        : BOOL;
                                 supportFS            : BOOL;
                                 version              : PChar;
                                 copyright            : PChar;
                                 description          : PChar;
                                 regKeyName           : PChar;
                                 extensions           : PChar;
                                 fsName               : PChar ) : BOOL; virtual; stdcall; abstract;

    // vraci ukazatel na interface k debugovacim funkcim Salamandera,
    // interface je platny po celou dobu existence pluginu (nejen v ramci
    // "SalamanderPluginEntry" funkce) a jde pouze o odkaz, takze se neuvolnuje
    function GetSalamanderDebug : TSalamanderDebugAbstract; virtual; stdcall; abstract;

    // nastaveni zakladnich dat o pluginu (data, ktera si o pluginu spolu se jmenem DLL souboru
    // Salamander pamatuje), nutne volat, jinak nemuze byt plugin pripojen;
    // 'pluginName' je jmeno pluginu; 'functions' obsahuje naORovane vsechny funkce, ktere plugin
    // podporuje (viz konstanty FUNCTION_XXX); 'version'+'copyright'+'description' jsou data pro
    // uzivatele zobrazovana v okne Plugins; 'regKeyName' je navrhovany nazev soukromeho klice
    // pro ulozeni konfigurace v registry (bez FUNCTION_LOADSAVECONFIGURATION je ignorovan);
    // 'extensions' jsou zakladni pripony (napr. jen "ARJ"; "A01", atd. uz ne) zpracovavanych
    // archivu oddelene ';' (zde nema ';' zadnou escape sekvenci) - Salamander tyto pripony pouziva
    // jen pri hledani nahrady za odstranene panelove archivatory (nastava pri odstraneni pluginu;
    // resi se problem "co se ted postara o priponu XXX, kdyz byl puvodni asociovany archivator
    // odstranen v ramci pluginu PPP?") (bez FUNCTION_PANELARCHIVERVIEW a bez FUNCTION_PANELARCHIVEREDIT
    // je ignorovan); 'fsName' je navrhovane jmeno (ziskani prideleneho jmena se provede pomoci
    // CSalamanderGeneralAbstract::GetPluginFSName) file systemu (bez FUNCTION_FILESYSTEM je
    // ignorovan, povolene znaky jsou 'a-zA-Z0-9_+-', min. delka 2 znaky), pokud plugin potrebuje
    // vic jmen file systemu, muze pouzit metodu CSalamanderPluginEntryAbstract::AddFSName;
    // vraci TRUE pri uspesnem prijeti dat
    function SetBasicPluginData2( pluginName  : PChar;
                                  functions   : DWORD;
                                  version     : PChar;
                                  copyright   : PChar;
                                  description : PChar;
                                  regKeyName  : PChar = nil;
                                  extensions  : PChar = nil;
                                  fsName      : PChar = nil ) : BOOL; virtual; stdcall; abstract;

    // vraci ukazatel na interface k obecne pouzitelnym funkcim Salamandera,
    // interface je platny po celou dobu existence pluginu (nejen v ramci
    // "SalamanderPluginEntry" funkce) a jde pouze o odkaz, takze se neuvolnuje
    function GetSalamanderGeneral : TSalamanderGeneralAbstract; virtual; stdcall; abstract;

    // vraci informace spojene s loadem pluginu; informace jsou vraceny v DWORD hodnote
    // jako logicky soucet flagu LOADINFO_XXX (pro test pritomnosti flagu pouzijte
    // podminku: (GetLoadInformation() & LOADINFO_XXX) != 0)
    function GetLoadInformation : DWORD; virtual; stdcall; abstract;

    // vraci informace z registracniho klice Salamandera, pokud je vystupni parametr NULL
    // hodnota se nevraci, jinak 'registeredVersion' je TRUE pokud je klic v poradku (existuje);
    // 'productID1'-'productID2'-'productID3' je cislo produktu (testuje se shoda s cislem v
    // klici pluginu); 'numberOfLicenses' - pocet koupenych licenci Salamandera (-1 = neomezeny
    // pocet); 'neverExpires' - TRUE pokud je casove neomezena, pokud je FALSE je v 'expiration'
    // datum platnosti; 'educationalLicense' - TRUE pokus jde o licenci pro vzdelavani
    procedure GetRegistrationKeyInfo( registeredVersion  : PBOOL;
                                      productID1         : PDWORD;
                                      productID2         : PDWORD;
                                      productID3         : PDWORD;
                                      numberOfLicenses   : PInteger;
                                      neverExpires       : PBOOL;
                                      expiration         : PSystemTime;
                                      educationalLicense : PBOOL ); virtual; stdcall; abstract;

    // nahraje modul s jazykove zavislymi resourcy (SLG-cko); vzdy zkusi nahrat modul
    // stejneho jazyku v jakem prave bezi Salamander, pokud takovy modul nenajde (nebo
    // nesouhlasi verze), necha uzivatele vybrat alternativni modul (pokud existuje vic
    // nez jedna alternativa + pokud uz nema ulozeny uzivateluv vyber z minuleho loadu
    // pluginu); pokud nenajde zadny modul, vraci NULL -> plugin by se mel ukoncit;
    // 'parent' je parent messageboxu s chybami a dialogu pro vyber alternativniho
    // jazykoveho modulu; 'pluginName' je jmeno pluginu (aby uzivatel tusil o jaky plugin
    // se jedna pri chybovem hlaseni nebo vyberu alternativniho jazykoveho modulu)
    // POZOR: tuto metodu je mozne volat jen jednou; ziskany handle jazykoveho modulu
    //        se uvolni automaticky pri unloadu pluginu
    function LoadLanguageModule( parent : HWND; pluginName : PChar ) : HINST; virtual; stdcall; abstract;

    // vraci ID aktualniho jazyku zvoleneho pro prostredi Salamandera (napr. english.spl =
    // English (US) = 0x409, czech.spl = Czech = 0x405)
    function GetCurrentSalamanderLanguageID : Word; virtual; stdcall; abstract;

    // vraci ukazatel na interface poskytujici upravene Windows controly pouzivane
    // v Salamanderovi, interface je platny po celou dobu existence pluginu (nejen
    // v ramci "SalamanderPluginEntry" funkce) a jde pouze o odkaz, takze se neuvolnuje
    function GetSalamanderGUI : TSalamanderGUIAbstract; virtual; stdcall; abstract;

    // vraci ukazatel na interface k Windows XP Visual Styles,
    // interface je platny po celou dobu existence pluginu (nejen v ramci
    // "SalamanderPluginEntry" funkce) a jde pouze o odkaz, takze se neuvolnuje
    // vraci NULL, pokud knihovna uxtheme.dll neni k dispozici (nebezime pod Windows XP
    // nebo pod Common Controls 6.0 a vyssi)
    function GetXpThemeSupport : TXpThemeSupportAbstract; virtual; stdcall; abstract;

    // vraci ukazatel na interface pro komfortni praci se soubory,
    // interface je platny po celou dobu existence pluginu (nejen v ramci
    // "SalamanderPluginEntry" funkce) a jde pouze o odkaz, takze se neuvolnuje
    function GetSalamanderSafeFile : TSalamanderSafeFileAbstract; virtual; stdcall; abstract;

    // nastavi URL, ktere se ma zobrazit v okne Plugins Manager jako home-page pluginu;
    // hodnotu si Salamander udrzuje az do pristiho loadu pluginu (URL se zobrazuje i pro
    // nenaloadene pluginy); pri kazdem loadu pluginu je URL nutne nastavit znovu, jinak
    // se zadne URL nezobrazi (obrana proti drzeni neplatneho URL home-page)
    procedure SetPluginHomePageURL( url : PChar ); virtual; stdcall; abstract;

    // pridani dalsiho jmena file systemu; bez FUNCTION_FILESYSTEM v parametru 'functions'
    // pri volani metody SetBasicPluginData2 vraci tato metoda vzdy jen chybu;
    // 'fsName' je navrhovane jmeno (ziskani prideleneho jmena se provede pomoci
    // CSalamanderGeneralAbstract::GetPluginFSName) file systemu (povolene znaky jsou
    // 'a-zA-Z0-9_+-', min. delka 2 znaky); v 'newFSNameIndex' (nesmi byt NULL) se
    // vraci index nove pridaneho jmena file systemu; vraci TRUE v pripade uspechu;
    // vraci FALSE pri fatalni chybe (nedostatek pameti) - v tomto pripade se
    // 'newFSNameIndex' ignoruje
    // omezeni: nesmi se volat pred metodou SetBasicPluginData2
    function AddFSName( fsName : PChar; newFSNameIndex : PInteger ) : BOOL; virtual; stdcall; abstract;

    // zobrazi dialog "plugin is not registered, please purchase a license"; 'pluginIcon' je ikona
    // 32x32 bodu zobrazena v dialogu; 'buyNowURL' je nakupni stranka pluginu (otevre se po
    // kliknuti na Buy NOW); vraci TRUE pokud se ma plugin naloadit (FALSE = uzivatel chce blokovat
    // pouzivani pluginu, plugin by se nemel naloadit)
    // omezeni: nesmi se volat pred metodou SetBasicPluginData2; pokud ma dialog obsahovat URL homepage,
    //          je predem treba zavolat SetPluginHomePageURL
    function ShowNotRegisteredDlg( pluginIcon : HICON; buyNowURL : PChar ) : BOOL; virtual; stdcall; abstract;
  end;

//
// ****************************************************************************
// FSalamanderPluginEntry
//
// Altap Salamander 1.6 or Later Plugin Entry Point Function Type,
// tuto funkci plugin vyvazi jako "SalamanderPluginEntry" a Salamander ji vola
// pro pripojeni pluginu v okamziku loadu pluginu
// vraci interface pluginu v pripade uspesneho pripojeni, jinak NULL,
// interface pluginu se uvolnuje volanim jeho metody Release pred unloadem pluginu
  TSalamanderPluginEntryFunc = function( salamander : TSalamanderPluginEntryAbstract ) : TPluginInterfaceAbstract; stdcall;

//
// ****************************************************************************
// FSalamanderPluginGetReqVer
//
// Altap Salamander 2.5 Beta 2 or Later Plugin Get Required Version of Salamander Function Type,
// tuto funkci plugin vyvazi jako "SalamanderPluginGetReqVer" a Salamander ji vola
// jako prvni funkci pluginu (pred "SalamanderPluginEntry") v okamziku loadu pluginu;
// vraci verzi Salamandera, pro kterou je plugin staven
  TSalamanderPluginGetReqVerFunc = function : Integer; stdcall;

function SALHOTKEY( vk : Byte; mods : Word ) : DWORD;

implementation

// makro pro pripravu 'HotKey' pro AddMenuItem()
// LOWORD - hot key (virtual key + modifikatory) (LOBYTE - virtual key, HIBYTE - modifikatory)
// mods: kombinace HOTKEYF_CONTROL, HOTKEYF_SHIFT, HOTKEYF_ALT
// examples: SALHOTKEY('A', HOTKEYF_CONTROL | HOTKEYF_SHIFT), SALHOTKEY(VK_F1, HOTKEYF_CONTROL | HOTKEYF_ALT | HOTKEYF_EXT)
//#define SALHOTKEY(vk,mods,cst) ((DWORD)(((BYTE)(vk)|((WORD)((BYTE)(mods))<<8))|(((DWORD)(BYTE)(cst))<<16)))
function SALHOTKEY( vk : Byte; mods : Word ) : DWORD;
begin
  Result:= (vk or (mods shl 8));
end;

end.

