(*
  Object Pascal translation of the Altap Salamander SDK library.
  Original source: spl_com.h

  Portions created by ALTAP are Copyright (c) ALTAP, spol. s r.o.
  All rights reserved.

  Portions created by LostGenius Posse are Copyright (c) 2007, LostGenius Posse (Timo 'jomanto' Boll)
  All rights reserved.

  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

      * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
      * Redistributions in binary form may reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
      * Neither the name of the LostGenius Posse nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*)
(*
  History:
  
  2007-05-29: initial version
*)
unit spl_com;

interface

uses
  winliblt;

const
//
// ****************************************************************************
// CSalamanderDirectoryAbstract
//
// trida reprezentuje adresarovou strukturu - soubory a adresare na pozadovanych cestach, root cesta je "",
// oddelovace v ceste jsou backslashe ('\\')
//

  // hodnota pro CFileData::IconOverlayIndex v pripade, ze ikona nema overlay
  ICONOVERLAYINDEX_NOTUSED = 15;

  // konstanty urcujici platnost dat, ktera jsou primo ulozena v CFileData (velikost, pripona, atd.)
  // nebo se generuji z primo ulozenych dat automaticky (file-type se generuje z pripony);
  // Name + NameLen jsou povinne (musi byt platne vzdy); platnost PluginData si ridi plugin sam
  // (Salamander tento atribut ignoruje)
  VALID_DATA_EXTENSION   = $0001;                                               // pripona je ulozena v Ext (bez: vsechny Ext = konec Name)
  VALID_DATA_DOSNAME     = $0002;                                               // DOS name je ulozeno v DosName (bez: vsechny DosName = NULL)
  VALID_DATA_SIZE        = $0004;                                               // velikost v bytech je ulozena v Size (bez: vsechny Size = 0)
  VALID_DATA_TYPE        = $0008;                                               // file-type muze byt generovan z Ext (bez: negeneruje se)
  VALID_DATA_DATE        = $0010;                                               // datum modifikace (UTC-based) je ulozen v LastWrite (bez: vsechny datumy v LastWrite jsou 1.1.1602 v local time)
  VALID_DATA_TIME        = $0020;                                               // cas modifikace (UTC-based) je ulozen v LastWrite (bez: vsechny casy v LastWrite jsou 0:00:00 v local time)
  VALID_DATA_ATTRIBUTES  = $0040;                                               // atributy jsou ulozeny v Attr (ORovane Win32 API konstanty FILE_ATTRIBUTE_XXX) (bez: vsechny Attr = 0)
  VALID_DATA_HIDDEN      = $0080;                                               // "ghosted" priznak ikony je ulozen v Hidden (bez: vsechny Hidden = 0)
  VALID_DATA_ISLINK      = $0100;                                               // IsLink obsahuje 1 pokud jde o link, ikonka ma overlay linku (bez: vsechny IsLink = 0)
  VALID_DATA_ISOFFLINE   = $0200;                                               // IsOffline obsahuje 1 pokud jde o offline soubor/adresar, ikonka ma offline overlay (bez: vsechny IsOffline = 0)
  VALID_DATA_PL_SIZE     = $0400;                                               // ma smysl jen bez pouziti VALID_DATA_SIZE: plugin ma aspon pro nektere soubory/adresare ulozenou velikost v bytech (nekde v PluginData), pro ziskani teto velikosti Salamander vola CPluginDataInterfaceAbstract::GetByteSize()
  VALID_DATA_PL_DATE     = $0800;                                               // ma smysl jen bez pouziti VALID_DATA_DATE: plugin ma aspon pro nektere soubory/adresare ulozeny datum modifikace (nekde v PluginData), pro ziskani teto velikosti Salamander vola CPluginDataInterfaceAbstract::GetLastWriteDate()
  VALID_DATA_PL_TIME     = $1000;                                               // ma smysl jen bez pouziti VALID_DATA_TIME: plugin ma aspon pro nektere soubory/adresare ulozeny cas modifikace (nekde v PluginData), pro ziskani teto velikosti Salamander vola CPluginDataInterfaceAbstract::GetLastWriteTime()

  VALID_DATA_NONE        = 0;                                                   // pomocna konstanta - platne je jen Name a NameLen

{$ifdef INSIDE_SALAMANDER}
    // VALID_DATA_ALL je jen pro interni pouziti v Salamanderovi (jadre), pluginy si naORuji
    // jen konstanty odpovidajici pluginem dodavanym datum (zamezi se tak problemum pri zavedeni
    // dalsich konstant a jim odpovidajicim datum)
    VALID_DATA_ALL       = $FFFF;
{$endif}// INSIDE_SALAMANDER

  // Pokud je zapnuto skryvani hidden a system souboru a adresaru, nezobrazuji se v panelech polozky s
  // Hidden==1 a Attr obsahujicim FILE_ATTRIBUTE_HIDDEN a/nebo FILE_ATTRIBUTE_SYSTEM.

  // konstanty priznaku pro CSalamanderDirectoryAbstract:
  // jmena souboru a adresaru (i v cestach) se maji porovnavat case-sensitive (bez tohoto flagu je
  // porovnavani case-insensitive - standardni chovani ve Windows)
  SALDIRFLAG_CASESENSITIVE   = $0001;
  // jmena podadresaru v ramci kazdeho adresare se nebudou testovat na duplicitu (tento
  // test je casove narocny a je nutny jen v archivech, pokud se pridavaji polozky nejen
  // do rootu - aby fungovalo napr. pridani "file1" na "dir1" nasledovane pridanim
  // "dir1" - "dir1" se prida prvni operaci (automaticky se prida neexistujici cesta),
  // druha operace uz jen obnovi udaje o "dir1" (nesmi ho pridat znovu))
  SALDIRFLAG_IGNOREDUPDIRS   = $0002;

//
// ****************************************************************************
// CSalamanderViewAbstract
//
// sada metod Salamandera pro praci se sloupci v panelu (vypinani/zapinani/pridavani/nastavovani)

  // rezimy pohledu panelu
  VIEW_MODE_TREE       = 1;
  VIEW_MODE_BRIEF      = 2;
  VIEW_MODE_DETAILED   = 3;
  VIEW_MODE_ICONS      = 4;
  VIEW_MODE_THUMBNAILS = 5;
  VIEW_MODE_TILES      = 6;

  TRANSFER_BUFFER_MAX    = 1024;                                                // velikost bufferu pro prenos obsahu sloupcu z pluginu do Salamandera
  COLUMN_NAME_MAX        = 30;
  COLUMN_DESCRIPTION_MAX = 100;

  // Identifikatory sloupcu. Sloupce vlozene pluginem maji nastaveno ID==COLUMN_ID_CUSTOM.
  // Standardni sloupce Salamandera maji ostatni ID.
  COLUMN_ID_CUSTOM      = 0;                                                    // sloupec je poskytovan pluginem - o ulozeni jeho dat se postara plugin
  COLUMN_ID_NAME        = 1;                                                    // zarovnano vlevo, podporuje FixedWidth
  // zarovnano vlevo, podporuje FixedWidth; samostatny sloupec "Ext", muze byt jen na indexu==1;
  // pokud sloupec neexistuje a v datech panelu (viz CSalamanderDirectoryAbstract::SetValidData())
  // se nastavi VALID_DATA_EXTENSION, je sloupec "Ext" zobrazen ve sloupci "Name"
  COLUMN_ID_EXTENSION   = 2;
  COLUMN_ID_DOSNAME     = 3;                                                    // zarovnano vlevo
  COLUMN_ID_SIZE        = 4;                                                    // zarovnano vpravo
  COLUMN_ID_TYPE        = 5;                                                    // zarovnano vlevo, podporuje FixedWidth
  COLUMN_ID_DATE        = 6;                                                    // zarovnano vpravo
  COLUMN_ID_TIME        = 7;                                                    // zarovnano vpravo
  COLUMN_ID_ATTRIBUTES  = 8;                                                    // zarovnano vpravo
  COLUMN_ID_DESCRIPTION = 9;                                                    // zarovnano vlevo, podporuje FixedWidth

type
//
// ****************************************************************************
// CSalamanderRegistryAbstract
//
// sada metod Salamandera pro praci se systemovym registry,
// pouziva se v CPluginInterfaceAbstract::LoadConfiguration
// a CPluginInterfaceAbstract::SaveConfiguration

  TSalamanderRegistryAbstract = class
    // vycisti klic 'key' od vsech podklicu a hodnot, vraci uspech
    function ClearKey( key : HKEY ) : BOOL; virtual; stdcall; abstract;

    // vytvori nebo otevre existujici podklic 'name' klice 'key', vraci 'createdKey' a uspech;
    // ziskany klic ('createdKey') je nutne zavrit volanim CloseKey
    function CreateKey( key : HKEY; name : PChar; var createdKey : HKEY ) : BOOL; virtual; stdcall; abstract;

    // otevre existujici podklic 'name' klice 'key', vraci 'openedKey' a uspech
    // ziskany klic ('openedKey') je nutne zavrit volanim CloseKey
    function OpenKey( key : HKEY; name : PChar; var openedKey : HKEY ) : BOOL; virtual; stdcall; abstract;

    // zavre klic otevreny pres OpenKey nebo CreateKey
    procedure CloseKey( key : HKEY ); virtual; stdcall; abstract;

    // smaze podklic 'name' klice 'key', vraci uspech
    function DeleteKey( key : HKEY; name : PChar ) : BOOL; virtual; stdcall; abstract;

    // nacte hodnotu 'name'+'type'+'buffer'+'bufferSize' z klice 'key', vraci uspech
    function GetValue( key : HKEY; name : PChar; _type : DWORD; buffer : Pointer;
                       bufferSize : DWORD ) : BOOL; virtual; stdcall; abstract;

    // ulozi hodnotu 'name'+'type'+'data'+'dataSize' do klice 'key', pro retezce je mozne
    // zadat 'dataSize' == -1 -> vypocet delky retezce pomoci funkce strlen,
    // vraci uspech
    function SetValue( key : HKEY; name : PChar; _type : DWORD; data : Pointer;
                       dataSize : DWORD ) : BOOL; virtual; stdcall; abstract;

    // smaze hodnotu 'name' klice 'key', vraci uspech
    function DeleteValue( key : HKEY; name : PChar ) : BOOL; virtual; stdcall; abstract;

    // vytahne do 'bufferSize' protrebnou velikost pro hodnotu 'name'+'type' z klice 'key', vraci uspech
    function GetSize( key : HKEY; name : PChar; _type : DWORD; var bufferSize : DWORD ) : BOOL; virtual; stdcall; abstract;
  end;

//
// ****************************************************************************
// CSalamanderDirectoryAbstract
//
// trida reprezentuje adresarovou strukturu - soubory a adresare na pozadovanych cestach, root cesta je "",
// oddelovace v ceste jsou backslashe ('\\')
//

  // zaznam kazdeho souboru a adresare v Salamanderovi (zakladni data o souboru/adresari)
  TFileData = packed record                                                     // nesmi sem prijit destruktor !
    Name : PChar;                                                               // naalokovane jmeno souboru (bez cesty), nutne alokovat na heapu
                                                                                // Salamandera (viz CSalamanderGeneralAbstract::Alloc/Free)
    Ext : PChar;                                                                // ukazatel do Name za prvni tecku zprava (vyjma tecky na zacatku jmena)
                                                                                // nebo na konec Name, pokud pripona neexistuje; je-li v konfiguraci
                                                                                // nastaveno FALSE pro SALCFG_SORTBYEXTDIRSASFILES, je v Ext pro
                                                                                // adresare ukazatel na konec Name (adresare nemaji pripony)
    Size : TLargeInteger;                                                       // velikost souboru v bytech
    Attr : DWORD;                                                               // atributy souboru - ORovane konstanty FILE_ATTRIBUTE_XXX
    LastWrite : TFileTime;                                                      // cas posledniho zapisu do souboru (UTC-based time)
    DosName : PChar;                                                            // naalokovane DOS 8.3 jmeno souboru, neni-li treba je NULL, nutne
                                                                                // alokovat na heapu Salamandera (viz CSalamanderGeneralAbstract::Alloc/Free)
    PluginData : DWORD;                                                         // pouziva plugin skrze CPluginDataInterfaceAbstract, Salamander ignoruje
    NameLen    : Cardinal;                                                      // delka retezce Name (strlen(Name)) - POZOR: maximalni delka jmena je (MAX_PATH - 5)
    Hidden     : Cardinal;                                                      // je hidden? (je-li 1, ikonka je pruhlednejsi o 50% - ghosted)
    IsLink     : Cardinal;                                                      // je link? (je-li 1, ikonka ma overlay linku) - standardni plneni viz CSalamanderGeneralAbstract::IsFileLink(CFileData::Ext)
    IsOffline  : Cardinal;                                                      // je offline? (je-li 1, ikonka ma overlay offline - cerne hodiny)

    // flagy pro interni pouziti v Salamanderovi: nuluji se pri pridani do CSalamanderDirectoryAbstract
    Association      : Cardinal;                                                // vyznam jen pro zobrazeni 'simple icons' - ikona asociovaneho souboru, jinak 0
    Selected         : Cardinal;                                                // read-only flag oznaceni (0 - polozka neoznacena, 1 - polozka oznacena)
    Shared           : Cardinal;                                                // je adresar sdileny? u souboru se nepouziva
    Archive          : Cardinal;                                                // jedna se o archiv? pouziva se pro zobrazovani ikony archivu v panelu
    SizeValid        : Cardinal;                                                // je u adresare napocitana jeho velikost?
    Dirty            : Cardinal;                                                // je potreba tuto polozku prekreslit? (pouze docasna platnost; mezi nastavenim bitu a prekreslenim panelu nesmi byt pumpovana message queue, jinak muze dojit k prekresleni ikonky (icon reader) a tim resetu bitu! v dusledku se neprekresli polozka)
    CutToClip        : Cardinal;                                                // je CUT-nutej na clipboardu? (je-li 1, ikonka je pruhlednejsi o 50% - ghosted)
    IconOverlayDone  : Cardinal;                                                // jen pro potreby icon-reader-threadu: ziskavame nebo uz jsme ziskavali icon-overlay? (0 - ne, 1 - ano)
    IconOverlayIndex : Cardinal;                                                // index icon-overlaye (pokud ikona nema zadny overlay, je zde hodnota ICONOVERLAYINDEX_NOTUSED)
  end;
  PFileData = ^TFileData;

  TPluginDataInterfaceAbstract = class;

  TSalamanderDirectoryAbstract = class
    // vycisti cely objekt, pripravi ho pro dalsi pouziti; pokud 'pluginData' neni NULL, pouzije
    // se pro soubory a adresare k uvolneni dat specifickych pluginu (CFileData::PluginData);
    // nastavuje standardni hodnotu masky platnych dat (sumu vsech VALID_DATA_XXX) a priznaku
    // objektu (viz metoda SetFlags)
    procedure Clear( pluginData : TPluginDataInterfaceAbstract ); virtual; stdcall; abstract;

    // zadani masky platnych dat, podle ktere se urcuje, ktera data z CFileData jsou platna
    // a ktera se maji pouze "nulovat" (viz komentar k VALID_DATA_XXX); maska 'validData'
    // obsahuje ORovane hodnoty VALID_DATA_XXX; standardni hodnota masky je suma vsech VALID_DATA_XXX;
    // masku platnych dat je potreba nastavit pred volanim AddFile/AddDir
    procedure SetValidData( validData : DWORD ); virtual; stdcall; abstract;

    // nastaveni priznaku pro tento objekt; 'flags' je kombinace ORovanych priznaku SALDIRFLAG_XXX,
    // standardni hodnota priznaku objektu je nula pro archivatory (zadny priznak neni nastaven)
    // a SALDIRFLAG_IGNOREDUPDIRS pro file-systemy (smi se pridavat jen do rootu, test na duplicitu
    // adresaru je zbytecny)
    procedure SetFlags( flags : DWORD ); virtual; stdcall; abstract;

    // prida soubor na zadanou cestou (relativni k tomuto "salamander-adresari"), vraci uspech
    // retezec path se pouziva jen uvnitr funkce, obsah struktury file se pouziva i mimo funkci
    // (neuvolnovat pamet naalokovanou pro promenne uvnitr struktury)
    // v pripade neuspechu, je treba obsah struktury file uvolnit;
    // parametr 'pluginData' neni NULL jen pro archivy (FS pouzivaji jen prazdne 'path' (==NULL));
    // neni-li 'pluginData' NULL, pouziva se 'pluginData' pri zakladani novych adresaru (pokud
    // 'path' neexistuje), viz CPluginDataInterfaceAbstract::GetFileDataForNewDir;
    // kontrola unikatnosti jmena souboru na ceste 'path' se neprovadi
    function AddFile( path : PChar; var _file : TFileData;
                      pluginData : TPluginDataInterfaceAbstract ) : BOOL; virtual; stdcall; abstract;

    // prida adresar na zadanou cestu (relativni k tomuto "salamander-adresari"), vraci uspech
    // retezec path se pouziva jen uvnitr funkce, obsah struktury file se pouziva i mimo funkci
    // (neuvolnovat pamet naalokovanou pro promenne uvnitr struktury)
    // v pripade neuspechu, je treba obsah struktury file uvolnit;
    // parametr 'pluginData' neni NULL jen pro archivy (FS pouzivaji jen prazdne 'path' (==NULL));
    // neni-li 'pluginData' NULL, pouziva se pri zakladani novych adresaru (pokud 'path' neexistuje),
    // viz CPluginDataInterfaceAbstract::GetFileDataForNewDir;
    // kontrola unikatnosti jmena adresare na ceste 'path' se provadi, dochazi-li k pridani
    // jiz existujiciho adresare, dojde k uvolneni puvodnich dat (neni-li 'pluginData' NULL, vola
    // se pro uvolneni dat i CPluginDataInterfaceAbstract::ReleasePluginData) a ulozeni dat z 'dir'
    // (je nutne pro obnovu dat adresaru, ktere se vytvori automaticky pri neexistenci 'path');
    // specialita pro FS (nebo objekt alokovany pres CSalamanderGeneralAbstract::AllocSalamanderDirectory
    // s 'isForFS'==TRUE): je-li dir.Name "..", je adresar pridan jako up-dir (muze byt jen jeden,
    // zobrazuje se vzdy na zacatku listingu a ma specialni ikonu)
    function AddDir( path : PChar; dir : TFileData;
                     pluginData : TPluginDataInterfaceAbstract ) : BOOL; virtual; stdcall; abstract;

    // vraci pocet souboru v objektu
    function GetFilesCount : Integer; virtual; stdcall; abstract;

    // vraci pocet adresaru v objektu
    function GetDirsCount : Integer; virtual; stdcall; abstract;

    // vraci soubor z indexu 'index', vracena data lze pouzit jen pro cteni
    function GetFile( index : Integer ) : PFileData; virtual; stdcall; abstract;

    // vraci adresar z indexu 'index', vracena data lze pouzit jen pro cteni
    function GetDir( index : Integer ) : PFileData; virtual; stdcall; abstract;

    // vraci objekt CSalamanderDirectory pro adresar z indexu 'index', vraceny objekt lze
    // pouzit jen pro cteni (objekty pro prazdne adresare nejsou alokovany, vraci se jeden
    // globalni prazdny objekt - zmena tohoto objektu by se projevila globalne)
    function GetSalDir( index : Integer ) : TSalamanderDirectoryAbstract; virtual; stdcall; abstract;

    // Pluginu umoznuje predem sdelit predpokladany pocet souboru a adresaru v tomto adresari.
    // Salamander si upravi realokacni strategii tak, aby pridavani prvku prilis nebrzdilo.
    // Ma smysl volat pro adresare obsahujici tisice souboru nebo adresaru. V pripade desitek
    // tisic uz je zavolani teto metody temer nutnost, jinak realokace zaberou nekolik vterin.
    // 'files' a 'dirs' tedy vyjadruji priblizny celkovy pocet souboru a adresaru. 
    // Pokud je nektera z hodnot -1, bude ji Salamander ignorovat.
    // Metodu ma vyznam volat pouze pokud je adresar prazdny, tedy nebylo volano AddFile nebo AddDir.
    procedure SetApproximateCount( files : Integer; dirs : Integer ); virtual; stdcall; abstract;
  end;

//
// ****************************************************************************
// SalEnumSelection a SalEnumSelection2
//

// enumerator, vraci jmena souboru, konci vracenim NULL;
// 'enumFiles' == -1 -> reset enumerace (po tomto volani zacina enumerace opet od zacatku), vsechny
//                      dalsi parametry (az na 'param') jsou ignorovany, nema navratove hodnoty (dava
//                      vse na nulu)
// 'enumFiles' == 0 -> enumerace souboru a podadresaru jen z korene
// 'enumFiles' == 1 -> enumerace vsech souboru a podadresaru
// 'enumFiles' == 2 -> enumerace vsech podadresaru, soubory jen z korene;
// v 'isDir' (neni-li NULL) vraci TRUE pokud jde o adresar; v 'size' (neni-li NULL) vraci
// velikost souboru (u adresaru se vraci velikost jen pri 'enumFiles' == 0 - jinak je nulova);
// neni-li 'fileData' NULL, vraci se v nem ukazatel na strukturu CFileData vraceneho
// souboru/adresare (pokud enumerator vraci NULL, vraci se v 'fileData' take NULL);
// 'param' je parametr 'nextParam' predavany spolu s ukazatelem na funkci tohoto
// typu
  TSalEnumSelectionFunc = function( enumFiles : Integer; isDir : PBOOL;
                                    size : PLargeInteger; fileData : PFileData;
                                    param : Pointer ) : PChar; stdcall;

// enumerator, vraci jmena souboru, konci vracenim NULL;
// 'enumFiles' == -1 -> reset enumerace (po tomto volani zacina enumerace opet od zacatku), vsechny
//                      dalsi parametry (az na 'param') jsou ignorovany, nema navratove hodnoty (dava
//                      vse na nulu)
// 'enumFiles' == 0 -> enumerace souboru a podadresaru jen z korene
// 'enumFiles' == 1 -> enumerace vsech souboru a podadresaru
// 'enumFiles' == 2 -> enumerace vsech podadresaru, soubory jen z korene;
// k chybe muze dojit jen pri 'enumFiles' == 1 nebo 'enumFiles' == 2 ('enumFiles' == 0 vubec
// nepracuje s diskem); 'parent' je parent pripadnych messageboxu s chybami (NULL znamena
// nezobrazovat chyby); v 'dosName' (neni-li NULL) vraci DOSovy nazev (8.3; jen pokud existuje,
// jinak NULL); v 'isDir' (neni-li NULL) vraci TRUE pokud jde o adresar; v 'size' (neni-li NULL)
// vraci velikost souboru (u adresaru nulu); v 'attr' (neni-li NULL) vraci atributy
// souboru/adresare; v 'lastWrite' (neni-li NULL) vraci cas posledniho zapisu do souboru/adresare;
// 'param' je parametr 'nextParam' predavany spolu s ukazatelem na funkci tohoto typu;
// v 'errorOccured' (neni-li NULL) se vraci TRUE, pokud doslo behem cteni dat z disku k chybe
// (uzivatel ji mohl preskocit - vraci jen soubory/adresare, ktere se podarilo nacist; nebo
// mohl uzivatel prerusit operaci - vraci uz pri prvnim volani NULL)
  TSalEnumSelection2Func = function( parent : HWND; enumFiles : Integer; dosName : PChar;
                                     isDir : PBOOL; size : PLargeInteger; attr : PDWORD;
                                     lastWrite : PFileTime; param : Pointer;
                                     errorOccured : PBOOL ) : PChar; stdcall;

//
// ****************************************************************************
// CSalamanderViewAbstract
//
// sada metod Salamandera pro praci se sloupci v panelu (vypinani/zapinani/pridavani/nastavovani)

  // Callback pro naplneni bufferu znakama, ktere se maji zobrazit v prislusnem sloupci.
  // Z duvodu optimalizace funkce nedostava/nevraci promenne prostrednictvim parametru,
  // ale prostrednictvim globalni promennych (CSalamanderViewAbstract::GetTransferVariables).
  TColumnGetTextProc = procedure; stdcall;

  // Callback pro ziskani indexu jednoduchych ikon pro FS s vlastnimi ikonami (pitFromPlugin).
  // Z duvodu optimalizace funkce nedostava/nevraci promenne prostrednictvim parametru,
  // ale prostrednictvim globalni promennych (CSalamanderViewAbstract::GetTransferVariables).
  // Z globalnich promennych callback vyuziva jen TransferFileData a TransferIsDir.
  TGetPluginIconIndexFunc = function : Integer; stdcall;

  // sloupec muze vzniknout dvema zpusoby:
  // 1) Sloupec vytvoril Salamander na zaklade sablony aktualniho pohledu.
  //    V tomto pripade ukazatel 'GetText' (na plnici funkci) ukazuje do Salamandera
  //    a ziskava texty standardne z CFileData.
  //    Hodnota promenne 'ID' je ruzna od COLUMN_ID_CUSTOM.
  //
  // 2) Sloupec pridal plugin na zaklade svych potreb.
  //    'GetText' ukazuje do pluginu a 'ID' je rovno COLUMN_ID_CUSTOM.

  TColumn = packed record
    Name           : Array[0..COLUMN_NAME_MAX] of Char;                         // "Name", "Ext", "Size", ... nazev sloupce, pod
                                                                                // kterym sloupec vystupuje v pohledu a v menu
                                                                                // Nesmi obsahovat prazdny retezec.
                                                                                // POZOR: Muze obsahovat (za prvnim null-terminatorem)
                                                                                // i nazev sloupce "Ext" - toto nastava pokud neexistuje
                                                                                // samostatny sloupec "Ext" a v datech panelu (viz
                                                                                // CSalamanderDirectoryAbstract::SetValidData()) se
                                                                                // nastavi VALID_DATA_EXTENSION. Pro spojeni dvou
                                                                                // retezcu poslouzi CSalamanderGeneralAbstract::AddStrToStr().

    Description    : Array[0..COLUMN_DESCRIPTION_MAX] of Char;                  // Tooltip v header line
                                                                                // Nesmi obsahovat prazdny retezec.
                                                                                // POZOR: Muze obsahovat (za prvnim null-terminatorem)
                                                                                // i popis sloupce "Ext" - toto nastava pokud neexistuje
                                                                                // samostatny sloupec "Ext" a v datech panelu (viz
                                                                                // CSalamanderDirectoryAbstract::SetValidData()) se
                                                                                // nastavi VALID_DATA_EXTENSION. Pro spojeni dvou
                                                                                // retezcu poslouzi CSalamanderGeneralAbstract::AddStrToStr().

    GetText        : TColumnGetTextProc;                                        // callback pro ziskani textu (popis u deklatace typu FColumnGetText)

    CustomData     : DWORD;                                                     // Neni pouzivana Salamanderem;  plugin ji muze
                                                                                // vyuzit pro rozliseni svych pridanych sloupcu.

    SupportSorting : Cardinal;                                                  // je sloupec mozne radit?

    LeftAlignment  : Cardinal;                                                  // pro TRUE je sloupec zarovnavan vlevo; jinak vpravo

    ID             : Cardinal;                                                  // identifikator sloupce
                                                                                // Pro standardni sloupce poskytovane Salamanderem
                                                                                // obsahuje hodnoty ruzne od COLUMN_ID_CUSTOM.
                                                                                // Pro sloupce pridane pluginem obsahuje vzdy
                                                                                // hodnotu COLUMN_ID_CUSTOM.

    // Promenne Width a FixedWidth muzou byt zmeneny uzivatelem behem prace s panelem.
    // Standardni sloupce poskytovane Salamanderem maji zajisteno ukladani/nacitani
    // techto hodnot.
    // Hodnoty techto promennych pro sloupce poskytovane pluginem je treba ulozit/nacist
    // v ramci pluginu.
    // Sloupce, jejichz sirku pocita Salamander na zaklade obsahu a uzivatel ji nemuze
    // menit, nazyvame 'elasticke'. Sloupce, pro ktere muze uzivatel nastavit sirku nazyvame
    // 'pevne'/'fixed'.
    Width          : Cardinal;                                                  // Sirka sloupce v pripade, ze je v rezimu pevne (nastavitelne) sirky.
    FixedWidth     : Cardinal;                                                  // Je sloupec v rezimu pevne (nastavitelne) sirky?

    // pracovni promenne (nikam se neukladaji a neni treba je inicializovat)
    // jsou urcene pro interni potreby Salamandera a pluginy je ignoruji,
    // protoze jejich obsah neni pri volani pluginu zaruceny
    MinWidth       : Cardinal;                                                  // Minimalni sirka, na kterou muze byt sloupce smrsten.
                                                                                // Je pocitana na zaklade nazvu sloupce a jeho raditelnosti
                                                                                // tak, aby byla hlavicka sloupce vzdy viditelna
  end;
  PColumn = ^TColumn;

// Plugin prostrednictvim tohoto rozhrani muze pri zmene cesty zmenit rezim
// zobrazeni v panelu. Veskera prace se sloupci se tyka jen vsech detailed rezimu
// (Detailed + Types + tri volitelne rezimy Alt+8/9/0). Pri zmene cesty dostane
// plugin standardni sadu sloupcu nagenerovanou na zaklade sablony aktualniho
// pohledu. Plugin muze tuto sadu modifikovat. Modifikace neni trvaleho razu
// a pri pristi zmene cesty obdrzi plugin opet standardni sadu sloupcu. Muze tak
// napriklad odstranit nektery ze std. sloupcu. Pred novym plnenim std. sloupci
// dostane plugin prilezitost ulozeni informaci o svych sloupcich (COLUMN_ID_CUSTOM).
// Muze tak ulozit jejich 'Width' a 'FixedWidth', ktere uzivatel mohl v panelu
// nastavit (viz ColumnFixedWidthShouldChange() a ColumnWidthWasChanged() v interfacu
// CPluginDataInterfaceAbstract). Pokud plugin zmeni rezim pohledu, zmena je trvala
// (napr. prepnuti na rezim Thumbnails zustane i po opusteni pluginove cesty).

  TSalamanderViewAbstract = class
    // -------------- panel ----------------

    // vraci rezim, ve kterem je zobrazen panel (tree/brief/detailed/icons/thumbnails/tiles)
    // vraci jednu z hodnot VIEW_MODE_xxxx (rezim Detailed, Types a tri volitelne rezimy jsou
    // vsechny VIEW_MODE_DETAILED)
    function GetViewMode : DWORD; virtual; stdcall; abstract;

    // Nastavi rezim panelu na 'viewMode'. Pokud jde o nektery z detailed rezimu, muze
    // odstranit nektere ze standardnich sloupcu (viz. 'validData'). Proto je vhodne tuto
    // funkci volat jako prvni - pred ostatnimi funkcemi z toho ifacu, ktere modifikuji
    // sloupce.
    //
    // 'viewMode' je jedna z hodnot VIEW_MODE_xxxx
    // Rezim panelu nelze zmenit ani na Types ani na jeden ze tri volitelnych detailed rezimu
    // (vsechny zastupuje konstanta VIEW_MODE_DETAILED pouzita pro Detailed rezim panelu).
    // Ovsem pokud je zrovna jeden z techto ctyr rezimu v panelu zvoleny a 'viewMode' je
    // VIEW_MODE_DETAILED, zustane tento rezim zvoleny (aneb neprepne se na Detailed rezim).
    // Zmena rezimu panelu je trvala (pretrva i po opusteni pluginove cesty).
    //
    // 'validData' informuje o tom, jaka data si plugin preje zobrazit v detailed rezimu, hodnota
    // se ANDuje s maskou platnych dat zadanou pomoci CSalamanderDirectoryAbstract::SetValidData
    // (nema smysl zobrazovat sloupce s "nulovanymi" hodnotami).
    procedure SetViewMode( viewMode : DWORD; validData : DWORD ); virtual; stdcall; abstract;

    // Vyzvedne ze Salamandera umisteni promennych, ktere nahrazuji parametry callbacku
    // CColumn::GetText. Na strane Salamandera se jedna o globalni promenne. Plugin si
    // ukazatele na ne ulozi do vlastnich globalnich promennych.
    //
    // promenne:
    //   transferFileData        [IN]     data, na jejichz zaklade se ma vykreslit polozka
    //   transferIsDir           [IN]     rovno 0, pokud jde o soubor (lezi v poli Files),
    //                                    rovno 1, pokud jde o adresar (lezi v poli Dirs),
    //                                    rovno 2, pokud jde o up-dir symbol
    //   transferBuffer          [OUT]    sem se nalejou data, maximalne TRANSFER_BUFFER_MAX znaku
    //                                    neni treba je terminovat nulou
    //   transferLen             [OUT]    pred navratem z callbacku se do teto promenne nastavi
    //                                    pocet naplnenych znaku bez terminatoru (terminator neni
    //                                    treba do bufferu zapisovat)
    //   transferRowData         [IN/OUT] ukazuje na DWORD, ktery je vzdy pred kreslenim sloupcu
    //                                    pro kazdy radek nulovan; lze pouzit pro optimalizace
    //                                    Salamander ma vyhrazene bity 0x00000001 az 0x00000008.
    //                                    Ostatni bity jsou k dispozici pro plugin.
    //   transferPluginDataIface [IN]     plugin-data-interface panelu, do ktereho se polozka
    //                                    vykresluje (patri k (*transferFileData)->PluginData)
    //   transferActCustomData   [IN]     CustomData sloupce, pro ktery se ziskava text (pro ktery
    //                                    se vola callback)
    procedure GetTransferVariables( var transferFileData        : PFileData;
                                    var transferIsDir           : PInteger;
                                    var transferBuffer          : PChar;
                                    var transferLen             : PInteger;
                                    var transferRowData         : PDWORD;
                                    var transferPluginDataIface : TPluginDataInterfaceAbstract;
                                    var transferActCustomData   : PDWORD ); virtual; stdcall; abstract;

    // jen pro FS s vlastnimi ikonami (pitFromPlugin):
    // Nastavi callback pro ziskani indexu jednoduchych ikon (viz
    // CPluginDataInterfaceAbstract::GetSimplePluginIcons). Pokud tento callback
    // plugin nenastavi, bude se vykreslovat vzdy jen ikona z indexu 0.
    // Z globalnich promennych callback vyuziva jen TransferFileData a TransferIsDir.
    procedure SetPluginSimpleIconCallback( callback : TGetPluginIconIndexFunc ); virtual; stdcall; abstract;

    // ------------- columns ---------------

    // vraci pocet sloupcu v panelu (vzdy minimalne jeden, protoze nazev bude vzdy zobrazen)
    function GetColumnsCount : Integer; virtual; stdcall; abstract;

    // vraci ukazatel na sloupec (pouze pro cteni)
    // 'index' udava, ktery ze sloupcu bude vracen; pokud sloupec 'index' neexistuje, vraci NULL
    function GetColumn( index : Integer ) : PColumn; virtual; stdcall; abstract;

    // Vlozi sloupec na pozici 'index'. Na pozici 0 je vzdy umisten sloupec Name,
    // pokud je zobrazen sloupec Ext, bude na pozici 1. Jinak lze sloupec umistit
    // libovolne. Struktura 'column' bude prekopirovana do vnitrnich struktur
    // Salamandera. Vraci TRUE pokud byl sloupec vlozen.
    function InsertColumn( index : Integer; column : PColumn ) : BOOL; virtual; stdcall; abstract;

    // Vlozi standardni sloupec s ID 'id' na pozici 'index'. Na pozici 0 je vzdy
    // umisten sloupec Name, pokud je vkladan sloupec Ext, musi to byt na pozici 1.
    // Jinak lze sloupec umistit libovolne. 'id' je jedna z hodnot COLUMN_ID_xxxx,
    // mimo COLUMN_ID_CUSTOM a COLUMN_ID_NAME.
    function InsertStandardColumn( index : Integer; id : DWORD ) : BOOL; virtual; stdcall; abstract;

    // Nastavi nazev a popis sloupce (nesmi byt prazdne retezce ani NULL). Delky
    // retezu se omezi na COLUMN_NAME_MAX a COLUMN_DESCRIPTION_MAX. Vraci uspech.
    // POZOR: Jmeno a popis sloupce "Name" muzou obsahovat (vzdy za prvnim
    // null-terminatorem) i jmeno a popis sloupce "Ext" - toto nastava pokud
    // neexistuje samostatny sloupec "Ext" a v datech panelu (viz
    // CSalamanderDirectoryAbstract::SetValidData()) se nastavi VALID_DATA_EXTENSION.
    // V tomto pripade je potreba nastavovat dvojite retezce (s dvoumi
    // null-terminatory) - viz CSalamanderGeneralAbstract::AddStrToStr().
    function SetColumnName( index : Integer; name : PChar; description : PChar ) : BOOL; virtual; stdcall; abstract;

    // Odstrani sloupec na pozici 'index'. Lze odstranit jak sloupce pridane pluginem,
    // tak standardni sloupce Salamandera. Nelze odstranit sloupec 'Name', ktery je vzdy
    // na indexu 0. Pozor pri odstranovani sloupce 'Ext', pokud je v datech pluginu
    // (viz CSalamanderDirectoryAbstract::SetValidData()) VALID_DATA_EXTENSION, musi
    // se jmeno+popis sloupce 'Ext' objevit u sloupce 'Name'.
    function DeleteColumn( index : Integer ) : BOOL; virtual; stdcall; abstract;
  end;

//
// ****************************************************************************
// CPluginDataInterfaceAbstract
//
// sada metod pluginu, ktere potrebuje Salamander pro ziskani specifickych dat
// pluginu do pluginem pridanych sloupcu (pracuje s CFileData::PluginData)

  TPluginDataInterfaceAbstract = class
    // vraci TRUE pokud se ma zavolat metoda ReleasePluginData pro vsechny soubory vazane
    // k tomuto interfacu, jinak vraci FALSE
    function CallReleaseForFiles : BOOL; virtual; stdcall; abstract;

    // vraci TRUE pokud se ma zavolat metoda ReleasePluginData pro vsechny adresare vazane
    // k tomuto interfacu, jinak vraci FALSE
    function CallReleaseForDirs : BOOL; virtual; stdcall; abstract;

    // uvolni data specificka pluginu (CFileData::PluginData) pro 'file' (soubor nebo
    // adresar - 'isDir' FALSE nebo TRUE; struktura vlozena do CSalamanderDirectoryAbstract
    // pri listovani archivu nebo FS); vola se pro vsechny soubory, pokud CallReleaseForFiles
    // vrati TRUE, a pro vsechny adresare, pokud CallReleaseForDirs vrati TRUE
    procedure ReleasePluginData( var _file : TFileData; isDir : BOOL ); virtual; stdcall; abstract;

    // jen pro data archivu (pro FS se nedoplnuje up-dir symbol):
    // pozmenuje navrhovany obsah up-dir symbolu (".." nahore v panelu); 'archivePath'
    // je cesta v archivu, pro kterou je symbol urcen; v 'upDir' vstupuji navrzena
    // data symbolu: jmeno ".." (nemenit), date&time archivu, zbytek nulovany;
    // v 'upDir' vystupuji zmeny pluginu, predevsim by mel zmenit 'upDir.PluginData',
    // ktery bude vyuzivan na up-dir symbolu pri ziskavani obsahu pridanych sloupcu;
    // pro 'upDir' se nebude volat ReleasePluginData, jakekoliv potrebne uvolnovani
    // je mozne provest vzdy pri dalsim volani GetFileDataForUpDir nebo pri uvolneni
    // celeho interfacu (v jeho destruktoru - volan z
    // CPluginInterfaceAbstract::ReleasePluginDataInterface)
    procedure GetFileDataForUpDir( archivePath : PChar; var upDir : TFileData ); virtual; stdcall; abstract;

    // jen pro data archivu (FS pouziva jen root cestu v CSalamanderDirectoryAbstract):
    // pri pridavani souboru/adresare do CSalamanderDirectoryAbstract se muze stat, ze
    // zadana cesta neexistuje a je ji tedy potreba vytvorit, jednotlive adresare teto
    // cesty se tvori automaticky a tato metoda umoznuje pluginu pridat sva specificka
    // data (pro sve sloupce) k temto vytvarenym adresarum; 'dirName' je plna cesta
    // pridavaneho adresare v archivu; v 'dir' vstupuji navrhovana data: jmeno adresare
    // (alokovane na heapu Salamandera), date&time prevzaty od pridavaneho souboru/adresare,
    // zbytek nulovany; v 'dir' vystupuji zmeny pluginu, predevsim by mel zmenit
    // 'dir.PluginData'; vraci TRUE pokud se pridani dat pluginu povedlo, jinak FALSE;
    // pokud vrati TRUE, bude 'dir' uvolnen klasickou cestou (Salamanderovska cast +
    // ReleasePluginData) a to bud az pri kompletnim uvolneni listingu nebo jeste behem
    // jeho tvorby v pripade, ze bude ten samy adresar pridan pomoci
    // CSalamanderDirectoryAbstract::AddDir (premazani automatickeho vytvoreni pozdejsim
    // normalnim pridanim); pokud vrati FALSE, bude z 'dir' uvolnena jen Salamanderovska cast
    function GetFileDataForNewDir( dirName : PChar; var dir : TFileData ) : BOOL; virtual; stdcall; abstract;

    // jen pro FS s vlastnimi ikonami (pitFromPlugin):
    // vraci image-list s jednoduchymi ikonami, behem kresleni polozek v panelu se
    // pomoci call-backu ziskava icon-index do tohoto image-listu; vola se vzdy po
    // ziskani noveho listingu (po volani CPluginFSInterfaceAbstract::ListCurrentPath),
    // takze je mozne image-list predelavat pro kazdy novy listing;
    // 'iconSize' urcuje pozadovanoy velikost ikon a jde o jednu z hodnot SALICONSIZE_xxx
    // destrukci image-listu si plugin zajisti pri dalsim volani GetSimplePluginIcons
    // nebo pri uvolneni celeho interfacu (v jeho destruktoru - volan z
    // CPluginInterfaceAbstract::ReleasePluginDataInterface)
    // pokud image-list nelze vytvorit, vraci NULL a aktualni plugin-icons-type
    // degraduje na pitSimple
    function GetSimplePluginIcons( iconSize : Integer ) : HIMAGELIST; virtual; stdcall; abstract;

    // jen pro FS s vlastnimi ikonami (pitFromPlugin):
    // vraci TRUE, pokud pro dany soubor/adresar ('isDir' FALSE/TRUE) 'file'
    // ma byt pouzita jednoducha ikona; vraci FALSE, pokud se ma pro ziskani ikony volat
    // z threadu pro nacitani ikon metoda GetPluginIcon (nacteni ikony "na pozadi");
    // zaroven v teto metode muze byt predpocitan icon-index pro jednoduchou ikonu
    // (u ikon ctenych "na pozadi" se az do okamziku nacteni pouzivaji take jednoduche
    // ikony) a ulozen do CFileData (nejspise do CFileData::PluginData);
    // omezeni: z CSalamanderGeneralAbstract je mozne pouzivat jen metody, ktere lze
    // volat z libovolneho threadu (metody nezavisle na stavu panelu)
    function HasSimplePluginIcon( var _file : TFileData; isDir : BOOL ) : BOOL; virtual; stdcall; abstract;

    // jen pro FS s vlastnimi ikonami (pitFromPlugin):
    // vraci ikonu pro soubor nebo adresar 'file' nebo NULL pokud ikona nelze ziskat; vraci-li
    // v 'destroyIcon' TRUE, vola se pro uvolneni vracene ikony Win32 API funkce DestroyIcon;
    // 'iconSize' urcuje velikost pozadovane ikony a jde o jednu z hodnot SALICONSIZE_xxx
    // omezeni: jelikoz se vola z threadu pro nacitani ikon (neni to hlavni thread), lze z
    // CSalamanderGeneralAbstract pouzivat jen metody, ktere lze volat z libovolneho threadu
    function GetPluginIcon( _file : PFileData; iconSize : Integer;
                            var destroyIcon : BOOL ) : HICON; virtual; stdcall; abstract;

    // jen pro FS s vlastnimi ikonami (pitFromPlugin):
    // porovna 'file1' (muze jit o soubor i adresar) a 'file2' (muze jit o soubor i adresar),
    // nesmi pro zadne dve polozky listingu vratit, ze jsou shodne (zajistuje jednoznacne
    // prirazeni vlastni ikony k souboru/adresari); pokud nehrozi duplicitni jmena v listingu
    // cesty (obvykly pripad), lze jednoduse implementovat jako:
    // {return strcmp(file1->Name, file2->Name);}
    // vraci cislo mensi nez nula pokud 'file1' < 'file2', nulu pokud 'file1' == 'file2' a
    // cislo vetsi nez nula pokud 'file1' > 'file2';
    // omezeni: jelikoz se vola i z threadu pro nacitani ikon (nejen z hlavniho threadu), lze
    // z CSalamanderGeneralAbstract pouzivat jen metody, ktere lze volat z libovolneho threadu
    function CompareFilesFromFS( file1 : PFileData; file2 : PFileData ) : Integer; virtual; stdcall; abstract;

    // slouzi k nastaveni parametru pohledu, tato metoda je zavolana vzdy pred zobrazenim noveho
    // obsahu panelu (pri zmene cesty) a pri zmene aktualniho pohledu (i rucni zmena sirky
    // sloupce); 'leftPanel' je TRUE pokud jde o levy panel (FALSE pokud jde o pravy panel);
    // 'view' je interface pro modifikaci pohledu (nastaveni rezimu, prace se
    // sloupci); jde-li o data archivu, obsahuje 'archivePath' soucasnou cestu v archivu,
    // pro data FS je 'archivePath' NULL; jde-li o data archivu, je 'upperDir' ukazatel na
    // nadrazeny adresar (je-li soucasna cesta root archivu, je 'upperDir' NULL), pro data
    // FS je vzdy NULL;
    // POZOR: behem volani teto metody nesmi dojit k prekresleni panelu (muze se zde zmenit
    //        velikost ikon, atd.), takze zadne messageloopy (zadne dialogy, atd.)!
    // omezeni: z CSalamanderGeneralAbstract je mozne pouzivat jen metody, ktere lze
    //          volat z libovolneho threadu (metody nezavisle na stavu panelu)
    procedure SetupView( leftPanel : BOOL; view : TSalamanderViewAbstract;
                         archivePath : PChar; upperDir : PFileData ); virtual; stdcall; abstract;

    // nastaveni nove hodnoty "column->FixedWidth" - uzivatel pouzil kontextove menu
    // na pluginem pridanem sloupci v header-line > "Automatic Column Width"; plugin
    // by si mel ulozit novou hodnotu column->FixedWidth ulozenou v 'newFixedWidth'
    // (je to vzdy negace column->FixedWidth), aby pri nasledujicich volanich SetupView() mohl
    // sloupec pridat uz se spravne nastavenou FixedWidth; zaroven pokud se zapina pevna
    // sirka sloupce, mel by si plugin nastavit soucasnou hodnotu "column->Width" (aby
    // se timto zapnutim pevne sirky nezmenila sirka sloupce) - idealni je zavolat
    // "ColumnWidthWasChanged(leftPanel, column, column->Width)"; 'column' identifikuje
    // sloupec, ktery se ma zmenit; 'leftPanel' je TRUE pokud jde o sloupec z leveho
    // panelu (FALSE pokud jde o sloupec z praveho panelu)
    procedure ColumnFixedWidthShouldChange( leftPanel : BOOL; column : PColumn;
                                            newFixedWidth : Integer ); virtual; stdcall; abstract;

    // nastaveni nove hodnoty "column->Width" - uzivatel mysi zmenil sirku pluginem pridaneho
    // sloupce v header-line; plugin by si mel ulozit novou hodnotu column->Width (je ulozena
    // i v 'newWidth'), aby pri nasledujicich volanich SetupView() mohl sloupec pridat uz se
    // spravne nastavenou Width; 'column' identifikuje sloupec, ktery se zmenil; 'leftPanel'
    // je TRUE pokud jde o sloupec z leveho panelu (FALSE pokud jde o sloupec z praveho panelu)
    procedure ColumnWidthWasChanged( leftPanel : BOOL; column : PColumn;
                                     newWidth : Integer ); virtual; stdcall; abstract;

    // ziska obsah Information Line pro soubor/adresar ('isDir' TRUE/FALSE) 'file'
    // nebo oznacene soubory a adresare ('file' je NULL a pocty oznacenych souboru/adresaru
    // jsou v 'selectedFiles'/'selectedDirs') v panelu ('panel' je jeden z PANEL_XXX);
    // je-li 'displaySize' TRUE, je znama velikost vsech oznacenych adresaru (viz
    // CFileData::SizeValid; pokud neni nic oznaceneho, je zde TRUE); v 'selectedSize' je
    // soucet cisel CFileData::Size oznacenych souboru a adresaru (pokud neni nic oznaceneho,
    // je zde nula); 'buffer' je buffer pro vraceny text (velikost 1000 bytu); 'hotTexts'
    // je pole (velikost 100 DWORDu), ve kterem se vraci informace o poloze hot-textu, vzdy
    // spodni WORD obsahuje pozici hot-textu v 'buffer', horni WORD obsahuje delku hot-textu;
    // v 'hotTextsCount' se vraci pocet zapsanych hot-textu v poli 'hotTexts'; vraci TRUE
    // pokud je 'buffer' + 'hotTexts' + 'hotTextsCount' nastaveno, vraci FALSE pokud se
    // ma Information Line plnit standardnim zpusobem (jako na disku)
    function GetInfoLineContent( panel : Integer; _file : PFileData; isDir : BOOL;
                                 selectedFiles : Integer; selectedDirs : Integer;
                                 displaySize : BOOL; var selectedSize : TLargeInteger;
                                 buffer : PChar; hotTexts : PDWORD;
                                 var hotTextsCount : Integer ) : BOOL; virtual; stdcall; abstract;

    // jen pro archivy: uzivatel ulozil soubory/adresare z archivu na clipboard, ted zavira
    // archiv v panelu: pokud metoda vrati TRUE, tento objekt zustane otevreny (optimalizace
    // pripadneho Paste z clipboardu - archiv uz je vylistovany), pokud metoda vrati FALSE,
    // tento objekt se uvolni (pripadny Paste z clipboardu zpusobi listovani archivu, pak
    // teprve dojde k vybaleni vybranych souboru/adresaru); POZNAMKA: pokud je po zivotnost
    // objektu otevreny soubor archivu, metoda by mela vracet FALSE, jinak bude po celou
    // dobu "pobytu" dat na clipboardu soubor archivu otevreny (nepujde smazat, atd.)
    function CanBeCopiedToClipboard : BOOL; virtual; stdcall; abstract;

    // jen pri zadani VALID_DATA_PL_SIZE do CSalamanderDirectoryAbstract::SetValidData():
    // vraci TRUE pokud je velikost souboru/adresare ('isDir' TRUE/FALSE) 'file' znama,
    // jinak vraci FALSE; velikost vraci v 'size'
    function GetByteSize( _file : PFileData; isDir : BOOL; size : PLargeInteger ) : BOOL; virtual; stdcall; abstract;

    // jen pri zadani VALID_DATA_PL_DATE do CSalamanderDirectoryAbstract::SetValidData():
    // vraci TRUE pokud je datum souboru/adresare ('isDir' TRUE/FALSE) 'file' znamy,
    // jinak vraci FALSE; datum vraci v "datumove" casti struktury 'date' ("casova" cast
    // by mela zustat netknuta)
    function GetLastWriteDate( _file : PFileData; isDir : BOOL; date : PSystemTime ) : BOOL; virtual; stdcall; abstract;

    // jen pri zadani VALID_DATA_PL_TIME do CSalamanderDirectoryAbstract::SetValidData():
    // vraci TRUE pokud je cas souboru/adresare ('isDir' TRUE/FALSE) 'file' znamy,
    // jinak vraci FALSE; cas vraci v "casove" casti struktury 'time' ("datumova" cast
    // by mela zustat netknuta)
    function GetLastWriteTime( _file : PFileData; isDir : BOOL; time : PSystemTime ) : BOOL; virtual; stdcall; abstract;
  end;

//
// ****************************************************************************
// CSalamanderForOperationsAbstract
//
// sada metod ze Salamandera pro podporu provadeni operaci, platnost interfacu je
// omezena na metodu, ktere je interface predan jako parametr; tedy lze volat pouze
// z tohoto threadu a v teto metode (objekt je na stacku, takze po navratu zanika)

  TSalamanderForOperationsAbstract = class
    // PROGRESS DIALOG: dialog obsahuje jeden/dva ('twoProgressBars' FALSE/TRUE) progress-metry
    // otevre progress-dialog s titulkem 'title'; 'parent' je parent okno progress-dialogu (je-li
    // NULL, pouzije se hlavni okno); pokud obsahuje jen jeden progress-metr, muze byt popsan
    // jako "File" ('fileProgress' je TRUE) nebo "Total" ('fileProgress' je FALSE)
    //
    // dialog nebezi ve vlastnim threadu; pro jeho fungovani (tlacitko Cancel + vnitrni timer)
    // je treba obcas vyprazdni message queue; to zajistuji metody ProgressDialogAddText,
    // ProgressAddSize a ProgressSetSize
    //
    // protoze real-time zobrazovani textu a zmen v progress bare silne zdrzuje, maji
    // metody ProgressDialogAddText, ProgressAddSize a ProgressSetSize parametr
    // 'delayedPaint'; ten by mel byt TRUE pro vsechny rychle se menici texty a hodnoty;
    // metody si pak ulozi texty a zobrazi je az po doruceni vnitrniho timeru dialogu;
    // 'delayedPaint' nastavime na FALSE pro inicializacni/koncove texty typu "preparing data..."
    // nebo "canceling operation...", po jejiz zobrazeni nedame dialogu prilezitost k distribuci
    // zprav (timeru); pokud je u takove operace pravdepodobne, ze bude trvat dlouho, meli
    // bychom behem teto doby dialog "obcerstvovat" volanim ProgressAddSize(CQuadWord(0, 0), TRUE)
    // a podle jeji navratove hodnoty akci pripadne predcasne ukoncit
    procedure OpenProgressDialog( title : PChar; twoProgressBars : BOOL;
                                  parent : HWND; fileProgress : BOOL ); virtual; stdcall; abstract;
    // vypise text 'txt' (i nekolik radku - provadi se rozpad na radky) do progress-dialogu
    procedure ProgressDialogAddText( txt : PChar; delayedPaint : BOOL ); virtual; stdcall; abstract;
    // neni-li 'totalSize1' CQuadWord(-1, -1), nastavi 'totalSize1' jako 100 procent prvniho progress-metru,
    // neni-li 'totalSize2' CQuadWord(-1, -1), nastavi 'totalSize2' jako 100 procent druheho progress-metru
    // (pro progress-dialog s jednim progress-metrem je povinne 'totalSize2' CQuadWord(-1, -1))
    procedure ProgressSetTotalSize( var totalSize1 : TLargeInteger;
                                    var totalSize2 : TLargeInteger ); virtual; stdcall; abstract;
    // neni-li 'size1' CQuadWord(-1, -1), nastavi velikost 'size1' (size1/total1*100 procent) na prvnim progress-metru,
    // neni-li 'size2' CQuadWord(-1, -1), nastavi velikost 'size2' (size2/total2*100 procent) na druhem progress-metru
    // (pro progress-dialog s jednim progress-metrem je povinne 'size2' CQuadWord(-1, -1)), vraci informaci jestli ma
    // akce pokracovat (FALSE = konec)
    function ProgressSetSize( var size1 : TLargeInteger; var size2 : TLargeInteger;
                              delayedPaint : BOOL ) : BOOL; virtual; stdcall; abstract;
    // prida (pripadne k oboum progress-metrum) velikost 'size' (size/total*100 procent progressu),
    // vraci informaci jestli ma akce pokracovat (FALSE = konec)
    function ProgressAddSize( size : Integer; delayedPaint : BOOL ) : BOOL; virtual; stdcall; abstract;
    // enabluje/disabluje tlacitko Cancel
    procedure ProgressEnableCancel( enable : BOOL ); virtual; stdcall; abstract;
    // vraci HWND dialogu progressu (hodi se pri vypisu chyb a dotazu pri otevrenem progress-dialogu)
    function ProgressGetHWND : HWND; virtual; stdcall; abstract;
    // zavre progress-dialog
    procedure CloseProgressDialog; virtual; stdcall; abstract;

    // presune vsechny soubory ze 'source' adresare do 'target' adresare,
    // navic premapovava predpony zobrazovanych jmen ('remapNameFrom' -> 'remapNameTo')
    // vraci uspech operace
    function MoveFiles( source : PChar; target : PChar; remapNameFrom : PChar;
                        remapNameTo : PChar ) : BOOL; virtual; stdcall; abstract;
  end;

implementation

end.
