(*
  Object Pascal translation of the Altap Salamander SDK library.
  Original source: spl_menu.h

  Portions created by ALTAP are Copyright (c) ALTAP, spol. s r.o.
  All rights reserved.

  Portions created by LostGenius Posse are Copyright (c) 2007, LostGenius Posse (Timo 'jomanto' Boll)
  All rights reserved.

  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

      * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
      * Redistributions in binary form may reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
      * Neither the name of the LostGenius Posse nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*)
(*
  History:

  2007-05-31: initial version
*)
unit spl_menu;

interface

uses
  winliblt,
  spl_com;

const
//
// ****************************************************************************
// CPluginInterfaceForMenuExtAbstract
//

  // flagy stavu polozek v menu (pro pluginy rozsireni menu)
  MENU_ITEM_STATE_ENABLED   = $01;                                              // enablovana, bez tohoto flagu je polozka disablovana
  MENU_ITEM_STATE_CHECKED   = $02;                                              // pred polozkou je "check" nebo "radio" znacka
  MENU_ITEM_STATE_RADIO     = $04;                                              // bez MENU_ITEM_STATE_CHECKED se ignoruje,
                                                                                // "radio" znacka, bez tohoto flagu "check" znacka
  MENU_ITEM_STATE_HIDDEN    = $08;                                              // polozka se v menu vubec nema objevit

type
//
// ****************************************************************************
// CPluginInterfaceForMenuExtAbstract
//

  TPluginInterfaceForMenuExtAbstract = class
    // vraci stav polozky menu s identifikacnim cislem 'id'; navratova hodnota je kombinaci
    // flagu (viz MENU_ITEM_STATE_XXX); 'eventMask' viz CSalamanderConnectAbstract::AddMenuItem
    function GetMenuItemState( id : Integer; eventMask : DWORD ) : DWORD; virtual; stdcall; abstract;

    // spousti prikaz menu s identifikacnim cislem 'id', 'eventMask' viz
    // CSalamanderConnectAbstract::AddMenuItem, 'salamander' je sada pouzitelnych metod
    // Salamandera pro provadeni operaci, 'parent' je parent messageboxu, vraci TRUE pokud
    // ma byt v panelu zruseno oznaceni (nebyl pouzit Cancel, mohl byt pouzit Skip), jinak
    // vraci FALSE (neprovede se odznaceni);
    // POZOR: Pokud prikaz zpusobi zmeny na nejake ceste (diskove/FS), mel by pouzit
    //        CSalamanderGeneralAbstract::PostChangeOnPathNotification pro informovani
    //        panelu bez automatickeho refreshe a otevrene FS (aktivni i odpojene)
    // POZNAMKA: pokud prikaz pracuje se soubory/adresari z cesty v aktualnim panelu nebo
    //           i primo s touto cestou, je treba volat
    //           CSalamanderGeneralAbstract::SetUserWorkedOnPanelPath pro aktualni panel,
    //           jinak nebude cesta v tomto panelu vlozena do seznamu pracovnich
    //           adresaru - List of Working Directories (Alt+F12)
    function ExecuteMenuItem( salamander : TSalamanderForOperationsAbstract; parent : HWND;
                              id : Integer; eventMask : DWORD ) : BOOL; virtual; stdcall; abstract;

    // zobrazi napovedu pro prikaz menu s identifikacnim cislem 'id' (user stiskne Shift+F1,
    // najde v menu Plugins menu tohoto pluginu a vybere z nej prikaz), 'parent' je parent
    // messageboxu, vraci TRUE pokud byla zobrazena nejaka napoveda, jinak se zobrazi z helpu
    // Salamandera kapitola "Using Plugins"
    function HelpForMenuItem( parent : HWND; id : Integer ) : BOOL; virtual; stdcall; abstract;
  end;

implementation

end.
