(*
  Object Pascal translation of the Altap Salamander SDK library.
  Original source: spl_thum.h

  Portions created by ALTAP are Copyright (c) ALTAP, spol. s r.o.
  All rights reserved.

  Portions created by LostGenius Posse are Copyright (c) 2007, LostGenius Posse (Timo 'jomanto' Boll)
  All rights reserved.

  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

      * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
      * Redistributions in binary form may reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
      * Neither the name of the LostGenius Posse nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*)
(*
  History:

  2007-05-31: initial version
*)
unit spl_thum;

interface

uses
  winliblt;

const
//
// ****************************************************************************
// CSalamanderThumbnailMakerAbstract
//

  // informace o obrazku, ze ktereho generujeme thumbnail, tyto flagy se pouzivaji
  // v CSalamanderThumbnailMakerAbstract::SetParameters():
  SSTHUMB_MIRROR_HOR   = 1;                                                     // obrazek je potreba horizontalne zrcadlit
  SSTHUMB_MIRROR_VERT  = 2;                                                     // obrazek je potreba vertikalne zrcadlit
  SSTHUMB_ROTATE_90CW  = 4;                                                     // obrazek je potreba otocit o 90 stupnu ve smeru hodinovych rucicek
  SSTHUMB_ROTATE_180   = (SSTHUMB_MIRROR_VERT or SSTHUMB_MIRROR_HOR);           // obrazek je potreba otocit o 180 stupnu
  SSTHUMB_ROTATE_90CCW = (SSTHUMB_ROTATE_90CW or SSTHUMB_ROTATE_180);           // obrazek je potreba otocit o 90 stupnu proti smeru hodinovych rucicek
  // obrazek je v horsi kvalite nebo mensi nez je potreba, Salamander po dokonceni prvniho kola
  // ziskavani "rychlych" thumbnailu zkusi pro tento obrazek ziskat "kvalitni" thumbnail
  SSTHUMB_ONLY_PREVIEW = 8;

type
//
// ****************************************************************************
// CSalamanderThumbnailMakerAbstract
//

  TSalamanderThumbnailMakerAbstract = class
    // nastaveni parametru zpracovani obrazku pri tvorbe thumbnailu; nutne volat
    // jako prvni metodu tohoto rozhrani; 'picWidth' a 'picHeight' jsou rozmery
    // zpracovavaneho obrazku (v bodech); 'flags' je kombinace flagu SSTHUMB_XXX,
    // ktera udava informace o obrazku predavanem v parametru 'buffer' v metode
    // ProcessBuffer; vraci TRUE, pokud se podarilo alokovat buffery pro zmensovani
    // a je mozne nasledne volat ProcessBuffer; pokud vrati FALSE, doslo k chybe
    // a je treba ukoncit nacitani thumbnailu
    function SetParameters( picWidth : Integer; picHeight : Integer; flags : DWORD ) : BOOL; virtual; stdcall; abstract;

    // zpracuje cast obrazku v bufferu 'buffer' (zpracovavana cast obrazku je ulozena
    // po radcich shora dolu, body na radcich jsou ulozeny zleva doprava, kazdy bod
    // reprezentuje 32-bitova hodnota, ktera je slozena z tri bytu s barvami R+G+B a
    // ctvrteho bytu, ktery se ignoruje); rozlisujeme dva typy zpracovani: kopie
    // obrazku do vysledneho thumbnailu (nepresahuje-li velikost zpracovavaneho obrazku
    // velikost thumbnailu) a zmenseni obrazku do thumbnailu (obrazek vetsi nez
    // thumbnail); 'buffer' je pouzit pouze pro cteni; 'rowsCount' urcuje kolik radku
    // obrazku je v bufferu;
    // je-li'buffer' NULL, berou se data z vlastniho bufferu (plugin ziska pres GetBuffer);
    // vraci TRUE pokud ma plugin pokracovat s nacitanim obrazku, vraci-li FALSE,
    // tvorba thumbnailu je hotova (byl zpracovan cely obrazek) nebo se ma co
    // nejdrive prerusit (napr. uzivatel zmenil cestu v panelu, thumbnail tedy jiz
    // neni potreba)
    //
    // POZOR: pokud je spustena metoda CPluginInterfaceForThumbLoader::LoadThumbnail,
    // je blokovana zmena cesty v panelu. Z toho duvodu je treba predavat a hlavne
    // nacitat vetsi obrazky po castech a testovatovanim navratove hodnoty
    // metody ProcessBuffer overovat, zda se nacitani nema prerusit.
    // Pokud je treba provest casove narocnejsi operace pred volanim metody SetParameters
    // nebo pred volanim ProcessBuffer, je behem teto doby nutne obcas volat GetCancelProcessing.
    function ProcessBuffer( buffer : Pointer; rowsCount : Integer ) : BOOL; virtual; stdcall; abstract;

    // vraci vlastni buffer o velikosti potrebne k ulozeni 'rowsCount' radku obrazku
    // (4 * 'rowsCount' * 'picWidth' bytu); pri nedostatku pameti nebo je-li objekt
    // v chybovem stavu (po volani SetError) vraci NULL;
    // plugin nesmi dealokovat ziskany buffer (dealokuje se v Salamanderovi automaticky)
    function GetBuffer( rowsCount : Integer ) : Pointer; virtual; stdcall; abstract;

    // oznameni chyby pri ziskavani obrazku (thumbnail je povazovan za vadny
    // a nepouzije se), ostatni metody tohoto rozhrani budou od chvile volani
    // SetError uz jen vracet chyby (GetBuffer a SetParameters) nebo preruseni
    // prace (ProcessBuffer)
    procedure SetError; virtual; stdcall; abstract;

    // vraci TRUE, pokud ma plugin preprusit nacitani thumbnailu
    // vraci FALSE, pokud ma plugin pokracovat s nacitanim obrazku
    //
    // metodu lze volat pred i po volani metody SetParameters
    //
    // slouzi k detekci pozadavku na preruseni v pripadech, kdy plugin
    // potrebuje vykonat casove narocne operace jeste pred volanim SetParameters
    // nebo v pripade, kdy plugin potrebuje obrazek predrenderovat, tedy po volani
    // SetParameters, ale pred volanim ProcessBuffer
    function GetCancelProcessing : BOOL; virtual; stdcall; abstract;
  end;

//
// ****************************************************************************
// CPluginInterfaceForThumbLoaderAbstract
//

  TPluginInterfaceForThumbLoaderAbstract = class
    // nacte thumbnail pro soubor 'filename'; 'thumbWidth' a 'thumbHeight' jsou
    // rozmery pozadovaneho thumbnailu; 'thumbMaker' je rozhrani algoritmu pro
    // tvorbu thumbnailu (umi prijmout hotovy thumbnail nebo ho vyrobit zmensenim
    // obrazku); vraci TRUE pokud je format souboru 'filename' znamy, pokud vrati
    // FALSE, Salamander zkusi nacist thumbnail pomoci jineho pluginu; chybu pri
    // ziskavani thumbnailu (napr. chybu cteni souboru nebo nedostatek pameti)
    // plugin hlasi pomoci rozhrani 'thumbMaker' - viz metoda SetError;
    // 'fastThumbnail' je TRUE v prvnim kole cteni thumbnailu - cilem je vratit
    // thumbnail co nejrychleji (klidne v horsi kvalite nebo mensi nez je potreba),
    // v druhem kole cteni thumbnailu (jen pokud se v prvnim kole nastavi flag
    // SSTHUMB_ONLY_PREVIEW) je 'fastThumbnail' FALSE - cilem je vratit kvalitni thumbnail
    // omezeni: jelikoz se vola z threadu pro nacitani ikon (neni to hlavni thread), lze z
    // CSalamanderGeneralAbstract pouzivat jen metody, ktere lze volat z libovolneho threadu
    //
    // Doporucene schema implementace:
    //   - pokusit se otevrit obrazek
    //   - pokud se nepodari, vratit FALSE
    //   - extrahovat rozmery obrazku
    //   - predat je do Salamandera pres thumbMaker->SetParameters
    //   - pokud vrati FALSE, uklid a odchod (nepovedlo se alokovat buffery)
    //   - SMYCKA
    //     - nacist cast dat z obrazku
    //     - poslat je do Salamandera pres thumbMaker->ProcessBuffer
    //     - pokud vrati FALSE, uklid a odchod (preruseni z duvodu zmeny cesty)
    //     - pokracovat ve SMYCCE, dokud nebude cely obrazek predan
    //   - uklid a odchod
    function LoadThumbnail( filename : PChar; thumbWidth : Integer; thumbHeight : Integer;
                            thumbMaker : TSalamanderThumbnailMakerAbstract; fastThumbnail : BOOL ) : BOOL; virtual; stdcall; abstract;
  end;

implementation

end.
