(*
  Object Pascal translation of the Altap Salamander SDK library.
  Original source: spl_view.h

  Portions created by ALTAP are Copyright (c) ALTAP, spol. s r.o.
  All rights reserved.

  Portions created by LostGenius Posse are Copyright (c) 2007, LostGenius Posse (Timo 'jomanto' Boll)
  All rights reserved.

  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

      * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
      * Redistributions in binary form may reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
      * Neither the name of the LostGenius Posse nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*)
(*
  History:

  2007-05-31: initial version
*)
unit spl_view;

interface

uses
  winliblt,
  spl_gen;

type
//
// ****************************************************************************
// CPluginInterfaceForViewerAbstract
//

  TPluginInterfaceForViewerAbstract = class
    // funkce pro "file viewer", vola se pri pozadavku na otevreni viewru a nacteni souboru
    // 'name', 'left'+'right'+'width'+'height'+'showCmd'+'alwaysOnTop' je doporucene umisteni
    // okna, je-li 'returnLock' FALSE nemaji 'lock'+'lockOwner' zadny vyznam, je-li 'returnLock'
    // TRUE, mel by viewer vratit system-event 'lock' v nonsignaled stavu, do signaled stavu 'lock'
    // prejde v okamziku ukonceni prohlizeni souboru 'name' (soubor je v tomto okamziku odstranen
    // z docasneho adresare), dale by mel vratit v 'lockOwner' TRUE pokud ma byt objekt 'lock' uzavren
    // volajicim (FALSE znamena, ze si viewer 'lock' rusi sam - v tomto pripade viewer musi pro
    // prechod 'lock' do signaled stavu pouzit metodu CSalamanderGeneralAbstract::UnlockFileInCache);
    // pokud viewer nenastavi 'lock' (zustava NULL) je soubor 'name' platny jen do ukonceni volani teto
    // metody ViewFile; neni-li 'viewerData' NULL, jde o predani rozsirenych parametru viewru (viz
    // CSalamanderGeneralAbstract::ViewFileInPluginViewer); 'enumFilesSourceUID' je UID zdroje (panelu
    // nebo Find okna), ze ktereho je viewer otviran, je-li -1, je zdroj neznamy (archivy a
    // file_systemy nebo Alt+F11, atd.) - viz napr. CSalamanderGeneralAbstract::GetNextFileNameForViewer;
    // 'enumFilesCurrentIndex' je index oteviraneho souboru ve zdroji (panelu nebo Find okne), je-li -1,
    // neni zdroj nebo index znamy; vraci TRUE pri uspechu (FALSE znamena neuspech, 'lock' a
    // 'lockOwner' v tomto pripade nemaji zadny vyznam)
    function ViewFile( name : PChar; left : Integer; top : Integer; width : Integer;
                       height : Integer; showCmd : UINT; alwaysOnTop : BOOL;
                       returnLock : BOOL; lock : PHandle; lockOwner : PBOOL;
                       viewerData : PSalamanderPluginViewerData; enumFilesSourceUID : Integer;
                       enumFilesCurrentIndex : Integer ) : BOOL; virtual; stdcall; abstract;

    // funkce pro "file viewer", vola se pri pozadavku na otevreni viewru a nacteni souboru
    // 'name'; tato fuknce by nemela zobrazovat zadna okna typu "invalid file format", tato
    // okna se zobrazi az pri volani metody ViewFile tohoto rozhrani; zjisti jestli je
    // soubor 'name' zobrazitelny (napr. soubor ma odpovidajici signaturu) ve vieweru
    // a pokud je, vraci TRUE; pokud vrati FALSE, zkusi Salamander pro 'name' najit jiny
    // viewer (v prioritnim seznamu vieweru, viz konfiguracni stranka Viewers)
    function CanViewFile( name : PChar ) : BOOL; virtual; stdcall; abstract;
  end;

implementation

end.
