(*
  Object Pascal translation of the Altap Salamander SDK library.
  Original source: spl_xpth.h

  Portions created by ALTAP are Copyright (c) ALTAP, spol. s r.o.
  All rights reserved.

  Portions created by LostGenius Posse are Copyright (c) 2007, LostGenius Posse (Timo 'jomanto' Boll)
  All rights reserved.

  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

      * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
      * Redistributions in binary form may reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
      * Neither the name of the LostGenius Posse nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*)
(*
  History:

  2007-05-31: initial version
*)
unit spl_xpth;

interface

uses
  winliblt;

type
//
// ****************************************************************************
// CXpThemeSupportAbstract
//
// metody volaji dynamicky nactenou knihovnu UXTHEME.DLL

  TXpThemeSupportAbstract = class
    //---------------------------------------------------------------------------
    //  OpenThemeData()     - Open the theme data for the specified HWND and
    //                        semi-colon separated list of class names.
    //
    //                        OpenThemeData() will try each class name, one at
    //                        a time, and use the first matching theme info
    //                        found.  If a match is found, a theme handle
    //                        to the data is returned.  If no match is found,
    //                        a "NULL" handle is returned.
    //
    //                        When the window is destroyed or a WM_THEMECHANGED
    //                        msg is received, "CloseThemeData()" should be
    //                        called to close the theme handle.
    //
    //  hwnd                - window handle of the control/window to be themed
    //
    //  pszClassList        - class name (or list of names) to match to theme data
    //                        section.  if the list contains more than one name,
    //                        the names are tested one at a time for a match.
    //                        If a match is found, OpenThemeData() returns a
    //                        theme handle associated with the matching class.
    //                        This param is a list (instead of just a single
    //                        class name) to provide the class an opportunity
    //                        to get the "best" match between the class and
    //                        the current theme.  For example, a button might
    //                        pass L"OkButton, Button" if its ID=ID_OK.  If
    //                        the current theme has an entry for OkButton,
    //                        that will be used.  Otherwise, we fall back on
    //                        the normal Button entry.
    //---------------------------------------------------------------------------
    function OpenThemeData( hwnd : HWND; pszClassList : LPCWSTR ) : HTHEME; virtual; stdcall; abstract;

    //---------------------------------------------------------------------------
    //  CloseTHemeData()    - closes the theme data handle.  This should be done
    //                        when the window being themed is destroyed or
    //                        whenever a WM_THEMECHANGED msg is received
    //                        (followed by an attempt to create a new Theme data
    //                        handle).
    //
    //  hTheme              - open theme data handle (returned from prior call
    //                        to OpenThemeData() API).
    //---------------------------------------------------------------------------
    function CloseThemeData( hTheme : HTHEME ) : HRESULT; virtual; stdcall; abstract;

    //---------------------------------------------------------------------------
    //    functions for basic drawing support
    //---------------------------------------------------------------------------
    // The following methods are the theme-aware drawing services.
    // Controls/Windows are defined in drawable "parts" by their author: a
    // parent part and 0 or more child parts.  Each of the parts can be
    // described in "states" (ex: disabled, hot, pressed).
    //---------------------------------------------------------------------------
    // For the list of all themed classes and the definition of all
    // parts and states, see the file "tmschmea.h".
    //---------------------------------------------------------------------------
    // Each of the below methods takes a "iPartId" param to specify the
    // part and a "iStateId" to specify the state of the part.
    // "iStateId=0" refers to the root part.  "iPartId" = "0" refers to
    // the root class.
    //-----------------------------------------------------------------------
    // Note: draw operations are always scaled to fit (and not to exceed)
    // the specified "Rect".
    //-----------------------------------------------------------------------

    //------------------------------------------------------------------------
    //  DrawThemeBackground()
    //                      - draws the theme-specified border and fill for
    //                        the "iPartId" and "iStateId".  This could be
    //                        based on a bitmap file, a border and fill, or
    //                        other image description.
    //
    //  hTheme              - theme data handle
    //  hdc                 - HDC to draw into
    //  iPartId             - part number to draw
    //  iStateId            - state number (of the part) to draw
    //  pRect               - defines the size/location of the part
    //  pClipRect           - optional clipping rect (don't draw outside it)
    //------------------------------------------------------------------------
    function DrawThemeBackground( hTheme : HTHEME; hdc : HDC; iPartId : Integer;
                                  iStateId : Integer; pRect : PRect; pClipRect : PRect ) : HRESULT; virtual; stdcall; abstract;

    //-------------------------------------------------------------------------
    //  DrawThemeText()     - draws the text using the theme-specified
    //                        color and font for the "iPartId" and
    //                        "iStateId".
    //
    //  hTheme              - theme data handle
    //  hdc                 - HDC to draw into
    //  iPartId             - part number to draw
    //  iStateId            - state number (of the part) to draw
    //  pszText             - actual text to draw
    //  dwCharCount         - number of chars to draw (-1 for all)
    //  dwTextFlags         - same as DrawText() "uFormat" param
    //  dwTextFlags2        - additional drawing options
    //  pRect               - defines the size/location of the part
    //-------------------------------------------------------------------------
    function DrawThemeText( hTheme : HTHEME; hdc : HDC; iPartId : Integer;
                            iStateId : Integer; pszText : LPCWSTR; iCharCount : Integer;
                            dwTextFlags : DWORD; dwTextFlags2 : DWORD;
                            pRect : PRect ) : HRESULT; virtual; stdcall; abstract;

    //-------------------------------------------------------------------------
    //  GetThemeBackgroundContentRect()
    //                      - gets the size of the content for the theme-defined
    //                        background.  This is usually the area inside
    //                        the borders or Margins.
    //
    //      hTheme          - theme data handle
    //      hdc             - (optional) device content to be used for drawing
    //      iPartId         - part number to draw
    //      iStateId        - state number (of the part) to draw
    //      pBoundingRect   - the outer RECT of the part being drawn
    //      pContentRect    - RECT to receive the content area
    //-------------------------------------------------------------------------
    function GetThemeBackgroundContentRect( hTheme : HTHEME; hdc : HDC; iPartId : Integer;
                                            iStateId : Integer; pBoundingRect : PRect;
                                            out pContentRect : PRect ) : HRESULT; virtual; stdcall; abstract;

    //-------------------------------------------------------------------------
    //  GetThemeBackgroundExtent() - calculates the size/location of the theme-
    //                               specified background based on the
    //                               "pContentRect".
    //
    //      hTheme          - theme data handle
    //      hdc             - (optional) device content to be used for drawing
    //      iPartId         - part number to draw
    //      iStateId        - state number (of the part) to draw
    //      pContentRect    - RECT that defines the content area
    //      pBoundingRect   - RECT to receive the overall size/location of part
    //-------------------------------------------------------------------------
    function GetThemeBackgroundExtent( hTheme : HTHEME; hdc : HDC; iPartId : Integer;
                                       iStateId : Integer; pContentRect : PRect;
                                       out pExtentRect : PRect ) : HRESULT; virtual; stdcall; abstract;

    //-------------------------------------------------------------------------
    //  GetThemePartSize() - returns the specified size of the theme part
    //
    //  hTheme              - theme data handle
    //  hdc                 - HDC to select font into & measure against
    //  iPartId             - part number to retrieve size for
    //  iStateId            - state number (of the part)
    //  prc                 - (optional) rect for part drawing destination
    //  eSize               - the type of size to be retreived
    //  psz                 - receives the specified size of the part
    //-------------------------------------------------------------------------
    function GetThemePartSize( hTheme : HTHEME; hdc : HDC; iPartId : Integer;
                               iStateId : Integer; prc : PRect; eSize : TThemeSize;
                               out psz : PSize ) : HRESULT; virtual; stdcall; abstract;


    //-------------------------------------------------------------------------
    //  GetThemeTextExtent() - calculates the size/location of the specified
    //                         text when rendered in the Theme Font.
    //
    //  hTheme              - theme data handle
    //  hdc                 - HDC to select font & measure into
    //  iPartId             - part number to draw
    //  iStateId            - state number (of the part)
    //  pszText             - the text to be measured
    //  dwCharCount         - number of chars to draw (-1 for all)
    //  dwTextFlags         - same as DrawText() "uFormat" param
    //  pszBoundingRect     - optional: to control layout of text
    //  pszExtentRect       - receives the RECT for text size/location
    //-------------------------------------------------------------------------
    function GetThemeTextExtent( hTheme : HTHEME; hdc : HDC; iPartId : Integer;
                                 iStateId : Integer; pszText : LPCWSTR; iCharCount : Integer;
                                 dwTextFlags : DWORD; pBoundingRect : PRect;
                                 out pExtentRect : PRect ) : HRESULT; virtual; stdcall; abstract;

    //-------------------------------------------------------------------------
    //  GetThemeTextMetrics()
    //                      - returns info about the theme-specified font
    //                        for the part/state passed in.
    //
    //  hTheme              - theme data handle
    //  hdc                 - optional: HDC for screen context
    //  iPartId             - part number to draw
    //  iStateId            - state number (of the part)
    //  ptm                 - receives the font info
    //-------------------------------------------------------------------------
    function GetThemeTextMetrics( hTheme : HTHEME; hdc : HDC; iPartId : Integer;
                                  iStateId : Integer; ptm : PTextMetric ) : HRESULT; virtual; stdcall; abstract;

    //-------------------------------------------------------------------------
    //  GetThemeBackgroundRegion()
    //                      - computes the region for a regular or partially
    //                        transparent theme-specified background that is
    //                        bound by the specified "pRect".
    //                        If the rectangle is empty, sets the HRGN to NULL
    //                        and return S_FALSE.
    //
    //  hTheme              - theme data handle
    //  hdc                 - optional HDC to draw into (DPI scaling)
    //  iPartId             - part number to draw
    //  iStateId            - state number (of the part)
    //  pRect               - the RECT used to draw the part
    //  pRegion             - receives handle to calculated region
    //-------------------------------------------------------------------------
    function GetThemeBackgroundRegion( hTheme : HTHEME; hdc : HDC; iPartId : Integer;
                                       iStateId : Integer; pRect : PRect;
                                       out pRegion : PHRGN ) : HRESULT; virtual; stdcall; abstract;

    //-------------------------------------------------------------------------
    //  HitTestThemeBackground()
    //                      - returns a HitTestCode (a subset of the values
    //                        returned by WM_NCHITTEST) for the point "ptTest"
    //                        within the theme-specified background
    //                        (bound by pRect).  "pRect" and "ptTest" should
    //                        both be in the same coordinate system
    //                        (client, screen, etc).
    //
    //      hTheme          - theme data handle
    //      hdc             - HDC to draw into
    //      iPartId         - part number to test against
    //      iStateId        - state number (of the part)
    //      pRect           - the RECT used to draw the part
    //      hrgn            - optional region to use; must be in same coordinates as
    //                      -    pRect and pTest.
    //      ptTest          - the hit point to be tested
    //      dwOptions       - HTTB_xxx constants
    //      pwHitTestCode   - receives the returned hit test code - one of:
    //
    //                        HTNOWHERE, HTLEFT, HTTOPLEFT, HTBOTTOMLEFT,
    //                        HTRIGHT, HTTOPRIGHT, HTBOTTOMRIGHT,
    //                        HTTOP, HTBOTTOM, HTCLIENT
    //-------------------------------------------------------------------------
    function HitTestThemeBackground( hTheme : HTHEME; hdc : HDC; iPartId : Integer;
                                     iStateId : Integer; dwOptions : DWORD; pRect : PRect;
                                     hrgn : HRGN; ptTest : TPoint;
                                     out pwHitTestCode : PWord ) : HRESULT; virtual; stdcall; abstract;

    //------------------------------------------------------------------------
    //  DrawThemeEdge()     - Similar to the DrawEdge() API, but uses part colors
    //                        and is high-DPI aware
    //  hTheme              - theme data handle
    //  hdc                 - HDC to draw into
    //  iPartId             - part number to draw
    //  iStateId            - state number of part
    //  pDestRect           - the RECT used to draw the line(s)
    //  uEdge               - Same as DrawEdge() API
    //  uFlags              - Same as DrawEdge() API
    //  pContentRect        - Receives the interior rect if (uFlags & BF_ADJUST)
    //------------------------------------------------------------------------
    function DrawThemeEdge( hTheme : HTHEME; hdc : HDC; iPartId : Integer; iStateId : Integer;
                            pDestRect : PRect; uEdge : UINT; uFlags : UINT;
                            out pContentRect : PRect ) : HRESULT; virtual; stdcall; abstract;


    //------------------------------------------------------------------------
    //  DrawThemeIcon()     - draws an image within an imagelist based on
    //                        a (possible) theme-defined effect.
    //
    //  hTheme              - theme data handle
    //  hdc                 - HDC to draw into
    //  iPartId             - part number to draw
    //  iStateId            - state number of part
    //  pRect               - the RECT to draw the image within
    //  himl                - handle to IMAGELIST
    //  iImageIndex         - index into IMAGELIST (which icon to draw)
    //------------------------------------------------------------------------
    function DrawThemeIcon( hTheme : HTHEME; hdc : HDC; iPartId : Integer;
                            iStateId : Integer; pRect : PRect; himl : HIMAGELIST;
                            iImageIndex : Integer ) : HRESULT; virtual; stdcall; abstract;

    //---------------------------------------------------------------------------
    //  IsThemePartDefined() - returns TRUE if the theme has defined parameters
    //                         for the specified "iPartId" and "iStateId".
    //
    //  hTheme              - theme data handle
    //  iPartId             - part number to find definition for
    //  iStateId            - state number of part
    //---------------------------------------------------------------------------
    function IsThemePartDefined( hTheme : HTHEME; iPartId : Integer; iStateId : Integer ) : BOOL; virtual; stdcall; abstract;

    //---------------------------------------------------------------------------
    //  IsThemeBackgroundPartiallyTransparent()
    //                      - returns TRUE if the theme specified background for
    //                        the part/state has transparent pieces or
    //                        alpha-blended pieces.
    //
    //  hTheme              - theme data handle
    //  iPartId             - part number
    //  iStateId            - state number of part
    //---------------------------------------------------------------------------
    function IsThemeBackgroundPartiallyTransparent( hTheme : HTHEME; iPartId : Integer;
                                                    iStateId : Integer ) : BOOL; virtual; stdcall; abstract;

    //---------------------------------------------------------------------------
    //    lower-level theme information services
    //---------------------------------------------------------------------------
    // The following methods are getter routines for each of the Theme Data types.
    // Controls/Windows are defined in drawable "parts" by their author: a
    // parent part and 0 or more child parts.  Each of the parts can be
    // described in "states" (ex: disabled, hot, pressed).
    //---------------------------------------------------------------------------
    // Each of the below methods takes a "iPartId" param to specify the
    // part and a "iStateId" to specify the state of the part.
    // "iStateId=0" refers to the root part.  "iPartId" = "0" refers to
    // the root class.
    //-----------------------------------------------------------------------
    // Each method also take a "iPropId" param because multiple instances of
    // the same primitive type can be defined in the theme schema.
    //-----------------------------------------------------------------------

    //-----------------------------------------------------------------------
    //  GetThemeColor()     - Get the value for the specified COLOR property
    //
    //  hTheme              - theme data handle
    //  iPartId             - part number
    //  iStateId            - state number of part
    //  iPropId             - the property number to get the value for
    //  pColor              - receives the value of the property
    //-----------------------------------------------------------------------
    function GetThemeColor( hTheme : HTHEME; iPartId : Integer; iStateId : Integer;
                            iPropId : Integer; out pColor : PColorRef ) : HRESULT; virtual; stdcall; abstract;

    //-----------------------------------------------------------------------
    //  GetThemeMetric()    - Get the value for the specified metric/size
    //                        property
    //
    //  hTheme              - theme data handle
    //  hdc                 - (optional) hdc to be drawn into (DPI scaling)
    //  iPartId             - part number
    //  iStateId            - state number of part
    //  iPropId             - the property number to get the value for
    //  piVal               - receives the value of the property
    //-----------------------------------------------------------------------
    function GetThemeMetric( hTheme : HTHEME; hdc : HDC; iPartId : Integer;
                             iStateId : Integer; iPropId : Integer; out piVal : PInteger ) : HRESULT; virtual; stdcall; abstract;

    //-----------------------------------------------------------------------
    //  GetThemeString()    - Get the value for the specified string property
    //
    //  hTheme              - theme data handle
    //  iPartId             - part number
    //  iStateId            - state number of part
    //  iPropId             - the property number to get the value for
    //  pszBuff             - receives the string property value
    //  cchMaxBuffChars     - max. number of chars allowed in pszBuff
    //-----------------------------------------------------------------------
    function GetThemeString( hTheme : HTHEME; iPartId : Integer; iStateId : Integer;
                             iPropId : Integer; out pszBuff : LPWSTR;
                             cchMaxBuffChars : Integer ) : HRESULT; virtual; stdcall; abstract;

    //-----------------------------------------------------------------------
    //  GetThemeBool()      - Get the value for the specified BOOL property
    //
    //  hTheme              - theme data handle
    //  iPartId             - part number
    //  iStateId            - state number of part
    //  iPropId             - the property number to get the value for
    //  pfVal               - receives the value of the property
    //-----------------------------------------------------------------------
    function GetThemeBool( hTheme : HTHEME; iPartId : Integer; iStateId : Integer;
                           iPropId : Integer; out pfVal : PBOOL ) : HRESULT; virtual; stdcall; abstract;

    //-----------------------------------------------------------------------
    //  GetThemeInt()       - Get the value for the specified int property
    //
    //  hTheme              - theme data handle
    //  iPartId             - part number
    //  iStateId            - state number of part
    //  iPropId             - the property number to get the value for
    //  piVal               - receives the value of the property
    //-----------------------------------------------------------------------
    function GetThemeInt( hTheme : HTHEME; iPartId : Integer; iStateId : Integer;
                          iPropId : Integer; out piVal : PInteger ) : HRESULT; virtual; stdcall; abstract;

    //-----------------------------------------------------------------------
    //  GetThemeEnumValue() - Get the value for the specified ENUM property
    //
    //  hTheme              - theme data handle
    //  iPartId             - part number
    //  iStateId            - state number of part
    //  iPropId             - the property number to get the value for
    //  piVal               - receives the value of the enum (cast to int*)
    //-----------------------------------------------------------------------
    function GetThemeEnumValue( hTheme : HTHEME; iPartId : Integer; iStateId : Integer;
                                iPropId : Integer; out piVal : PInteger ) : HRESULT; virtual; stdcall; abstract;

    //-----------------------------------------------------------------------
    //  GetThemePosition()  - Get the value for the specified position
    //                        property
    //
    //  hTheme              - theme data handle
    //  iPartId             - part number
    //  iStateId            - state number of part
    //  iPropId             - the property number to get the value for
    //  pPoint              - receives the value of the position property
    //-----------------------------------------------------------------------
    function GetThemePosition( hTheme : HTHEME; iPartId : Integer; iStateId : Integer;
                               iPropId : Integer; out pPoint : PPoint ) : HRESULT; virtual; stdcall; abstract;

    //-----------------------------------------------------------------------
    //  GetThemeFont()      - Get the value for the specified font property
    //
    //  hTheme              - theme data handle
    //  hdc                 - (optional) hdc to be drawn to (DPI scaling)
    //  iPartId             - part number
    //  iStateId            - state number of part
    //  iPropId             - the property number to get the value for
    //  pFont               - receives the value of the LOGFONT property
    //                        (scaled for the current logical screen dpi)
    //-----------------------------------------------------------------------
    function GetThemeFont( hTheme : HTHEME; hdc : HDC; iPartId : Integer; iStateId : Integer;
                           iPropId : Integer; out pFont : PLogFont ) : HRESULT; virtual; stdcall; abstract;

    //-----------------------------------------------------------------------
    //  GetThemeRect()      - Get the value for the specified RECT property
    //
    //  hTheme              - theme data handle
    //  iPartId             - part number
    //  iStateId            - state number of part
    //  iPropId             - the property number to get the value for
    //  pRect               - receives the value of the RECT property
    //-----------------------------------------------------------------------
    function GetThemeRect( hTheme : HTHEME; iPartId : Integer; iStateId : Integer;
                           iPropId : Integer; out pRect : PRect ) : HRESULT; virtual; stdcall; abstract;

    //-----------------------------------------------------------------------
    //  GetThemeMargins()   - Get the value for the specified MARGINS property
    //
    //      hTheme          - theme data handle
    //      hdc             - (optional) hdc to be used for drawing
    //      iPartId         - part number
    //      iStateId        - state number of part
    //      iPropId         - the property number to get the value for
    //      prc             - RECT for area to be drawn into
    //      pMargins        - receives the value of the MARGINS property
    //-----------------------------------------------------------------------
    function GetThemeMargins( hTheme : HTHEME; hdc : HDC; iPartId : Integer;
                              iStateId : Integer; iPropId : Integer; prc : PRect;
                              out pMargins : PMargins ) : HRESULT; virtual; stdcall; abstract;

    //-----------------------------------------------------------------------
    //  GetThemeIntList()   - Get the value for the specified INTLIST struct
    //
    //      hTheme          - theme data handle
    //      iPartId         - part number
    //      iStateId        - state number of part
    //      iPropId         - the property number to get the value for
    //      pIntList        - receives the value of the INTLIST property
    //-----------------------------------------------------------------------
    function GetThemeIntList( hTheme : HTHEME; iPartId : Integer; iStateId : Integer;
                              iPropId : Integer; out pIntList : PIntList ) : HRESULT; virtual; stdcall; abstract;

    //-----------------------------------------------------------------------
    //  GetThemePropertyOrigin()
    //                      - searches for the specified theme property
    //                        and sets "pOrigin" to indicate where it was
    //                        found (or not found)
    //
    //  hTheme              - theme data handle
    //  iPartId             - part number
    //  iStateId            - state number of part
    //  iPropId             - the property number to search for
    //  pOrigin             - receives the value of the property origin
    //-----------------------------------------------------------------------
    function GetThemePropertyOrigin( hTheme : HTHEME; iPartId : Integer; iStateId : Integer;
                                     iPropId : Integer; out pOrigin : PPropertyOrigin ) : HRESULT; virtual; stdcall; abstract;

    //---------------------------------------------------------------------------
    //  SetWindowTheme()
    //                      - redirects an existing Window to use a different
    //                        section of the current theme information than its
    //                        class normally asks for.
    //
    //  hwnd                - the handle of the window (cannot be NULL)
    //
    //  pszSubAppName       - app (group) name to use in place of the calling
    //                        app's name.  If NULL, the actual calling app
    //                        name will be used.
    //
    //  pszSubIdList        - semicolon separated list of class Id names to
    //                        use in place of actual list passed by the
    //                        window's class.  if NULL, the id list from the
    //                        calling class is used.
    //---------------------------------------------------------------------------
    // The Theme Manager will remember the "pszSubAppName" and the
    // "pszSubIdList" associations thru the lifetime of the window (even
    // if themes are subsequently changed).  The window is sent a
    // "WM_THEMECHANGED" msg at the end of this call, so that the new
    // theme can be found and applied.
    //---------------------------------------------------------------------------
    // When "pszSubAppName" or "pszSubIdList" are NULL, the Theme Manager
    // removes the previously remember association.  To turn off theme-ing for
    // the specified window, you can pass an empty string (L"") so it
    // won't match any section entries.
    //---------------------------------------------------------------------------
    function SetWindowTheme( hwnd : HWND; pszSubAppName : LPCWSTR; pszSubIdList : LPCWSTR ) : HRESULT; virtual; stdcall; abstract;

    //---------------------------------------------------------------------------
    //  GetThemeFilename()  - Get the value for the specified FILENAME property.
    //
    //  hTheme              - theme data handle
    //  iPartId             - part number
    //  iStateId            - state number of part
    //  iPropId             - the property number to search for
    //  pszThemeFileName    - output buffer to receive the filename
    //  cchMaxBuffChars     - the size of the return buffer, in chars
    //---------------------------------------------------------------------------
    function GetThemeFilename( hTheme : HTHEME; iPartId : Integer; iStateId : Integer;
                               iPropId : Integer; out pszThemeFileName : LPWSTR;
                               cchMaxBuffChars : Integer ) : HRESULT; virtual; stdcall; abstract;

    //---------------------------------------------------------------------------
    //  GetThemeSysColor()  - Get the value of the specified System color.
    //
    //  hTheme              - the theme data handle.  if non-NULL, will return
    //                        color from [SysMetrics] section of theme.
    //                        if NULL, will return the global system color.
    //
    //  iColorId            - the system color index defined in winuser.h
    //---------------------------------------------------------------------------
    function GetThemeSysColor( hTheme : HTHEME; iColorId : Integer ) : TColorRef; virtual; stdcall; abstract;

    //---------------------------------------------------------------------------
    //  GetThemeSysColorBrush()
    //                      - Get the brush for the specified System color.
    //
    //  hTheme              - the theme data handle.  if non-NULL, will return
    //                        brush matching color from [SysMetrics] section of
    //                        theme.  if NULL, will return the brush matching
    //                        global system color.
    //
    //  iColorId            - the system color index defined in winuser.h
    //---------------------------------------------------------------------------
    function GetThemeSysColorBrush( hTheme : HTHEME; iColorId : Integer ) : HBRUSH; virtual; stdcall; abstract;

    //---------------------------------------------------------------------------
    //  GetThemeSysBool()   - Get the boolean value of specified System metric.
    //
    //  hTheme              - the theme data handle.  if non-NULL, will return
    //                        BOOL from [SysMetrics] section of theme.
    //                        if NULL, will return the specified system boolean.
    //
    //  iBoolId             - the TMT_XXX BOOL number (first BOOL
    //                        is TMT_FLATMENUS)
    //---------------------------------------------------------------------------
    function GetThemeSysBool( hTheme : HTHEME; iBoolId : Integer ) : BOOL; virtual; stdcall; abstract;

    //---------------------------------------------------------------------------
    //  GetThemeSysSize()   - Get the value of the specified System size metric.
    //                        (scaled for the current logical screen dpi)
    //
    //  hTheme              - the theme data handle.  if non-NULL, will return
    //                        size from [SysMetrics] section of theme.
    //                        if NULL, will return the global system metric.
    //
    //  iSizeId             - the following values are supported when
    //                        hTheme is non-NULL:
    //
    //                          SM_CXBORDER   (border width)
    //                          SM_CXVSCROLL  (scrollbar width)
    //                          SM_CYHSCROLL  (scrollbar height)
    //                          SM_CXSIZE     (caption width)
    //                          SM_CYSIZE     (caption height)
    //                          SM_CXSMSIZE   (small caption width)
    //                          SM_CYSMSIZE   (small caption height)
    //                          SM_CXMENUSIZE (menubar width)
    //                          SM_CYMENUSIZE (menubar height)
    //
    //                        when hTheme is NULL, iSizeId is passed directly
    //                        to the GetSystemMetrics() function
    //---------------------------------------------------------------------------
    function GetThemeSysSize( hTheme : HTHEME; iSizeId : Integer ) : Integer; virtual; stdcall; abstract;

    //---------------------------------------------------------------------------
    //  GetThemeSysFont()   - Get the LOGFONT for the specified System font.
    //
    //  hTheme              - the theme data handle.  if non-NULL, will return
    //                        font from [SysMetrics] section of theme.
    //                        if NULL, will return the specified system font.
    //
    //  iFontId             - the TMT_XXX font number (first font
    //                        is TMT_CAPTIONFONT)
    //
    //  plf                 - ptr to LOGFONT to receive the font value.
    //                        (scaled for the current logical screen dpi)
    //---------------------------------------------------------------------------
    function GetThemeSysFont( hTheme : HTHEME; iFontId : Integer; out plf : PLogFont ) : HRESULT; virtual; stdcall; abstract;

    //---------------------------------------------------------------------------
    //  GetThemeSysString() - Get the value of specified System string metric.
    //
    //  hTheme              - the theme data handle (required)
    //
    //  iStringId           - must be one of the following values:
    //
    //                          TMT_CSSNAME
    //                          TMT_XMLNAME
    //
    //  pszStringBuff       - the buffer to receive the string value
    //
    //  cchMaxStringChars   - max. number of chars that pszStringBuff can hold
    //---------------------------------------------------------------------------
    function GetThemeSysString( hTheme : HTHEME; iStringId : Integer; out pszStringBuff : LPWSTR;
                                cchMaxStringChars : Integer ) : HRESULT; virtual; stdcall; abstract;

    //---------------------------------------------------------------------------
    //  GetThemeSysInt() - Get the value of specified System int.
    //
    //  hTheme              - the theme data handle (required)
    //
    //  iIntId              - must be one of the following values:
    //
    //                          TMT_DPIX
    //                          TMT_DPIY
    //                          TMT_MINCOLORDEPTH
    //
    //  piValue             - ptr to int to receive value
    //---------------------------------------------------------------------------
    function GetThemeSysInt( hTheme : HTHEME; iIntId : Integer; piValue : PInteger ) : HRESULT; virtual; stdcall; abstract;

    //---------------------------------------------------------------------------
    //  IsThemeActive()     - can be used to test if a system theme is active
    //                        for the current user session.
    //
    //                        use the API "IsAppThemed()" to test if a theme is
    //                        active for the calling process.
    //---------------------------------------------------------------------------
    function IsThemeActive : BOOL; virtual; stdcall; abstract;

    //---------------------------------------------------------------------------
    //  IsAppThemed()       - returns TRUE if a theme is active and available to
    //                        the current process
    //---------------------------------------------------------------------------
    function IsAppThemed : BOOL; virtual; stdcall; abstract;


    //---------------------------------------------------------------------------
    //  GetWindowTheme()    - if window is themed, returns its most recent
    //                        HTHEME from OpenThemeData() - otherwise, returns
    //                        NULL.
    //
    //      hwnd            - the window to get the HTHEME of
    //---------------------------------------------------------------------------
    function GetWindowTheme( hwnd : HWND ) : HTHEME; virtual; stdcall; abstract;

    //---------------------------------------------------------------------------
    //  EnableThemeDialogTexture()
    //
    //  - Enables/disables dialog background theme.  This method can be used to
    //    tailor dialog compatibility with child windows and controls that
    //    may or may not coordinate the rendering of their client area backgrounds
    //    with that of their parent dialog in a manner that supports seamless
    //    background texturing.
    //
    //      hdlg         - the window handle of the target dialog
    //      dwFlags      - ETDT_ENABLE to enable the theme-defined dialog background texturing,
    //                     ETDT_DISABLE to disable background texturing,
    //                     ETDT_ENABLETAB to enable the theme-defined background
    //                          texturing using the Tab texture
    //---------------------------------------------------------------------------
    function EnableThemeDialogTexture( hwnd : HWND; dwFlags : DWORD ) : HRESULT; virtual; stdcall; abstract;

    //---------------------------------------------------------------------------
    //  IsThemeDialogTextureEnabled()
    //
    //  - Reports whether the dialog supports background texturing.
    //
    //      hdlg         - the window handle of the target dialog
    //---------------------------------------------------------------------------
    function IsThemeDialogTextureEnabled( hwnd : HWND ) : BOOL; virtual; stdcall; abstract;


    //---------------------------------------------------------------------------
    //  GetThemeAppProperties()
    //                      - returns the app property flags that control theming
    //---------------------------------------------------------------------------
    function GetThemeAppProperties : DWORD; virtual; stdcall; abstract;

    //---------------------------------------------------------------------------
    //  SetThemeAppProperties()
    //                      - sets the flags that control theming within the app
    //
    //      dwFlags         - the flag values to be set
    //---------------------------------------------------------------------------
    procedure SetThemeAppProperties( dwFlags : DWORD ); virtual; stdcall; abstract;

    //---------------------------------------------------------------------------
    //  GetCurrentThemeName()
    //                      - Get the name of the current theme in-use.
    //                        Optionally, return the ColorScheme name and the
    //                        Size name of the theme.
    //
    //  pszThemeFileName    - receives the theme path & filename
    //  cchMaxNameChars     - max chars allowed in pszNameBuff
    //
    //  pszColorBuff        - (optional) receives the canonical color scheme name
    //                        (not the display name)
    //  cchMaxColorChars    - max chars allowed in pszColorBuff
    //
    //  pszSizeBuff         - (optional) receives the canonical size name
    //                        (not the display name)
    //  cchMaxSizeChars     - max chars allowed in pszSizeBuff
    //---------------------------------------------------------------------------
    function GetCurrentThemeName( out pszThemeFileName : LPWSTR; cchMaxNameChars : Integer;
                                  out pszColorBuff : LPWSTR; cchMaxColorChars : Integer;
                                  out pszSizeBuff : LPWSTR; cchMaxSizeChars : Integer ) : HRESULT; virtual; stdcall; abstract;

    //---------------------------------------------------------------------------
    //  GetThemeDocumentationProperty()
    //                      - Get the value for the specified property name from
    //                        the [documentation] section of the themes.ini file
    //                        for the specified theme.  If the property has been
    //                        localized in the theme files string table, the
    //                        localized version of the property value is returned.
    //
    //  pszThemeFileName    - filename of the theme file to query
    //  pszPropertyName     - name of the string property to retreive a value for
    //  pszValueBuff        - receives the property string value
    //  cchMaxValChars      - max chars allowed in pszValueBuff
    //---------------------------------------------------------------------------
    function GetThemeDocumentationProperty( pszThemeName : LPCWSTR; pszPropertyName : LPCWSTR;
                                            out pszValueBuff : LPWSTR; cchMaxValChars : Integer ) : HRESULT; virtual; stdcall; abstract;

    //---------------------------------------------------------------------------
    //  Theme API Error Handling
    //
    //      All functions in the Theme API not returning an HRESULT (THEMEAPI_)
    //      use the WIN32 function "SetLastError()" to record any call failures.
    //
    //      To retreive the error code of the last failure on the
    //      current thread for these type of API's, use the WIN32 function
    //      "GetLastError()".
    //
    //      All Theme API error codes (HRESULT's and GetLastError() values)
    //      should be normal win32 errors which can be formatted into
    //      strings using the Win32 API FormatMessage().
    //---------------------------------------------------------------------------

    //---------------------------------------------------------------------------
    // DrawThemeParentBackground()
    //                      - used by partially-transparent or alpha-blended
    //                        child controls to draw the part of their parent
    //                        that they appear in front of.
    //
    //  hwnd                - handle of the child control
    //  hdc                 - hdc of the child control
    //  prc                 - (optional) rect that defines the area to be
    //                        drawn (CHILD coordinates)
    //---------------------------------------------------------------------------
    function DrawThemeParentBackground( hwnd : HWND; hdc : HDC; prc : PRect ) : HRESULT; virtual; stdcall; abstract;

    //---------------------------------------------------------------------------
    //  EnableTheming()     - enables or disables themeing for the current user
    //                        in the current and future sessions.
    //
    //  fEnable             - if FALSE, disable theming & turn themes off.
    //                      - if TRUE, enable themeing and, if user previously
    //                        had a theme active, make it active now.
    //---------------------------------------------------------------------------
    function EnableTheming( fEnable : BOOL ) : HRESULT; virtual; stdcall; abstract;
  end;

implementation

end.

