(*
  Object Pascal translation of the Altap Salamander SDK library.
  Original source: winliblt.h

  Portions created by ALTAP are Copyright (c) ALTAP, spol. s r.o.
  All rights reserved.

  Portions created by Microsoft are Copyright (c) Microsoft Corporation
  All rights reserved.

  Portions created by LostGenius Posse are Copyright (c) 2007, LostGenius Posse (Timo 'jomanto' Boll)
  All rights reserved.

  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

      * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
      * Redistributions in binary form may reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
      * Neither the name of the LostGenius Posse nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*)
(*
  History:

  2007-05-31: initial version
*)
unit winliblt;

interface

const
  LF_FACESIZE       = 32;
  MAX_INTLIST_COUNT = 10;
  WM_APP            = $8000;

type
  HWND       = type LongWord;
  THandle    = LongWord;
  PHandle    = ^THandle;
  HTHEME     = THandle;
  HDC        = type LongWord;
  DWORD      = LongWord;
  PDWORD     = ^DWORD;
  LPDWORD    = PDWORD;
  HRGN       = type LongWord;
  PHRGN      = ^HRGN;
  HIMAGELIST = THandle;
  BOOL       = LongBool;
  PBOOL      = ^BOOL;
  TColorRef  = DWORD;
  PColorRef  = ^TColorRef;
  HBRUSH     = type LongWord;
  UINT       = LongWord;
  HICON      = type LongWord;
  PHICON     = ^HICON;
  HKEY       = type LongWord;
  PHKEY      = ^HKEY;
  LONGLONG   = Int64;
  HBITMAP    = type LongWord;
  HMENU      = type LongWord;
  WPARAM     = Longint;
  LPARAM     = Longint;

  WCHAR = WideChar;
  {$EXTERNALSYM WCHAR}
  PWChar = PWideChar;

  LPSTR   = PAnsiChar;
  PLPSTR  = ^LPSTR;
  LPCSTR  = PAnsiChar;
  LPCTSTR = PAnsiChar;
  LPTSTR  = PAnsiChar;
  LPWSTR  = PWideChar;
  PLPWSTR = ^LPWSTR;
  LPCWSTR = PWideChar;

  va_list = PChar;

  TPoint = packed record
    X: Longint;
    Y: Longint;
  end;
  PPoint = ^TPoint;

  TRect = packed record
    case Integer of
      0: (Left, Top, Right, Bottom: Longint);
      1: (TopLeft, BottomRight: TPoint);
  end;
  PRect = ^TRect;

  TThemeSize =
  (
    TS_MIN,                                                                     // minimum size
    TS_TRUE,                                                                    // size without stretching
    TS_DRAW                                                                     // size that theme mgr will use to draw part
  );

  TSize = packed record
    cx : Longint;
    cy : Longint;
  end;
  PSize = ^TSize;

  tagTEXTMETRICA = record
    tmHeight: Longint;
    tmAscent: Longint;
    tmDescent: Longint;
    tmInternalLeading: Longint;
    tmExternalLeading: Longint;
    tmAveCharWidth: Longint;
    tmMaxCharWidth: Longint;
    tmWeight: Longint;
    tmOverhang: Longint;
    tmDigitizedAspectX: Longint;
    tmDigitizedAspectY: Longint;
    tmFirstChar: AnsiChar;
    tmLastChar: AnsiChar;
    tmDefaultChar: AnsiChar;
    tmBreakChar: AnsiChar;
    tmItalic: Byte;
    tmUnderlined: Byte;
    tmStruckOut: Byte;
    tmPitchAndFamily: Byte;
    tmCharSet: Byte;
  end;
  tagTEXTMETRICW = record
    tmHeight: Longint;
    tmAscent: Longint;
    tmDescent: Longint;
    tmInternalLeading: Longint;
    tmExternalLeading: Longint;
    tmAveCharWidth: Longint;
    tmMaxCharWidth: Longint;
    tmWeight: Longint;
    tmOverhang: Longint;
    tmDigitizedAspectX: Longint;
    tmDigitizedAspectY: Longint;
    tmFirstChar: WideChar;
    tmLastChar: WideChar;
    tmDefaultChar: WideChar;
    tmBreakChar: WideChar;
    tmItalic: Byte;
    tmUnderlined: Byte;
    tmStruckOut: Byte;
    tmPitchAndFamily: Byte;
    tmCharSet: Byte;
  end;
  TTextMetricA = tagTEXTMETRICA;
  TTextMetricW = tagTEXTMETRICW;
  TTextMetric  = TTextMetricA;
  PTextMetricA = ^TTextMetricA;
  PTextMetricW = ^TTextMetricW;
  PTextMetric  = PTextMetricA;

  tagLOGFONTA = packed record
    lfHeight: Longint;
    lfWidth: Longint;
    lfEscapement: Longint;
    lfOrientation: Longint;
    lfWeight: Longint;
    lfItalic: Byte;
    lfUnderline: Byte;
    lfStrikeOut: Byte;
    lfCharSet: Byte;
    lfOutPrecision: Byte;
    lfClipPrecision: Byte;
    lfQuality: Byte;
    lfPitchAndFamily: Byte;
    lfFaceName: array[0..LF_FACESIZE - 1] of AnsiChar;
  end;
  tagLOGFONTW = packed record
    lfHeight: Longint;
    lfWidth: Longint;
    lfEscapement: Longint;
    lfOrientation: Longint;
    lfWeight: Longint;
    lfItalic: Byte;
    lfUnderline: Byte;
    lfStrikeOut: Byte;
    lfCharSet: Byte;
    lfOutPrecision: Byte;
    lfClipPrecision: Byte;
    lfQuality: Byte;
    lfPitchAndFamily: Byte;
    lfFaceName: array[0..LF_FACESIZE - 1] of WideChar;
  end;
  TLogFontA = tagLOGFONTA;
  TLogFontW = tagLOGFONTW;
  TLogFont  = TLogFontA;
  PLogFontA = ^TLogFontA;
  PLogFontW = ^TLogFontW;
  PLogFont  = PLogFontA;

  TMargins = record
    cxLeftWidth    : Integer;
    cxRightWidth   : Integer;
    cyTopHeight    : Integer;
    cyBottomHeight : Integer;
  end;
  PMargins = ^TMargins;

  TIntList = record
    iValueCount : Integer;
    iValues     : array [0..MAX_INTLIST_COUNT - 1] of Integer;
  end;
  PIntList = ^TIntList;

  TPropertyOrigin =
  (
    PO_STATE,
    PO_PART,
    PO_CLASS,
    PO_GLOBAL,
    PO_NOTFOUND
  );
  PPropertyOrigin = ^TPropertyOrigin;

  THelpInfo = packed record
    cbSize: UINT;
    iContextType: Integer;
    iCtrlId: Integer;
    hItemHandle: THandle;
    dwContextId: DWORD;
    MousePos: TPoint;      
  end;
  PHelpInfo = ^THelpInfo;

  TLargeInteger = record
    case Integer of
    0: (
        LowPart  : DWORD;
        HighPart : Longint
       );
    1: (
        QuadPart : LONGLONG
       );
  end;
  PLargeInteger = ^TLargeInteger;

  TFileTime = record
    dwLowDateTime  : DWORD;
    dwHighDateTime : DWORD;
  end;
  PFileTime = ^TFileTime;

  TSystemTime = record
    wYear         : Word;
    wMonth        : Word;
    wDayOfWeek    : Word;
    wDay          : Word;
    wHour         : Word;
    wMinute       : Word;
    wSecond       : Word;
    wMilliseconds : Word;
  end;
  PSystemTime = ^TSystemTime;

  TMsg = packed record
    hwnd    : HWND;
    message : UINT;
    wParam  : WPARAM;
    lParam  : LPARAM;
    time    : DWORD;
    pt      : TPoint;
  end;
  PMsg = ^TMsg;

  tagOFNA = packed record
    lStructSize: DWORD;
    hWndOwner: HWND;
    hInstance: HINST;
    lpstrFilter: PAnsiChar;
    lpstrCustomFilter: PAnsiChar;
    nMaxCustFilter: DWORD;
    nFilterIndex: DWORD;
    lpstrFile: PAnsiChar;
    nMaxFile: DWORD;
    lpstrFileTitle: PAnsiChar;
    nMaxFileTitle: DWORD;
    lpstrInitialDir: PAnsiChar;
    lpstrTitle: PAnsiChar;
    Flags: DWORD;
    nFileOffset: Word;
    nFileExtension: Word;
    lpstrDefExt: PAnsiChar;
    lCustData: LPARAM;
    lpfnHook: function(Wnd: HWND; Msg: UINT; wParam: WPARAM; lParam: LPARAM): UINT stdcall;
    lpTemplateName: PAnsiChar;
    pvReserved: Pointer;
    dwReserved: DWORD;
    FlagsEx: DWORD;
  end;
  tagOFNW = packed record
    lStructSize: DWORD;
    hWndOwner: HWND;
    hInstance: HINST;
    lpstrFilter: PWideChar;
    lpstrCustomFilter: PWideChar;
    nMaxCustFilter: DWORD;
    nFilterIndex: DWORD;
    lpstrFile: PWideChar;
    nMaxFile: DWORD;
    lpstrFileTitle: PWideChar;
    nMaxFileTitle: DWORD;
    lpstrInitialDir: PWideChar;
    lpstrTitle: PWideChar;
    Flags: DWORD;
    nFileOffset: Word;
    nFileExtension: Word;
    lpstrDefExt: PWideChar;
    lCustData: LPARAM;
    lpfnHook: function(Wnd: HWND; Msg: UINT; wParam: WPARAM; lParam: LPARAM): UINT stdcall;
    lpTemplateName: PWideChar;
    pvReserved: Pointer;
    dwReserved: DWORD;
    FlagsEx: DWORD;
  end;
  TOpenFilenameA = tagOFNA;
  TOpenFilenameW = tagOFNW;
  TOpenFilename  = TOpenFilenameA;
  POpenFilenameA = ^TOpenFilenameA;
  POpenFilenameW = ^TOpenFilenameW;
  POpenFilename  = POpenFilenameA;

implementation

end.
