(*
  Object Pascal translation of the Altap Salamander SDK library.
  Original source: spl_gen.h

  Portions created by ALTAP are Copyright (c) ALTAP, spol. s r.o.
  All rights reserved.

  Portions created by LostGenius Posse are Copyright (c) 2007, LostGenius Posse (Timo 'jomanto' Boll)
  All rights reserved.

  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

      * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
      * Redistributions in binary form may reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
      * Neither the name of the LostGenius Posse nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*)
(*
  History:
  
  2007-05-30: initial version
*)
unit spl_gen;

interface

uses
  winliblt,
  spl_com,
  spl_fs;

const
//
// ****************************************************************************
// CSalamanderGeneralAbstract
//
// obecne pouzitelne metody Salamandera (pro vsechny typy pluginu)

  // typy message-boxu
  MSGBOX_INFO        = 0;
  MSGBOX_ERROR       = 1;
  MSGBOX_EX_ERROR    = 2;
  MSGBOX_QUESTION    = 3;
  MSGBOX_EX_QUESTION = 4;
  MSGBOX_WARNING     = 5;
  MSGBOX_EX_WARNING  = 6;

  // konstanty pro CSalamanderGeneralAbstract::SalMessageBoxEx
  MSGBOXEX_OK                   = $00000000;                                    // MB_OK
  MSGBOXEX_OKCANCEL             = $00000001;                                    // MB_OKCANCEL
  MSGBOXEX_ABORTRETRYIGNORE     = $00000002;                                    // MB_ABORTRETRYIGNORE
  MSGBOXEX_YESNOCANCEL          = $00000003;                                    // MB_YESNOCANCEL
  MSGBOXEX_YESNO                = $00000004;                                    // MB_YESNO
  MSGBOXEX_RETRYCANCEL          = $00000005;                                    // MB_RETRYCANCEL
  MSGBOXEX_CANCELTRYCONTINUE    = $00000006;                                    // MB_CANCELTRYCONTINUE
  MSGBOXEX_CONTINUEABORT        = $00000007;                                    // MB_CONTINUEABORT

  MSGBOXEX_ICONHAND             = $00000010;                                    // MB_ICONHAND / MB_ICONSTOP / MB_ICONERROR
  MSGBOXEX_ICONQUESTION         = $00000020;                                    // MB_ICONQUESTION
  MSGBOXEX_ICONEXCLAMATION      = $00000030;                                    // MB_ICONEXCLAMATION / MB_ICONWARNING
  MSGBOXEX_ICONINFORMATION      = $00000040;                                    // MB_ICONASTERISK / MB_ICONINFORMATION

  MSGBOXEX_DEFBUTTON1           = $00000000;                                    // MB_DEFBUTTON1
  MSGBOXEX_DEFBUTTON2           = $00000100;                                    // MB_DEFBUTTON2
  MSGBOXEX_DEFBUTTON3           = $00000200;                                    // MB_DEFBUTTON3
  MSGBOXEX_DEFBUTTON4           = $00000300;                                    // MB_DEFBUTTON4

  MSGBOXEX_HELP                 = $00004000;                                    // MB_HELP (bit mask)

  MSGBOXEX_SETFOREGROUND        = $00010000;                                    // MB_SETFOREGROUND (bit mask)

  // altap specific
  MSGBOXEX_SILENT               = $10000000;                                    // messagebox nevyda pri otevreni zadny zvuk (bit mask)
  MSGBOXEX_ESCAPEENABLED        = $20000000;                                    // v pripade MB_YESNO messageboxu povoli Escape (generuje IDNO) (bit mask)
  MSGBOXEX_HINT                 = $40000000;                                    // pokud se pouziva CheckBoxText, bude v nem vyhledan oddelovac \t a zobrazen jako hint

  MSGBOXEX_TYPEMASK             = $0000000F;                                    // MB_TYPEMASK
  MSGBOXEX_ICONMASK             = $000000F0;                                    // MB_ICONMASK
  MSGBOXEX_DEFMASK              = $00000F00;                                    // MB_DEFMASK
  MSGBOXEX_MODEMASK             = $00003000;                                    // MB_MODEMASK
  MSGBOXEX_MISCMASK             = $0000C000;                                    // MB_MISCMASK
  MSGBOXEX_EXMASK               = $F0000000;

  // navratove hodnoty message boxu
  DIALOG_FAIL                   = $00000000;                                    // dialog se nepodarilo otevrit
  // jednotliva tlacitka
  DIALOG_OK                     = $00000001;                                    // IDOK
  DIALOG_CANCEL                 = $00000002;                                    // IDCANCEL
  DIALOG_ABORT                  = $00000003;                                    // IDABORT
  DIALOG_RETRY                  = $00000004;                                    // IDRETRY
  DIALOG_IGNORE                 = $00000005;                                    // IDIGNORE
  DIALOG_YES                    = $00000006;                                    // IDYES
  DIALOG_NO                     = $00000007;                                    // IDNO
  DIALOG_TRYAGAIN               = $0000000a;                                    // IDTRYAGAIN
  DIALOG_CONTINUE               = $0000000b;                                    // IDCONTINUE
  // altap specific
  DIALOG_SKIP                   = $10000000;
  DIALOG_SKIPALL                = $20000000;
  DIALOG_ALL                    = $30000000;

  // identifikace panelu
  PANEL_SOURCE = 1;                                                             // zdrojovy panel (aktivni panel)
  PANEL_TARGET = 2;                                                             // cilovy panel (neaktivni panel)
  PANEL_LEFT   = 3;                                                             // levy panel
  PANEL_RIGHT  = 4;                                                             // pravy panel

  // typy cest
  PATH_TYPE_WINDOWS = 1;                                                        // windowsova cesta ("c:\path" nebo UNC cesta)
  PATH_TYPE_ARCHIVE = 2;                                                        // cesta do archivu (archiv lezi na windowsove ceste)
  PATH_TYPE_FS      = 3;                                                        // cesta na pluginovy file-system

  // Z nasledujici skupiny flagu lze vybrat pouze jeden.
  // Definuji sadu zobrazenych tlacitek v ruznych chybovych hlasenich.
  BUTTONS_OK               = $00000000;                                         // OK
  BUTTONS_RETRYCANCEL      = $00000001;                                         // Retry / Cancel
  BUTTONS_SKIPCANCEL       = $00000002;                                         // Skip / Skip all / Cancel
  BUTTONS_RETRYSKIPCANCEL  = $00000003;                                         // Retry / Skip / Skip all / Cancel
  BUTTONS_YESALLSKIPCANCEL = $00000004;                                         // Yes / All / Skip / Skip all / Cancel
  BUTTONS_YESNOCANCEL      = $00000005;                                         // Yes / No / Cancel
  BUTTONS_YESALLCANCEL     = $00000006;                                         // Yes / All / Cancel
  BUTTONS_MASK             = $000000FF;                                         // interni maska, nepouzivat

  // chybove konstanty pro CSalamanderGeneralAbstract::SalGetFullName
  GFN_SERVERNAMEMISSING     = 1;                                                // v UNC ceste chybi server name
  GFN_SHARENAMEMISSING      = 2;                                                // v UNC ceste chybi share name
  GFN_TOOLONGPATH           = 3;                                                // operaci by vznikla prilis dlouha cesta
  GFN_INVALIDDRIVE          = 4;                                                // u normalni cesty (c:\) neni pismenko A-Z (ani a-z)
  GFN_INCOMLETEFILENAME     = 5;                                                // relativni cesta bez zadaneho 'curDir' -> neresitelne
  GFN_EMPTYNAMENOTALLOWED   = 6;                                                // prazdny retezec 'name'
  GFN_PATHISINVALID         = 7;                                                // nelze vyloucit "..", napr. "c:\.."

  // chybovy kod pro stav, kdy uzivatel prerusi CSalamanderGeneralAbstract::SalCheckPath klavesou ESC
  ERROR_USER_TERMINATED = -100;

  PATH_MAX_PATH = 248;                                                          // limit pro max. delku cesty (plne jmeno adresare),
                                                                                // pozor: v limitu uz je zapocteny null-terminator (max. delka retezce je 247 znaku)

  // chybove konstanty pro CSalamanderGeneralAbstract::SalParsePath:
  // vstupem byla prazdna cesta a 'curPath' bylo NULL (prazdna cesta se nahrazuje aktualni cestou,
  // ale ta tu neni znama)
  SPP_EMPTYPATHNOTALLOWED     = 1;
  // windowsova cesta (normal + UNC) neexistuje, neni pristupna nebo uzivatel prerusil test
  // na pristupnost cesty (soucasti je i pokus o obnoveni sit. spojeni)
  SPP_WINDOWSPATHERROR        = 2;
  // windowsova cesta zacina jmenem souboru, ktery ale neni archiv (jinak by slo o cestu do archivu)
  SPP_NOTARCHIVEFILE          = 3;
  // FS cesta - jmeno pluginoveho FS (fs-name - pred ':' v ceste) neni zname (zadnemu pluginu
  // nebylo toto jmeno zaregistrovano)
  SPP_NOTPLUGINFS             = 4;
  // jde o relativni cestu, ale aktualni cesta neni znama nebo jde o FS (tam nelze poznat root
  // a vubec nezname strukturu fs-user-part cesty, takze nelze provest prevod na absolutni cestu)
  // je-li aktualni cesta FS ('curPathIsDiskOrArchive' je FALSE), nedojde v tomto pripade ke hlaseni
  // chyby uzivateli (predpoklada se dalsi zpracovani na strane FS, ktere metodu SalParsePath volalo)
  SPP_INCOMLETEPATH           = 5;

  // konstanty vnitrnich barev Salamandera
  SALCOL_FOCUS_ACTIVE_NORMAL        = 0;                                        // barvy pera pro ramecek kolem polozky
  SALCOL_FOCUS_ACTIVE_SELECTED      = 1;
  SALCOL_FOCUS_FG_INACTIVE_NORMAL   = 2;
  SALCOL_FOCUS_FG_INACTIVE_SELECTED = 3;
  SALCOL_FOCUS_BK_INACTIVE_NORMAL   = 4;
  SALCOL_FOCUS_BK_INACTIVE_SELECTED = 5;
  SALCOL_ITEM_FG_NORMAL             = 6;                                        // barvy textu polozek v panelu
  SALCOL_ITEM_FG_SELECTED           = 7;
  SALCOL_ITEM_FG_FOCUSED            = 8;
  SALCOL_ITEM_FG_FOCSEL             = 9;
  SALCOL_ITEM_FG_HIGHLIGHT          = 10;
  SALCOL_ITEM_BK_NORMAL             = 11;                                       // barvy pozadi polozek v panelu
  SALCOL_ITEM_BK_SELECTED           = 12;
  SALCOL_ITEM_BK_FOCUSED            = 13;
  SALCOL_ITEM_BK_FOCSEL             = 14;
  SALCOL_ITEM_BK_HIGHLIGHT          = 15;
  SALCOL_ICON_BLEND_SELECTED        = 16;                                       // barvy pro blend ikonek
  SALCOL_ICON_BLEND_FOCUSED         = 17;
  SALCOL_ICON_BLEND_FOCSEL          = 18;
  SALCOL_PROGRESS_FG_NORMAL         = 19;                                       // barvy progress bary
  SALCOL_PROGRESS_FG_SELECTED       = 20;
  SALCOL_PROGRESS_BK_NORMAL         = 21;
  SALCOL_PROGRESS_BK_SELECTED       = 22;
  SALCOL_HOT_PANEL                  = 23;                                       // barva hot polozky v panelu
  SALCOL_HOT_ACTIVE                 = 24;                                       //                   v aktivnim window caption
  SALCOL_HOT_INACTIVE               = 25;                                       //                   v neaktivni caption, statusbar,...
  SALCOL_ACTIVE_CAPTION_FG          = 26;                                       // barva textu v aktivnim titulku panelu
  SALCOL_ACTIVE_CAPTION_BK          = 27;                                       // barva pozadi v aktivnim titulku panelu
  SALCOL_INACTIVE_CAPTION_FG        = 28;                                       // barva textu v neaktivnim titulku panelu
  SALCOL_INACTIVE_CAPTION_BK        = 29;                                       // barva pozadi v neaktivnim titulku panelu
  SALCOL_VIEWER_FG_NORMAL           = 30;                                       // barva textu v internim text/hex vieweru
  SALCOL_VIEWER_BK_NORMAL           = 31;                                       // barva pozadi v internim text/hex vieweru
  SALCOL_VIEWER_FG_SELECTED         = 32;                                       // barva oznaceneho textu v internim text/hex vieweru
  SALCOL_VIEWER_BK_SELECTED         = 33;                                       // barva oznaceneho pozadi v internim text/hex vieweru
  SALCOL_THUMBNAIL_NORMAL           = 34;                                       // barvy pera pro ramecek kolem thumbnail
  SALCOL_THUMBNAIL_SELECTED         = 35;
  SALCOL_THUMBNAIL_FOCUSED          = 36;
  SALCOL_THUMBNAIL_FOCSEL           = 37;

  // konstanty duvodu, proc metody CSalamanderGeneralAbstract::ChangePanelPathToXXX vratily neuspech:
  CHPPFR_SUCCESS          = 0;                                                  // v panelu je nova cesta, uspech (navratova hodnota je TRUE)
  // novou cestu (nebo jmeno archivu) nelze prevest z relativni na absolutni nebo
  // nova cesta (nebo jmeno archivu) neni pristupna nebo
  // cestu na FS nelze otevrit (neni plugin, odmita svuj load, odmita otevreni FS, fatalni chyba ChangePath)
  CHPPFR_INVALIDPATH      = 1;
  CHPPFR_INVALIDARCHIVE   = 2;                                                  // soubor neni archiv nebo se jako archiv neda vylistovat
  CHPPFR_LOWMEMORY        = 3;                                                  // nedostatek pameti pro nacteni obsahu archivu nebo FS
  CHPPFR_CANNOTCLOSEPATH  = 4;                                                  // aktualni cestu nelze uzavrit
  // v panelu je zkracena nova cesta,
  // upresneni pro FS: v panelu je bud zkracena nova cesta nebo puvodni cesta nebo zkracena
  // puvodni cesta - puvodni cesta se do panelu zkousi vratit jen pokud se nova cesta otevirala
  // v aktualnim FS (metoda IsOurPath pro ni vratila TRUE) a pokud nova cesta neni pristupna
  // (ani zadna jeji podcesta)
  CHPPFR_SHORTERPATH      = 5;
  // v panelu je zkracena nova cesta; duvodem zkraceni bylo to, ze pozadovana cesta byla jmeno
  // souboru - v panelu je cesta k souboru a soubor bude vyfokusen
  CHPPFR_FILENAMEFOCUSED  = 6;

  // konstanty typu parametru konfigurace Salamandera (viz CSalamanderGeneralAbstract::GetConfigParameter)
  SALCFGTYPE_NOTFOUND   = 0;                                                    // parameter not found
  SALCFGTYPE_BOOL       = 1;                                                    // TRUE/FALSE
  SALCFGTYPE_INT        = 2;                                                    // 32-bit integer
  SALCFGTYPE_STRING     = 3;                                                    // null-terminated multibyte string
  SALCFGTYPE_LOGFONT    = 4;                                                    // Win32 LOGFONT structure

  // konstanty parametru konfigurace Salamandera (viz CSalamanderGeneralAbstract::GetConfigParameter);
  // v komentari je uveden typ parametru (BOOL, INT, STRING), za STRING je v zavorce potrebna
  // velikost bufferu pro retezec
  //
  // general parameters
  SALCFG_SELOPINCLUDEDIRS         = 1;                                          // BOOL, select/deselect operations (num *, num +, num -) work also with directories
  SALCFG_SAVEONEXIT               = 2;                                          // BOOL, save configuration on Salamander exit
  SALCFG_MINBEEPWHENDONE          = 3;                                          // BOOL, should it beep (play sound) after end of work in inactive window?
  SALCFG_HIDEHIDDENORSYSTEMFILES  = 4;                                          // BOOL, should it hide system and/or hidden files?
  SALCFG_USEOPENNOTEXPLORE        = 5;                                          // BOOL, when opening spec. folder, use "open" or "explore"?
  SALCFG_ALWAYSONTOP              = 6;                                          // BOOL, main window is Always On Top?
  SALCFG_SORTUSESLOCALE           = 7;                                          // BOOL, should it use regional settings when sorting?
  SALCFG_SINGLECLICK              = 8;                                          // BOOL, single click mode (single click to open file, etc.)
  SALCFG_TOPTOOLBARVISIBLE        = 9;                                          // BOOL, is top toolbar visible?
  SALCFG_BOTTOMTOOLBARVISIBLE     = 10;                                         // BOOL, is bottom toolbar visible?
  SALCFG_USERMENUTOOLBARVISIBLE   = 11;                                         // BOOL, is user-menu toolbar visible?
  SALCFG_INFOLINECONTENT          = 12;                                         // STRING (200), content of Information Line (string with parameters)
  SALCFG_FILENAMEFORMAT           = 13;                                         // INT, how to alter file name before displaying (parameter 'format' to CSalamanderGeneralAbstract::AlterFileName)
  SALCFG_SAVEHISTORY              = 14;                                         // BOOL, may history related data be stored to configuration?
  SALCFG_ENABLECMDLINEHISTORY     = 15;                                         // BOOL, is command line history enabled?
  SALCFG_SAVECMDLINEHISTORY       = 16;                                         // BOOL, may command line history be stored to configuration?
  SALCFG_MIDDLETOOLBARVISIBLE     = 17;                                         // BOOL, is middle toolbar visible?
  SALCFG_SORTDETECTNUMBERS        = 18;                                         // BOOL, should it use numerical sort for numbers contained in strings when sorting?
  SALCFG_SORTBYEXTDIRSASFILES     = 19;                                         // BOOL, should it treat dirs as files when sorting by extension? (directories have no extensions, only files has extensions, but many people have requested sort by extension even for directories)
  SALCFG_SIZEFORMAT               = 20;                                         // INT, units for custom size columns, 0 - Bytes, 1 - KB, 2 - short (mixed B, KB, MB, GB, ...)
  // recycle bin parameters
  SALCFG_USERECYCLEBIN            = 50;                                         // INT, 0 - do not use, 1 - use for all, 2 - use for files matching at
                                                                                //      least one of masks (see SALCFG_RECYCLEBINMASKS)
  SALCFG_RECYCLEBINMASKS          = 51;                                         // STRING (MAX_PATH), masks for SALCFG_USERECYCLEBIN==2
  // time resolution of file compare (used in command Compare Directories)
  SALCFG_COMPDIRSUSETIMERES       = 60;                                         // BOOL, should it use time resolution? (FALSE==exact match)
  SALCFG_COMPDIRTIMERES           = 61;                                         // INT, time resolution for file compare (from 0 to 3600 second)
  // confirmations
  SALCFG_CNFRMFILEDIRDEL          = 70;                                         // BOOL, files or directories delete
  SALCFG_CNFRMNEDIRDEL            = 71;                                         // BOOL, non-empty directory delete
  SALCFG_CNFRMFILEOVER            = 72;                                         // BOOL, file overwrite
  SALCFG_CNFRMSHFILEDEL           = 73;                                         // BOOL, system or hidden file delete
  SALCFG_CNFRMSHDIRDEL            = 74;                                         // BOOL, system or hidden directory delete
  SALCFG_CNFRMSHFILEOVER          = 75;                                         // BOOL, system or hidden file overwrite
  SALCFG_CNFRMCREATEPATH          = 76;                                         // BOOL, show "do you want to create target path?" in Copy/Move operations
  SALCFG_CNFRMDIROVER             = 77;                                         // BOOL, directory overwrite (copy/move selected directory: ask user if directory already exists
                                                                                //                            on target path - standard behaviour is to join contents of both directories)
  // drive specific settings
  SALCFG_DRVSPECFLOPPYMON         = 88;                                         // BOOL, floppy disks - use automatic refresh (changes monitoring)
  SALCFG_DRVSPECFLOPPYSIM         = 89;                                         // BOOL, floppy disks - use simple icons
  SALCFG_DRVSPECREMOVABLEMON      = 90;                                         // BOOL, removable disks - use automatic refresh (changes monitoring)
  SALCFG_DRVSPECREMOVABLESIM      = 91;                                         // BOOL, removable disks - use simple icons
  SALCFG_DRVSPECFIXEDMON          = 92;                                         // BOOL, fixed disks - use automatic refresh (changes monitoring)
  SALCFG_DRVSPECFIXEDSIMPLE       = 93;                                         // BOOL, fixed disks - use simple icons
  SALCFG_DRVSPECREMOTEMON         = 94;                                         // BOOL, remote (network) disks - use automatic refresh (changes monitoring)
  SALCFG_DRVSPECREMOTESIMPLE      = 95;                                         // BOOL, remote (network) disks - use simple icons
  SALCFG_DRVSPECREMOTEDONOTREF    = 96;                                         // BOOL, remote (network) disks - do not refresh on activation of Salamander
  SALCFG_DRVSPECCDROMMON          = 97;                                         // BOOL, CDROM disks - use automatic refresh (changes monitoring)
  SALCFG_DRVSPECCDROMSIMPLE       = 98;                                         // BOOL, CDROM disks - use simple icons
  SALCFG_IFPATHISINACCESSIBLEGOTO = 99;                                         // STRING (MAX_PATH), path where to go if path in panel is inaccessible
  // internal text/hex viewer
  SALCFG_VIEWEREOLCRLF           = 120;                                         // BOOL, accept CR-LF ("\r\n") line ends?
  SALCFG_VIEWEREOLCR             = 121;                                         // BOOL, accept CR ("\r") line ends?
  SALCFG_VIEWEREOLLF             = 122;                                         // BOOL, accept LF ("\n") line ends?
  SALCFG_VIEWEREOLNULL           = 123;                                         // BOOL, accept NULL ("\0") line ends?
  SALCFG_VIEWERTABSIZE           = 124;                                         // INT, size of tab ("\t") character in spaces
  SALCFG_VIEWERSAVEPOSITION      = 125;                                         // BOOL, TRUE = save position of viewer window, FALSE = always use position of main window
  SALCFG_VIEWERFONT              = 126;                                         // LOGFONT, viewer font
  SALCFG_VIEWERWRAPTEXT          = 127;                                         // BOOL, wrap text (divide long text line to more lines)
  // archivers
  SALCFG_ARCOTHERPANELFORPACK    = 140;                                         // BOOL, should it pack to other panel path?
  SALCFG_ARCOTHERPANELFORUNPACK  = 141;                                         // BOOL, should it unpack to other panel path?
  SALCFG_ARCSUBDIRBYARCFORUNPACK = 142;                                         // BOOL, should it unpack to subdirectory named by archive?
  SALCFG_ARCUSESIMPLEICONS       = 143;                                         // BOOL, should it use simple icons in archives?

  // flagy pro textove vyhledavaci algoritmy (CSalamanderBMSearchData a CSalamanderREGEXPSearchData);
  // flagy se daji logicky scitat
  SASF_CASESENSITIVE  = $01;                                                    // velikost pismen je dulezita (pokud neni nastaven, hleda se bez ohledu na vel. pismen)
  SASF_FORWARD        = $02;                                                    // hledani smerem dopredu (pokud neni nastaven, hleda se smerem zpet)

  // ikony pro GetSalamanderIcon
  SALICON_EXECUTABLE       = 1;                                                 // exe/bat/pif/com
  SALICON_DIRECTORY        = 2;                                                 // dir
  SALICON_NONASSOCIATED    = 3;                                                 // neasociovany soubor
  SALICON_ASSOCIATED       = 4;                                                 // asociovany soubor
  SALICON_UPDIR            = 5;                                                 // up-dir ".."
  SALICON_ARCHIVE          = 6;                                                 // archiv

  // velikosti ikon pro GetSalamanderIcon
  SALICONSIZE_16           = 1;                                                 // 16x16
  SALICONSIZE_32           = 2;                                                 // 32x32
  SALICONSIZE_48           = 3;                                                 // 48x48

  // typy prikazu Salamandera pouzite v metode CSalamanderGeneralAbstract::EnumSalamanderCommands
  sctyUnknown                        = 0;
  sctyForFocusedFile                 = 1;                                       // jen pro focusly soubor (napr. View)
  sctyForFocusedFileOrDirectory      = 2;                                       // pro focusly soubor nebo adresar (napr. Open)
  sctyForSelectedFilesAndDirectories = 3;                                       // pro oznacene/fokusle soubory a adresare (napr. Copy)
  sctyForCurrentPath                 = 4;                                       // pro aktualni cestu v panelu (napr. Create Directory)
  sctyForConnectedDrivesAndFS        = 5;                                       // pro pripojene svazky a FS (napr. Disconnect)

  // prikazy Salamandera pouzite v metodach CSalamanderGeneralAbstract::EnumSalamanderCommands
  // a CSalamanderGeneralAbstract::PostSalamanderCommand
  // (POZOR: pro cisla prikazu je rezerovan jen interval <0, 499>)
  SALCMD_VIEW            = 0;                                                   // view (klavesa F3 v panelu)
  SALCMD_ALTVIEW         = 1;                                                   // alternate view (klavesa Alt+F3 v panelu)
  SALCMD_VIEWWITH        = 2;                                                   // view with (klavesa Ctrl+Shift+F3 v panelu)
  SALCMD_EDIT            = 3;                                                   // edit (klavesa F4 v panelu)
  SALCMD_EDITWITH        = 4;                                                   // edit with (klavesa Ctrl+Shift+F4 v panelu)

  SALCMD_OPEN            = 20;                                                  // open (klavesa Enter v panelu)
  SALCMD_QUICKRENAME     = 21;                                                  // quick rename (klavesa F2 v panelu)

  SALCMD_COPY            = 40;                                                  // copy (klavesa F5 v panelu)
  SALCMD_MOVE            = 41;                                                  // move/rename (klavesa F6 v panelu)
  SALCMD_EMAIL           = 42;                                                  // email (klavesa Ctrl+E v panelu)
  SALCMD_DELETE          = 43;                                                  // delete (klavesa Delete v panelu)
  SALCMD_PROPERTIES      = 44;                                                  // show properties (klavesa Alt+Enter v panelu)
  SALCMD_CHANGECASE      = 45;                                                  // change case (klavesa Ctrl+F7 v panelu)
  SALCMD_CHANGEATTRS     = 46;                                                  // change attributes (klavesa Ctrl+F2 v panelu)
  SALCMD_OCCUPIEDSPACE   = 47;                                                  // calculate occupied space (klavesa Alt+F10 v panelu)

  SALCMD_EDITNEWFILE     = 70;                                                  // edit new file (klavesa Shift+F4 v panelu)
  SALCMD_REFRESH         = 71;                                                  // refresh (klavesa Ctrl+R v panelu)
  SALCMD_CREATEDIRECTORY = 72;                                                  // create directory (klavesa F7 v panelu)
  SALCMD_DRIVEINFO       = 73;                                                  // drive info (klavesa Ctrl+F1 v panelu)
  SALCMD_CALCDIRSIZES    = 74;                                                  // calculate directory sizes (klavesa Ctrl+Shift+F10 v panelu)

  SALCMD_DISCONNECT      = 90;                                                  // disconnect (network drive or plugin-fs) (klavesa F12 v panelu)

  MAX_GROUPMASK = 1001;                                                         // max. pocet znaku (vcetne nuly na konci) ve skupinove masce

  // identifikatory sdilenych historii (posledne pouzitych hodnot v comboboxech) pro
  // CSalamanderGeneral::GetStdHistoryValues()
  SALHIST_QUICKRENAME    = 1;                                                   // jmena v Quick Rename dialogu (F2)
  SALHIST_COPYMOVETGT    = 2;                                                   // cilove cesty v Copy/Move dialogu (F5/F6)
  SALHIST_CREATEDIR      = 3;                                                   // jmena adresaru v Create Directory dialogu (F7)
  SALHIST_CHANGEDIR      = 4;                                                   // cesty v Change Directory dialogu (Shift+F7)
  SALHIST_EDITNEW        = 5;                                                   // jmena v Edit New dialogu (Shift+F4)
  SALHIST_CONVERT        = 6;                                                   // jmena v Conver dialogu (Ctrl+K)

  // rezimy pro metodu CSalamanderGeneralAbstract::ExpandPluralFilesDirs
  epfdmNormal    = 0;                                                           // XXX files and YYY directories
  epfdmSelected  = 1;                                                           // XXX selected files and YYY selected directories
  epfdmHidden    = 2;                                                           // XXX hidden files and YYY hidden directories

type
//
// ****************************************************************************
// CSalamanderGeneralAbstract
//
// obecne pouzitelne metody Salamandera (pro vsechny typy pluginu)

  TMsgBoxExCallback = procedure( helpInfo : PHelpInfo ); stdcall;

  TMsgBoxExParams = packed record
    HParent       : HWND;
    Text          : PChar;
    Caption       : PChar;
    Flags         : DWORD;
    HIcon         : HICON;
    ContextHelpId : DWORD;
    HelpCallback  : TMsgBoxExCallback;
    CheckBoxText  : PChar;
    CheckBoxValue : PBOOL;
    AliasBtnNames : PChar;
  end;
  PMsgBoxExParams = ^TMsgBoxExParams;

  (*
  HParent
    Handle to the owner window. Message box is centered to this window.
    If this parameter is NULL, the message box has no owner window.

  Text
    Pointer to a null-terminated string that contains the message to be displayed.

  Caption
    Pointer to a null-terminated string that contains the message box title.
    If this member is NULL, the default title "Error" is used.

  Flags
    Specifies the contents and behavior of the message box.
    This parameter can be a combination of flags from the following groups of flags.

     To indicate the buttons displayed in the message box, specify one of the following values.
      MSGBOXEX_OK                   (MB_OK)
        The message box contains one push button: OK. This is the default.
        Message box can be closed using Escape and return value will be DIALOG_OK (IDOK).
      MSGBOXEX_OKCANCEL             (MB_OKCANCEL)
        The message box contains two push buttons: OK and Cancel.
      MSGBOXEX_ABORTRETRYIGNORE     (MB_ABORTRETRYIGNORE)
        The message box contains three push buttons: Abort, Retry, and Ignore.
      MSGBOXEX_YESNOCANCEL          (MB_YESNOCANCEL)
        The message box contains three push buttons: Yes, No, and Cancel.
      MSGBOXEX_YESNO                (MB_YESNO)
        The message box contains two push buttons: Yes and No.
        Message box can be closed using Escape when MSGBOXEX_ESCAPEENABLED flag is specified.
        In that case return value will be DIALOG_NO (IDNO).
      MSGBOXEX_RETRYCANCEL          (MB_RETRYCANCEL)
        The message box contains two push buttons: Retry and Cancel.
      MSGBOXEX_CANCELTRYCONTINUE    (MB_CANCELTRYCONTINUE)
        The message box contains three push buttons: Cancel, Try Again, Continue.

     To display an icon in the message box, specify one of the following values.
      MSGBOXEX_ICONHAND             (MB_ICONHAND / MB_ICONSTOP / MB_ICONERROR)
        A stop-sign icon appears in the message box.
      MSGBOXEX_ICONQUESTION         (MB_ICONQUESTION)
        A question-mark icon appears in the message box.
      MSGBOXEX_ICONEXCLAMATION      (MB_ICONEXCLAMATION / MB_ICONWARNING)
        An exclamation-point icon appears in the message box.
      MSGBOXEX_ICONINFORMATION      (MB_ICONASTERISK / MB_ICONINFORMATION)
        An icon consisting of a lowercase letter i in a circle appears in the message box.

     To indicate the default button, specify one of the following values.
      MSGBOXEX_DEFBUTTON1           (MB_DEFBUTTON1)
        The first button is the default button.
        MSGBOXEX_DEFBUTTON1 is the default unless MSGBOXEX_DEFBUTTON2, MSGBOXEX_DEFBUTTON3,
        or MSGBOXEX_DEFBUTTON4 is specified.
      MSGBOXEX_DEFBUTTON2           (MB_DEFBUTTON2)
        The second button is the default button.
      MSGBOXEX_DEFBUTTON3           (MB_DEFBUTTON3)
        The third button is the default button.
      MSGBOXEX_DEFBUTTON4           (MB_DEFBUTTON4)
        The fourth button is the default button.

     To specify other options, use one or more of the following values.
      MSGBOXEX_HELP                 (MB_HELP)
        Adds a Help button to the message box.
        When the user clicks the Help button or presses F1, the system sends a WM_HELP message to the owner
        or calls HelpCallback (see HelpCallback for details).
      MSGBOXEX_SETFOREGROUND        (MB_SETFOREGROUND)
        The message box becomes the foreground window. Internally, the system calls the SetForegroundWindow
        function for the message box.
      MSGBOXEX_SILENT
        No sound will be played when message box is displayed.
      MSGBOXEX_ESCAPEENABLED
        When MSGBOXEX_YESNO is specified, user can close message box using Escape key and DIALOG_NO (IDNO)
        will be returned. Otherwise this option is ignored.

  HIcon
    Handle to the icon to be drawn in the message box. Icon will not be destroyed when messagebox is closed.
    If this parameter is NULL, MSGBOXEX_ICONxxx style will be used.

  ContextHelpId
    Identifies a help context. If a help event occurs, this value is specified in
    the HELPINFO structure that the message box sends to the owner window or callback function.

  HelpCallback
    Pointer to the callback function that processes help events for the message box.
    The callback function has the following form:
      VOID CALLBACK MSGBOXEX_CALLBACK(LPHELPINFO helpInfo)
    If this member is NULL, the message box sends WM_HELP messages to the owner window
    when help events occur.

  CheckBoxText
    Pointer to a null-terminated string that contains the checkbox text.
    If the MSGBOXEX_HINT flag is specified in the Flags, this text must contain HINT.
    Hint is separated from string by the TAB character. Hint is divided by the second TAB character
    on two parts. The first part is label, that will be displayed behind the check box.
    The second part is the text displayed when user clicks the hint label.

    Example: "This is text for checkbox\tHint Label\tThis text will be displayed when user click the Hint Label."
    If this member is NULL, checkbox will not be displayed.

  CheckBoxValue
    Pointer to a BOOL variable contains the checkbox initial and return state (TRUE:checked, FALSE: unchecked).
    This parameter is ignored if CheckBoxText parameter is NULL. Otherwise this parameter must be set.

  AliasBtnNames
    Pointer to a buffer containing pairs of id and alias strings. The last string in the
    buffer must be terminated by NULL character.

    The first string in each pair is a decimal number that specifies button ID.
    Number must be one of the DIALOG_xxx values. The second string specifies alias text
    for this button.

    First and second string in each pair are separated by TAB character.
    Pairs are separated by TAB character too.

    If this member is NULL, normal names of buttons will displayed.

    Example: sprintf(buffer, "%d\t%s\t%d\t%s", DIALOG_OK, "&Start", DIALOG_CANCEL, "E&xit");
             buffer: "1\t&Start\t2\tE&xit"
  *)

  // typy pro CSalamanderGeneralAbstract::ValidateVarString() a CSalamanderGeneralAbstract::ExpandVarString()
  TSalamanderVarStrGetValueFunc = function( msgParent : HWND; param : Pointer ) : PChar; stdcall;
  TSalamanderVarStrEntry = packed record
    Name    : PChar;                                                            // jmeno promenne v retezci (napr. u retezce "$(name)" je to "name")
    Execute : TSalamanderVarStrGetValueFunc;                                    // funkce, ktera vraci text reprezentujici promennou
  end;
  PSalamanderVarStrEntry = ^TSalamanderVarStrEntry;

  // typ call-backu pouzivany pri load/save konfigurace pomoci
  // CSalamanderGeneral::CallLoadOrSaveConfiguration; 'regKey' je NULL pokud jde o load
  // defaultni konfigurace (save se pri 'regKey' == NULL nevola); 'registry' je objekt pro
  // praci s registry; 'param' je uzivatelsky parametr funkce (viz
  // CSalamanderGeneral::CallLoadOrSaveConfiguration)
  TSalLoadOrSaveConfigurationProc = procedure( load : BOOL; regKey : HKEY;
                                               registry : TSalamanderRegistryAbstract;
                                               param : Pointer ); stdcall;

  // zaklad struktury pro CSalamanderGeneralAbstract::ViewFileInPluginViewer (kazdy plugin
  // viewer muze mit tuto strukturu rozsirenou o sve parametry - struktura se predava do
  // CPluginInterfaceForViewerAbstract::ViewFile - parametry muzou byt napr. titulek okna,
  // mod vieweru, offset od zacatku souboru, pozice oznaceni, atp.); POZOR !!! na pakovani
  // struktur (pozadovane je 4 byty - viz "#pragma pack(4)")
  TSalamanderPluginViewerData = packed record
    // kolik bytu od zacatku struktury je platnych (pro rozliseni verzi struktury)
    Size     : Integer;
    // jmeno souboru, ktery se ma otevrit ve viewru (nepouzivat v metode
    // CPluginInterfaceForViewerAbstract::ViewFile - jmeno souboru je dano parametrem 'name')
    FileName : PChar;
  end;
  PSalamanderPluginViewerData = ^TSalamanderPluginViewerData;

  // rozsireni struktury CSalamanderPluginViewerData pro interni text/hex viewer
  TSalamanderPluginInternalViewerData = packed record
    Mode         : Integer;                                                     // 0 - textovy mod, 1 - hexa mod
    Caption      : PChar;                                                       // NULL -> obsahuje caption okna FileName, jinak Caption
    WholeCaption : BOOL;                                                        // ma vyznam pokud je Caption != NULL. TRUE -> v titulku
                                                                                // vieweru bude zobrazen pouze retezec Caption; FALSE -> za
                                                                                // Caption se pripoji standardni " - Viewer".
  end;

  // typ callbacku pouzivany v metode CSalamanderGeneral::SalSplitGeneralPath
  TSGP_IsTheSamePathFunc = function( path1 : PChar; path2 : PChar ) : BOOL; stdcall;

  // typ callbacku pouzivany v metode CSalamanderGeneralAbstract::CallPluginOperationFromDisk
  // 'sourcePath' je zdrojova cesta na disku (ostatni cesty jsou od ni vztazeny relativne);
  // oznacene soubory/adresare jsou zadany enumeracni funkci 'next' jejimz parametrem je
  // 'nextParam'; 'param' je parametr predavany do CallPluginOperationFromDisk jako 'param'
  TSalPluginOperationFromDiskProc = procedure( sourcePath : PChar; next : TSalEnumSelection2Func;
                                               nextParam : Pointer; param : Pointer ); stdcall;

  // interface objektu Boyer-Moorova algoritmu pro vyhledavani v textu
  // POZOR: kazdy alokovany objekt je mozne pouzivat jen v ramci jednoho threadu
  // (nemusi jit o hlavni thread, nemusi jit u vsech objektu o jeden thread)
  TSalamanderBMSearchData = class
    // nastaveni vzorku; 'pattern' je null-terminated text vzorku; 'flags' jsou priznaky
    // algoritmu (viz konstanty SASF_XXX)
    procedure Set_( pattern : PChar; flags : Word ); overload; virtual; stdcall; abstract;

    // nastaveni vzorku; 'pattern' je binarni vzorek o delce 'length' (buffer 'pattern' musi
    // mit delku alespon ('length' + 1) znaku - jen pro kompatibilitu s textovymi vzorky);
    // 'flags' jsou priznaky algoritmu (viz konstanty SASF_XXX)
    procedure Set_( pattern : PChar; const length : Integer; flags : Word ); overload; virtual; stdcall; abstract;

    // nastaveni priznaku algoritmu; 'flags' jsou priznaky algoritmu (viz konstanty SASF_XXX)
    procedure SetFlags( flags : Word ); virtual; stdcall; abstract;

    // vraci delku vzorku (pouzitelne az po uspesnem volani metody Set)
    function GetLength : Integer; virtual; stdcall; abstract;

    // vraci vzorek (pouzitelne az po uspesnem volani metody Set)
    function GetPattern : PChar; virtual; stdcall; abstract;

    // vraci TRUE pokud je mozne zacit vyhledavat (vzorek i priznaky byly uspesne nastaveny,
    // neuspech hrozi jen pri nedostatku pameti nebo prazdnem vzorku)
    function IsGood : BOOL; virtual; stdcall; abstract;

    // hledani vzorku v textu 'text' o delce 'length' od offsetu 'start' smerem dopredu;
    // vraci offset nalezeneho vzorku nebo -1 pokud vzorek nebyl nalezen;
    // POZOR: algoritmus musi mit nastaveny priznak SASF_FORWARD
    function SearchForward( text : PChar; length : Integer; start : Integer ) : Integer; virtual; stdcall; abstract;

    // hledani vzorku v textu 'text' o delce 'length' smerem zpet (zacina hledat na konci textu);
    // vraci offset nalezeneho vzorku nebo -1 pokud vzorek nebyl nalezen;
    // POZOR: algoritmus nesmi mit nastaveny priznak SASF_FORWARD
    function SearchBackward( text : PChar; length : Integer ) : Integer; virtual; stdcall; abstract;
  end;

  // interface objektu algoritmu pro vyhledavani pomoci regularnich vyrazu v textu
  // POZOR: kazdy alokovany objekt je mozne pouzivat jen v ramci jednoho threadu
  // (nemusi jit o hlavni thread, nemusi jit u vsech objektu o jeden thread)
  TSalamanderREGEXPSearchData = class
    // nastaveni regularniho vyrazu; 'pattern' je null-terminated text regularniho vyrazu; 'flags'
    // jsou priznaky algoritmu (viz konstanty SASF_XXX); pri chybe vraci FALSE a popis chyby
    // je mozne ziskat volanim metody GetLastErrorText
    function Set_( pattern : PChar; flags : Word ) : BOOL; virtual; stdcall; abstract;

    // nastaveni priznaku algoritmu; 'flags' jsou priznaky algoritmu (viz konstanty SASF_XXX);
    // pri chybe vraci FALSE a popis chyby je mozne ziskat volanim metody GetLastErrorText
    function SetFlags( flags : Word ) : BOOL; virtual; stdcall; abstract;

    // vraci text chyby, ktera nastala v poslednim volani Set nebo SetFlags (muze byt i NULL)
    function GetLastErrorText : PChar; virtual; stdcall; abstract;

    // vraci text regularniho vyrazu (pouzitelne az po uspesnem volani metody Set)
    function GetPattern : PChar; virtual; stdcall; abstract;

    // nastaveni radky textu (radka je od 'start' do 'end', 'end' ukazuje za posledni znak radky),
    // ve kterem se vyhledava; pri chybe (hrozi jen nedostatek pameti) vraci FALSE a popis chyby
    // je mozne ziskat volanim metody GetLastErrorText
    function SetLine( start : PChar; _end : PChar ) : BOOL; virtual; stdcall; abstract;

    // hledani podretezce odpovidajiciho regularnimu vyrazu v radce nastavene metodou SetLine;
    // hleda od offsetu 'start' smerem dopredu; vraci offset nalezeneho podretezce a jeho delku
    // (ve 'foundLen') nebo -1 pokud podretezec nebyl nalezen;
    // POZOR: algoritmus musi mit nastaveny priznak SASF_FORWARD
    function SearchForward( start : Integer; var foundLen : Integer ) : Integer; virtual; stdcall; abstract;

    // hledani podretezce odpovidajiciho regularnimu vyrazu v radce nastavene metodou SetLine;
    // hleda smerem zpet (zacina hledat na konci textu o delce 'length' od zacatku radky);
    // vraci offset nalezeneho podretezce a jeho delku (ve 'foundLen') nebo -1 pokud podretezec
    // nebyl nalezen;
    // POZOR: algoritmus nesmi mit nastaveny priznak SASF_FORWARD
    function SearchBackward( length : Integer; var foundLen : Integer ) : Integer; virtual; stdcall; abstract;
  end;

  // interface objektu pro praci se skupinou souborovych masek
  // POZOR: metody objektu nejsou synchronizovane, takze je mozne je pouzivat jen
  //        v ramci jednoho threadu (nemusi jit o hlavni thread) nebo si jejich
  //        synchronizaci musi zajistit plugin (nesmi se provadet "zapis" behem
  //        provadeni jine metody; "zapis"=SetMasksString+PrepareMasks;
  //        provadeni "cteni" je mozne z vice threadu najednou; "cteni"=GetMasksString+
  //        AgreeMasks)
  //
  // Zivotni cyklus objektu:
  //   1) Alokujeme metodou CSalamanderGeneralAbstract::AllocSalamanderMaskGroup
  //   2) V metode SetMasksString predame skupinu masek.
  //   3) Zavolame PrepareMasks pro stavbu vnitrnich dat; v pripade neuspechu
  //      zobrazime chybne misto a po oprave masky se vracime do bodu (3)
  //   4) Libovolne volame AgreeMasks pro zjisteni, zda jmeno odpovida skupine masek.
  //   5) Po pripadnem zavolani SetMasksString pokracujeme od (3)
  //   6) Destrukce objektu metodou CSalamanderGeneralAbstract::FreeSalamanderMaskGroup
  //
  // Maska:
  //   '?' - libovolny znak
  //   '*' - libovolny retezec (i prazdny)
  //   '#' - libovolna cislice (pouze je-li 'extendedMode'==TRUE)
  //
  //   Priklady:
  //     *     - vsechna jmena
  //     *.*   - vsechna jmena
  //     *.exe - jmena s priponou "exe"
  //     *.t?? - jmena s priponou zacinajici znakem 't' a obsahujici jeste dva libovolne znaky
  //     *.r## - jmena s priponou zacinajici znakem 'r' a obsahujici jeste dve libovolne cislice
  //
  TSalamanderMaskGroup = class
    // nastavi retezec masek (masky jsou oddelene ';' (escape sekvence pro ';' je ";;"));
    // 'masks' je retezec masek (max. delka vcetne nuly na konci je MAX_GROUPMASK)
    // pokud je 'extendedMode' rovno TRUE, znak '#' odpovida libovolne cislici ('0'-'9')
    // jako oddelovac lze pouzit znak '|'; nasledujici masky (uz zase oddelene ';')
    // budou vyhodnocovany inverzne, tedy pokud jim bude odpovidat jmeno,
    // AgreeMasks vrati FALSE; znak '|' muze stat na zacatku retezce
    //
    //   Priklady:
    //     *.txt;*.cpp - vsechna jmena s priponou txt nebo cpp
    //     *.h*|*.html - vsechna jmena s priponou zacinajici znakem 'h', ale ne jmena s priponou "html"
    //     |*.txt      - vsechna jmena s jinou priponou nez "txt"
    procedure SetMasksString( masks : PChar; extendedMode : BOOL ); virtual; stdcall; abstract;

    // vraci retezec masek; 'buffer' je buffer o delce alespon MAX_GROUPMASK
    procedure GetMasksString( buffer : PChar ); virtual; stdcall; abstract;

    // vraci 'extendedMode' nastaveny v metode SetMasksString
    function GetExtendedMode : BOOL; virtual; stdcall; abstract;

    // prace se souborovymi maskami: ('?' lib. znak, '*' lib. retezec - i prazdny, pokud
    //  byl 'extendedMode' v metode SetMasksString TRUE, '#' lib. cislice - '0'..'9'):
    // 1) prevedeme masky do jednodussiho formatu; 'errorPos' vraci pozici chyby v retezci masek;
    //    vraci TRUE pokud nenastala chyba (vraci FALSE -> je nastaveno 'errorPos'; pri nedostatku
    //    pameti je 'errorPos'==0)
    function PrepareMasks( var errorPos : Integer ) : BOOL; virtual; stdcall; abstract;
    // 2) muzeme pouzit prevedene masky na test, jestli nektere z nich odpovida soubor 'filename';
    //    'fileExt' ukazuje bud na konec 'fileName' nebo na priponu (pokud existuje), 'fileExt'
    //    muze byt NULL (pripona se dohledava podle std. pravidel); vraci TRUE pokud soubor
    //    odpovida alespon jedne z masek
    function AgreeMasks( fileName : PChar; fileExt : PChar ) : BOOL; virtual; stdcall; abstract;
  end;

  // interface objektu pro praci vypocet MD5
  //
  // Zivotni cyklus objektu:
  //
  //   1) Alokujeme metodou CSalamanderGeneralAbstract::AllocSalamanderMD5
  //   2) Postupne volame metodu Update() pro data, pro ktere chceme vypocitat MD5
  //   3) Zavolame metodu Finalize()
  //   4) Vyzvedneme vypocitanou MD5 pomoci metody GetDigest()
  //   5) Pokud chceme objekt znovu pouzit, zavolame metodu Init()
  //      (vola se automaticky v kroku (1)) a jdeme do kroku (2)
  //   6) Destrukce objektu metodou CSalamanderGeneralAbstract::FreeSalamanderMD5
  //
  TSalamanderMD5 = class
    // inicializace objektu, je automaticky zavolana v konstruktoru
    // metoda je publikovana pro vicenasobne pouziti alokovaneho objektu
    procedure Init; virtual; stdcall; abstract;

    // aktualizuje vnitrni stav objektu na zaklade bloku dat urceneho promennou 'input',
    // 'input_length' udava velikost bufferu v bajtech
    procedure Update( input : Pointer; input_length : DWORD ); virtual; stdcall; abstract;

    // pripravi MD5 pro vyzvednuti pomoci metody GetDigest
    // po zavolani metody Finalize lze volat pouze metodu GetDigest() a Init()
    procedure Finalize; virtual; stdcall; abstract;

    // vyzvedne MD5, 'dest' musi ukazovat do bufferu o velikosti 16 bajtu
    // metodu lze volat pouze po zavolani metody Finalize()
    procedure GetDigest( dest : Pointer ); virtual; stdcall; abstract;
  end;

  // prikazy pro HTML help: viz metoda CSalamanderGeneralAbstract::OpenHtmlHelp
  THtmlHelpCommand = (
                      HHCDisplayTOC,                                            // viz HH_DISPLAY_TOC: dwData = 0 (zadny topic) nebo: pointer to a topic within a compiled help file
                      HHCDisplayIndex,                                          // viz HH_DISPLAY_INDEX: dwData = 0 (zadny keyword) nebo: keyword to select in the index (.hhk) file
                      HHCDisplaySearch,                                         // viz HH_DISPLAY_SEARCH: dwData = 0 (prazdne hledani) nebo: pointer to an HH_FTS_QUERY structure
                      HHCDisplayContext                                         // viz HH_HELP_CONTEXT: dwData = numeric ID of the topic to display
                     );

  TSalamanderGeneralAbstract = class
    // zobrazi message-box se zadanym textem a titulkem, parent message-boxu je HWND
    // vracene metodou GetMsgBoxParent() (viz nize); pouziva SalMessageBox (viz nize)
    // type = MSGBOX_INFO        - informace (ok)
    // type = MSGBOX_ERROR       - chybova hlaska (ok)
    // type = MSGBOX_EX_ERROR    - chybova hlaska (ok/cancel) - vraci IDOK, IDCANCEL
    // type = MSGBOX_QUESTION    - dotaz (yes/no) - vraci IDYES, IDNO
    // type = MSGBOX_EX_QUESTION - dotaz (yes/no/cancel) - vraci IDYES, IDNO, IDCANCEL
    // type = MSGBOX_WARNING     - varovani (ok)
    // type = MSGBOX_EX_WARNING  - varovani (yes/no/cancel) - vraci IDYES, IDNO, IDCANCEL
    // v pripade chyby vraci 0
    // omezeni: hlavni thread
    function ShowMessageBox( text : PChar; title : PChar; _type : Integer ) : Integer; virtual; stdcall; abstract;

    // SalMessageBox a SalMessageBoxEx vytvori, zobrazi a po zvoleni nektereho z tlacitek
    // zavrou message box. Message box muze obsahovat uzivatelem definovany titulek, zpravu,
    // tlacitka, ikonu, checkbox s nejakym textem.
    //
    // Pokud neni 'hParent' prave foreground window (msgbox v neaktivni aplikaci), dojde
    // pred vybalenim msgboxu k volani FlashWindow(mainwnd, TRUE) a po zavreni msgboxu
    // k volani FlashWindow(mainwnd, FALSE) (druhe volani FlashWindow je nutne kvuli
    // Win9x), mainwnd je v rade parentu okna 'hParent' to, ktere uz nema parenta (typicky
    // jde o hlavni okno Salamandera).
    //
    // SalMessageBox naplni strukturu MSGBOXEX_PARAMS (hParent->HParent, lpText->Text,
    // lpCaption->Caption a uType->Flags; ostatni polozky struktury vynuluje) a vola
    // SalMessageBoxEx, takze dale budeme popisovat pouze SalMessageBoxEx.
    //
    // SalMessageBoxEx se snazi v maximalni mire chovat stejne jako Windows API funkce
    // MessageBox a MessageBoxIndirect. Rozdily jsou:
    //   - message box je centrovany k hParent (pokud jde o child okno, dohleda se non-child parent)
    //   - v pripade MB_YESNO message boxu lze povolit zavreni okna klavesou Escape nebo
    //     kliknutim na krizek v titulku (flag MSGBOXEX_ESCAPEENABLED); navratova hodnota
    //     pak bude IDNO
    //   - lze potlacit pipnuti (flag MSGBOXEX_SILENT)
    //
    // Komentar k uType viz komentar k MSGBOXEX_PARAMS::Flags
    //
    // Return Values
    //    DIALOG_FAIL       (0)            The function fails.
    //    DIALOG_OK         (IDOK)         'OK' button was selected.
    //    DIALOG_CANCEL     (IDCANCEL)     'Cancel' button was selected.
    //    DIALOG_ABORT      (IDABORT)      'Abort' button was selected.
    //    DIALOG_RETRY      (IDRETRY)      'Retry' button was selected.
    //    DIALOG_IGNORE     (IDIGNORE)     'Ignore' button was selected.
    //    DIALOG_YES        (IDYES)        'Yes' button was selected.
    //    DIALOG_NO         (IDNO)         'No' button was selected.
    //    DIALOG_TRYAGAIN   (IDTRYAGAIN)   'Try Again' button was selected.
    //    DIALOG_CONTINUE   (IDCONTINUE)   'Continue' button was selected.
    //    DIALOG_SKIP                      'Skip' button was selected.
    //    DIALOG_SKIPALL                   'Skip All' button was selected.
    //    DIALOG_ALL                       'All' button was selected.
    //
    // SalMessageBox a SalMessageBoxEx je mozne volat z libovolneho threadu
    function SalMessageBox( hParent : HWND; lpText : PChar; lpCaption : PChar;
                            uType : UINT ) : Integer; virtual; stdcall; abstract;
    function SalMessageBoxEx( const params : PMsgBoxExParams ) : Integer; virtual; stdcall; abstract;

    // vraci HWND vhodneho parenta pro otevirane message-boxy (nebo jina modalni okna),
    // jde o hlavni okno, progress-dialog, Plugins/Plugins dialog nebo jine k hlavnimu oknu
    // otevrene modalni okno
    // omezeni: hlavni thread, vracene HWND je vzdy z hlavniho threadu
    function GetMsgBoxParent : HWND; virtual; stdcall; abstract;

    // vraci handle hlavniho okna Salamandera
    // mozne volat z libovolneho threadu
    function GetMainWindowHWND : HWND; virtual; stdcall; abstract;

    // obnovi fokus v panelu nebo v command line (podle toho co bylo naposledy aktivovano); toto
    // volani je potreba, pokud plugin disabluje/enabluje hlavni okno Salamandera (vznikaji tak
    // situace, kdy se aktivuje disablovane hl. okno - v disablovanem okne nelze nastavit
    // fokus - po enablovani hl. okna je treba fokus obnovit touto metodou)
    procedure RestoreFocusInSourcePanel; virtual; stdcall; abstract;

    // casto pouzivane dialogy, parent dialogu 'parent', navratove hodnoty DIALOG_XXX;
    // pokud neni 'parent' prave foreground window (dialog v neaktivni aplikaci), dojde
    // pred vybalenim dialogu k volani FlashWindow(mainwnd, TRUE) a po zavreni dialogu
    // k volani FlashWindow(mainwnd, FALSE) (druhe volani FlashWindow je nutne kvuli
    // Win9x), mainwnd je v rade parentu okna 'parent' to, ktere uz nema parenta (typicky
    // jde o hlavni okno Salamandera)
    // ERROR: filename+error+title (pokud 'title' == NULL, jde o std. titulek "Error")
    //
    // Promenna 'flags' urcuje zobrazena tlacitka, pro DialogError lze pouzit jednu z hodnot:
    // BUTTONS_OK               // OK                                    (old DialogError3)
    // BUTTONS_RETRYCANCEL      // Retry / Cancel                        (old DialogError4)
    // BUTTONS_SKIPCANCEL       // Skip / Skip all / Cancel              (old DialogError2)
    // BUTTONS_RETRYSKIPCANCEL  // Retry / Skip / Skip all / Cancel      (old DialogError)
    //
    // vse je mozne volat z libovolneho threadu
    function DialogError( parent : HWND; flags : DWORD; fileName : PChar; error : PChar;
                          title : PChar ) : Integer; virtual; stdcall; abstract;

    // CONFIRM FILE OVERWRITE: filename1+filedata1+filename2+filedata2
    // Promenna 'flags' urcuje zobrazena tlacitka, pro DialogOverwrite lze pouzit jednu z hodnot:
    // BUTTONS_YESALLSKIPCANCEL // Yes / All / Skip / Skip all / Cancel  (old DialogOverwrite)
    // BUTTONS_YESNOCANCEL      // Yes / No / Cancel                     (old DialogOverwrite2)
    function DialogOverwrite( parent : HWND; flags : DWORD; fileName1 : PChar; fileData1 : PChar;
                              fileName2 : PChar; fileData2 : PChar ) : Integer; virtual; stdcall; abstract;

    // QUESTION: filename+question+title (pokud 'title' == NULL, jde o std. titulek "Question")
    // Promenna 'flags' urcuje zobrazena tlacitka, pro DialogQuestion lze pouzit jednu z hodnot:
    // BUTTONS_YESALLSKIPCANCEL // Yes / All / Skip / Skip all / Cancel  (old DialogQuestion)
    // BUTTONS_YESNOCANCEL      // Yes / No / Cancel                     (old DialogQuestion2)
    // BUTTONS_YESALLCANCEL     // Yes / All / Cancel                    (old DialogQuestion3)
    function DialogQuestion( parent : HWND; flags : DWORD; fileName : PChar;
                             question : PChar; title : PChar ) : Integer; virtual; stdcall; abstract;

    // pokud cesta 'dir' neexistuje, umozni ji vytvorit (zepta se usera; vytvari pripadne
    // i vice adresaru na konci cesty); pokud cesta existuje nebo je uspesne vytvorena vraci TRUE;
    // pokud cesta neexistuje a je-li 'quiet' TRUE, nepta se usera jestli chce vytvaret
    // cestu 'dir'; pokud je 'errBuf' NULL, ukazuje chyby v oknech; pokud 'errBuf' neni NULL,
    // dava popis chyb do bufferu 'errBuf' o velikosti 'errBufSize' (zadna chybova okna se
    // neoteviraji); vsechny otevirana okna maji za parenta 'parent', je-li 'parent' NULL,
    // pouziva se hlavni okno Salamandera; neni-li 'firstCreatedDir' NULL, jde o buffer
    // o velikosti MAX_PATH pro ulozeni plneho jmena prvniho vytvareneho adresare na ceste
    // 'dir' (vraci prazdny retezec pokud jiz cesta 'dir' existuje)
    // mozne volat z libovolneho threadu
    function CheckAndCreateDirectory( dir : PChar; parent : HWND = 0; quiet : BOOL = True;
                                      errBuf : PChar = nil; errBufSize : Integer = 0;
                                      firstCreatedDir : PChar = nil ) : BOOL; virtual; stdcall; abstract;

    // zjisti volne misto na ceste path a pokud neni >= totalSize zepta se jestli chce user pokracovat;
    // okno dotazu ma parenta 'parent', vraci TRUE pokud je dost mista nebo pokud uzivatel odpovedel
    // "pokracovat"; pokud neni 'parent' prave foreground window (dialog v neaktivni aplikaci), dojde
    // pred vybalenim dialogu k volani FlashWindow(mainwnd, TRUE) a po zavreni dialogu
    // k volani FlashWindow(mainwnd, FALSE) (druhe volani FlashWindow je nutne kvuli
    // Win9x), mainwnd je v rade parentu okna 'parent' to, ktere uz nema parenta (typicky
    // jde o hlavni okno Salamandera)
    // 'messageTitle' bude zobrazena v titulku messageboxu s dotazem a melo by se jednat
    // o nazev pluginu, ktery metodu zavolal
    // mozne volat z libovolneho threadu
    function TestFreeSpace( parent : HWND; path : PChar; var totalSize : TLargeInteger;
                            messageTitle : PChar ) : BOOL; virtual; stdcall; abstract;

    // vraci v 'retValue' (nesmi byt NULL) volne misto na dane ceste (zatim nejkorektnejsi
    // udaj, jaky se da z Windows ziskat, na NT umi pracovat i s junction-pointy); 'path'
    // je cesta, na ktere zjistujeme volne misto (nemusi byt koren); neni-li 'total' NULL,
    // vraci se v nem celkova velikost disku, pokud dojde k chybe, vraci CQuadWord(-1, -1)
    // mozne volat z libovolneho threadu
    procedure GetDiskFreeSpace( retValue : PLargeInteger; path : PChar; total : PLargeInteger ); virtual; stdcall; abstract;

    // vlastni klon windowsove GetDiskFreeSpace: umi zjistit korektni udaje i na FAT32
    // discich pod Win9x a pro cesty obsahujici junction-pointy pod Windows 2000/XP;
    // 'path' je cesta, na ktere zjistujeme volne misto; ostatni parametry odpovidaji
    // standardni Win32 API funkci GetDiskFreeSpace
    // mozne volat z libovolneho threadu
    function SalGetDiskFreeSpace( path : PChar; lpSectorsPerCluster : LPDWORD;
                                  lpBytesPerSector : LPDWORD; lpNumberOfFreeClusters : LPDWORD;
                                  lpTotalNumberOfClusters : LPDWORD ) : BOOL; virtual; stdcall; abstract;

    // vlastni klon windowsove GetVolumeInformation: umi zjistit korektni udaje i pro
    // cesty obsahujici junction-pointy pod Windows 2000/XP; 'path' je cesta, pro kterou
    // zjistujeme informace; v 'rootOrJunctionPoint' (neni-li NULL, musi byt alespon MAX_PATH
    // znaku velky buffer) se vraci cesta, pro kterou se podarilo zjistit udaje nebo
    // aspon korenovy adresar cesty 'path'; ostatni parametry odpovidaji standardni Win32
    // API funkci GetVolumeInformation
    // mozne volat z libovolneho threadu
    function SalGetVolumeInformation( path : PChar; rootOrJunctionPoint : PChar; lpVolumeNameBuffer : PChar;
                                      nVolumeNameSize : DWORD; lpVolumeSerialNumber : LPDWORD;
                                      lpMaximumComponentLength : LPDWORD; lpFileSystemFlags : LPDWORD;
                                      lpFileSystemNameBuffer : PChar; nFileSystemNameSize : DWORD ) : BOOL; virtual; stdcall; abstract;

    // vlastni klon windowsove GetDriveType: umi zjistit korektni udaje i pro cesty
    // obsahujici junction-pointy pod Windows 2000/XP; 'path' je cesta, jejiz typ
    // zjistujeme
    // mozne volat z libovolneho threadu
    function SalGetDriveType( path : PChar ) : UINT; virtual; stdcall; abstract;

    // protoze windowsova GetTempFileName nefunguje, napsali jsme si vlastni klon:
    // vytvori soubor/adresar (podle 'file') na ceste 'path' (NULL -> Windows TEMP dir),
    // s prefixem 'prefix', vraci jmeno vytvoreneho souboru v 'tmpName' (min. velikost MAX_PATH),
    // vraci uspech (pri neuspechu vraci v 'err' (neni-li NULL) kod Windows chyby)
    // mozne volat z libovolneho threadu
    function SalGetTempFileName( path : PChar; prefix : PChar; tmpName : PChar;
                                 _file : BOOL; err : PDWORD ) : BOOL; virtual; stdcall; abstract;

    // odstraneni adresare vcetne jeho obsahu (SHFileOperation je priserne pomaly)
    // mozne volat z libovolneho threadu
    procedure RemoveTemporaryDir( dir : PChar ); virtual; stdcall; abstract;

    // protoze windowsova verze MoveFile nezvlada prejmenovani souboru s read-only atributem na Novellu,
    // napsali jsme si vlastni (nastane-li chyba pri MoveFile, zkusi shodit read-only, provest operaci,
    // a pak ho zase nahodit); vraci uspech (pri neuspechu vraci v 'err' (neni-li NULL) kod Windows chyby)
    // mozne volat z libovolneho threadu
    function SalMoveFile( srcName : PChar; destName : PChar; err : PDWORD ) : BOOL; virtual; stdcall; abstract;

    // protoze windowsova verze GetFileSize nezvlada ziskani velikosti nad 4GB pod Win9x/ME,
    // napsali jsme si vlastni (pod Win9x/ME krome GetFileSize vyuziva jeste FindFirstFile);
    // 'file' je otevreny soubor pro volani GetFileSize(); 'fullName' je jmeno souboru, jehoz
    // velikost zjistujeme; v 'size' vraci ziskanou velikost souboru; vraci uspech, pri FALSE
    // (chyba) je v 'err' windowsovy kod chyby
    // mozne volat z libovolneho threadu
    function SalGetFileSize( _file : THandle; fullName : PChar; var size : TLargeInteger;
                             var err : DWORD ) : BOOL; virtual; stdcall; abstract;

    // otevre soubor/adresar 'name' na ceste 'path'; ridi se Windows asociacemi, otevira
    // pres polozku Open v kontextovem menu (muze pouzit i salopen.exe, zalezi na konfiguraci);
    // pred spustenim nastavi aktualni adresare na lokalnich discich podle panelu;
    // 'parent' je parent pripadnych oken (napr. pri otevirani neasociovaneho souboru)
    // omezeni: hlavni thread (jinak by nefungoval salopen.exe - pouziva jednu sdilenou pamet)
    procedure ExecuteAssociation( parent : HWND; path : PChar; name : PChar ); virtual; stdcall; abstract;

    // otevre browse dialog, ve kterem uzivatel vybere cestu; 'parent' je parent browse dialogu;
    // 'hCenterWindow' - okno, ke kteremu bude dialog centrovan; 'title' je titulek browse dialogu;
    // 'comment' je komentar v browse dialogu; 'path' je buffer pro vyslednou cestu (min. MAX_PATH
    // znaku); je-li 'onlyNet' TRUE, lze prochazet jen sitove cesty (jinak neni omezeno); neni-li
    // 'initDit' NULL, obsahuje cestu na ktere se ma browse dialog otevrit; vraci TRUE pokud je
    // v 'path' nova vybrana cesta
    // POZOR: pokud se vola mimo hlavni thread, je nutne drive inicializovat COM (mozna lepe cele
    //        OLE - viz CoInitialize nebo OLEInitialize)
    // mozne volat z libovolneho threadu
    function GetTargetDirectory( parent : HWND; hCenterWindow : HWND; title : PChar;
                                 comment : PChar; path : PChar; onlyNet : BOOL;
                                 initDir : PChar ) : BOOL; virtual; stdcall; abstract;

    // prace se souborovymi maskami: ('?' lib. znak, '*' lib. retezec - i prazdny)
    // vse je mozne volat z libovolneho threadu
    // 1) prevedeme masku do jednodussiho formatu (src -> bufferu mask - min. velikost
    //    bufferu 'mask' je (strlen(src) + 1))
    procedure PrepareMask( mask : PChar; src : PChar ); virtual; stdcall; abstract;
    // 2) muzeme pouzit prevedenou masku na test, jestli ji odpovida soubor filename,
    //    hasExtension = TRUE pokud soubor ma priponu
    //    vraci TRUE pokud soubor odpovida masce
    function AgreeMask( filename : PChar; mask : PChar; hasExtension : BOOL ) : BOOL; virtual; stdcall; abstract;
    // 3) neupravenou masku (nevolat pro ni PrepareMask) muzeme pouzit k vytvoreni jmena podle
    //    zadaneho jmena a masky ("a.txt" + "*.cpp" -> "a.cpp" atd.),
    //    buffer by mel byt velky alespon strlen(name)+strlen(mask) (hodi se 2*MAX_PATH)
    //    vraci vytvorene jmeno (ukazatel 'buffer')
    function MaskName( buffer : PChar; bufSize : Integer; name : PChar; mask : PChar ) : PChar; virtual; stdcall; abstract;

    // prace s rozsirenymi souborovymi maskami: ('?' lib. znak, '*' lib. retezec - i prazdny,
    // '#' lib. cislice - '0'..'9')
    // vse je mozne volat z libovolneho threadu
    // 1) prevedeme masku do jednodussiho formatu (src -> bufferu mask - min. delka strlen(src) + 1)
    procedure PrepareExtMask( mask : PChar; src : PChar ); virtual; stdcall; abstract;
    // 2) muzeme pouzit prevedenou masku na test, jestli ji odpovida soubor filename,
    //    hasExtension = TRUE pokud soubor ma priponu
    //    vraci TRUE pokud soubor odpovida masce
    function AgreeExtMask( filename : PChar; mask : PChar; hasExtension : BOOL ) : BOOL; virtual; stdcall; abstract;

    // alokuje novy objekt pro praci se skupinou souborovych masek; pri nedostatku pameti vraci NULL
    // mozne volat z libovolneho threadu
    function AllocSalamanderMaskGroup : TSalamanderMaskGroup; virtual; stdcall; abstract;

    // uvolni objekt pro praci se skupinou souborovych masek (ziskany metodou AllocSalamanderMaskGroup)
    // mozne volat z libovolneho threadu
    procedure FreeSalamanderMaskGroup( maskGroup : TSalamanderMaskGroup ); virtual; stdcall; abstract;

    // alokace pameti na heapu Salamandera (pri pouziti salrtl.dll zbytecne - staci klasicky malloc)
    // mozne volat z libovolneho threadu
    function Alloc( size : Integer ) : Pointer; virtual; stdcall; abstract;
    // dealokace pameti z heapu Salamandera (pri pouziti salrtl.dll zbytecne - staci klasicky free)
    // mozne volat z libovolneho threadu
    procedure Free( ptr : Pointer ); virtual; stdcall; abstract;

    // duplikace stringu - alokace pameti (na heapu Salamadera - heapu dostupnem pres salrtl.dll)
    // + kopie stringu; pri nedostatku mista (vypise TRACE_E) nebo 'str'==NULL vraci NULL (ale
    // nevypisuje zadnou hlasku mimo TRACE)
    // mozne volat z libovolneho threadu
    function DupStr( str : PChar ) : PChar; virtual; stdcall; abstract;

    // duplikace stringu - alokace pameti (na heapu Salamadera - heapu dostupnem pres salrtl.dll)
    // + kopie stringu; pri nedostatku mista (vypise TRACE_E) nebo 'str'==NULL vraci NULL (ale
    // nevypisuje zadnou hlasku mimo TRACE); pri nedostatku mista nastavuje 'err' na TRUE
    // mozne volat z libovolneho threadu
    function DupStrEx( str : PChar; var err : BOOL ) : PChar; virtual; stdcall; abstract;

    // vraci mapovaci tabulku na mala a velka pismena (pole 256 znaku - male/velke pismeno na
    // indexu zjistovaneho pismene); neni-li 'lowerCase' NULL, vraci se v nem tabulka malych pismen;
    // neni-li 'upperCase' NULL, vraci se v nem tabulka velkych pismen
    // mozne volat z libovolneho threadu
    procedure GetLowerAndUpperCase( lowerCase : PChar; upperCase : PChar ); virtual; stdcall; abstract;

    // prevod retezce 'str' na mala/velka pismena; narozdil od ANSI C tolower/toupper pracuje
    // rovnou s retezcem a podporuje nejen znaky 'A' az 'Z' (prevod na mala pismena provadi pres
    // pole inicializovane Win32 API funkci CharLower)
    procedure ToLowerCase( str : PChar ); virtual; stdcall; abstract;
    procedure ToUpperCase( str : PChar ); virtual; stdcall; abstract;

    //*****************************************************************************
    //
    // StrCmpEx
    //
    // Function compares two substrings.
    // If the two substrings are of different lengths, they are compared up to the
    // length of the shortest one. If they are equal to that point, then the return
    // value will indicate that the longer string is greater.
    //
    // Parameters
    //   s1, s2: strings to compare
    //   l1    : compared length of s1 (must be less or equal to strlen(s1))
    //   l2    : compared length of s2 (must be less or equal to strlen(s1))
    //
    // Return Values
    //   -1 if s1 < s2 (if substring pointed to by s1 is less than the substring pointed to by s2)
    //    0 if s1 = s2 (if the substrings are equal)
    //   +1 if s1 > s2 (if substring pointed to by s1 is greater than the substring pointed to by s2)
    //
    // mozne volat z libovolneho threadu
    function StrCmpEx( s1 : PChar; l1 : Integer; s2 : PChar; l2 : Integer ) : Integer; virtual; stdcall; abstract;

    //*****************************************************************************
    //
    // StrICpy
    //
    // Function copies characters from source to destination. Upper case letters are mapped to
    // lower case using LowerCase array.
    //
    // Parameters
    //   dest: pointer to the destination string
    //   src: pointer to the null-terminated source string
    //
    // Return Values
    //   The StrICpy returns the number of bytes stored in buffer, not counting
    //   the terminating null character.
    //
    // prekopirovani retezce 'src' do 'dest' bez ohledu na velikost pismen (ignore-case) a bez
    // ohledu na regionalni nastaveni Windows (ciste bytove porovnani s prevodem na mala
    // pismena); podporuje nejen znaky 'A' az 'Z' (prevod na mala pismena provadi pres
    // pole inicializovane Win32 API funkci CharLower);
    // mozne volat z libovolneho threadu
    function StrICpy( dest : PChar; src : PChar ) : Integer; virtual; stdcall; abstract;

    //*****************************************************************************
    //
    // StrICmp
    //
    // Function compares two strings. The comparison is not case sensitive.
    // For differences, upper case letters are mapped to lower case using LowerCase array.
    // Thus, "abc_" < "ABCD" since '_' < 'd'.
    //
    // Parameters
    //   s1, s2: null-terminated strings to compare
    //
    // Return Values
    //   -1 if s1 < s2 (if string pointed to by s1 is less than the string pointed to by s2)
    //    0 if s1 = s2 (if the strings are equal)
    //   +1 if s1 > s2 (if string pointed to by s1 is greater than the string pointed to by s2)
    //
    // porovnani dvou retezcu 's1' a 's2' bez ohledu na velikost pismen (ignore-case) a bez
    // ohledu na regionalni nastaveni Windows (ciste bytove porovnani s prevodem na mala
    // pismena); narozdil od ANSI C stricmp podporuje nejen znaky 'A' az 'Z' (prevod na mala
    // pismena provadi pres pole inicializovane Win32 API funkci CharLower);
    // vraci -1 ('s1' < 's2'), 0 ('s1' == 's2'), 1 ('s1' > 's2')
    // mozne volat z libovolneho threadu
    function StrICmp( s1 : PChar; s2 : PChar ) : Integer; virtual; stdcall; abstract;

    //*****************************************************************************
    //
    // StrICmpEx
    //
    // Function compares two substrings. The comparison is not case sensitive.
    // For the purposes of the comparison, all characters are converted to lower case
    // using LowerCase array.
    // If the two substrings are of different lengths, they are compared up to the
    // length of the shortest one. If they are equal to that point, then the return
    // value will indicate that the longer string is greater.
    //
    // Parameters
    //   s1, s2: strings to compare
    //   l1    : compared length of s1 (must be less or equal to strlen(s1))
    //   l2    : compared length of s2 (must be less or equal to strlen(s2))
    //
    // Return Values
    //   -1 if s1 < s2 (if substring pointed to by s1 is less than the substring pointed to by s2)
    //    0 if s1 = s2 (if the substrings are equal)
    //   +1 if s1 > s2 (if substring pointed to by s1 is greater than the substring pointed to by s2)
    //
    // stejna pravidla pro prevod na mala pismena, jako u StrICmp
    // mozne volat z libovolneho threadu
    function StrICmpEx( s1 : PChar; l1 : Integer; s2 : PChar; l2 : Integer ) : Integer; virtual; stdcall; abstract;

    //*****************************************************************************
    //
    // StrNICmp
    //
    // Function compares two strings. The comparison is not case sensitive.
    // For the purposes of the comparison, all characters are converted to lower case
    // using LowerCase array. The comparison stops after: (1) a difference between the
    // strings is found, (2) the end of the strings is reached, or (3) n characters
    // have been compared.
    //
    // Parameters
    //   s1, s2: strings to compare
    //   n:      maximum length to compare
    //
    // Return Values
    //   -1 if s1 < s2 (if substring pointed to by s1 is less than the substring pointed to by s2)
    //    0 if s1 = s2 (if the substrings are equal)
    //   +1 if s1 > s2 (if substring pointed to by s1 is greater than the substring pointed to by s2)
    //
    // porovnani dvou retezcu 's1' a 's2' max. do delky 'n' bez ohledu na velikost pismen
    // (ignore-case) a bez ohledu na regionalni nastaveni Windows (ciste bytove porovnani
    // s prevodem na mala pismena); narozdil od ANSI C strnicmp podporuje nejen znaky
    // 'A' az 'Z' (prevod na mala pismena provadi pres pole inicializovane Win32 API funkci
    // CharLower); vraci -1 ('s1' < 's2'), 0 ('s1' == 's2'), 1 ('s1' > 's2'), pricemz
    // porovnava maximalne prvnich 'n' znaku ('n' ma vliv na vysledek porovnani jen
    // v pripade, ze jsou oba retezce delsi nez 'n')
    // mozne volat z libovolneho threadu
    function StrNICmp( s1 : PChar; s2 : PChar; n : Integer ) : Integer; virtual; stdcall; abstract;

    //*****************************************************************************
    //
    // MemICmp
    //
    // Compares n bytes of the two blocks of memory stored at buf1
    // and buf2. The characters are converted to lowercase before
    // comparing (not permanently; using LowerCase array), so case
    // is ignored in the search.
    //
    // Parameters
    //   buf1, buf2: memory buffers to compare
    //   n:          maximum length to compare
    //
    // Return Values
    //   -1 if buf1 < buf2 (if buffer pointed to by buf1 is less than the buffer pointed to by buf2)
    //    0 if buf1 = buf2 (if the buffers are equal)
    //   +1 if buf1 > buf2 (if buffer pointed to by buf1 is greater than the buffer pointed to by buf2)
    //
    // stejna pravidla pro prevod na mala pismena, jako u StrICmp
    // mozne volat z libovolneho threadu
    function MemICmp( buf1 : Pointer; buf2 : Pointer; n : Integer ) : Integer; virtual; stdcall; abstract;

    // porovnani dvou retezcu 's1' a 's2' bez ohledu na velikost pismen (ignore-case),
    // je-li SALCFG_SORTUSESLOCALE TRUE, pouziva razeni podle regionalniho nastaveni Windows,
    // jinak porovnava stejne jako CSalamanderGeneral::StrICmp, je-li SALCFG_SORTDETECTNUMBERS
    // TRUE, pouziva ciselne razeni pro cisla obsazene v retezcich
    // vraci <0 ('s1' < 's2'), ==0 ('s1' == 's2'), >0 ('s1' > 's2')
    function RegSetStrICmp( s1 : PChar; s2 : PChar ) : Integer; virtual; stdcall; abstract;

    // porovnani dvou retezcu 's1' a 's2' (o delkach 'l1' a 'l2') bez ohledu na velikost
    // pismen (ignore-case), je-li SALCFG_SORTUSESLOCALE TRUE, pouziva razeni podle
    // regionalniho nastaveni Windows, jinak porovnava stejne jako CSalamanderGeneral::StrICmp,
    // je-li SALCFG_SORTDETECTNUMBERS TRUE, pouziva ciselne razeni pro cisla obsazene
    // v retezcich; v 'numericalyEqual' (neni-li NULL) vraci TRUE, pokud jsou retezce
    // numericky shodne (napr. "a01" a "a1"), je automaticky TRUE pokud jsou retezce shodne
    // vraci <0 ('s1' < 's2'), ==0 ('s1' == 's2'), >0 ('s1' > 's2')
    function RegSetStrICmpEx( s1 : PChar; l1 : Integer; s2 : PChar; l2 : Integer;
                              numericalyEqual : PBOOL ) : Integer; virtual; stdcall; abstract;

    // porovnani (case-sensitive) dvou retezcu 's1' a 's2', je-li SALCFG_SORTUSESLOCALE TRUE,
    // pouziva razeni podle regionalniho nastaveni Windows, jinak porovnava stejne jako
    // standardni C knihovni funkce strcmp, je-li SALCFG_SORTDETECTNUMBERS TRUE, pouziva
    // ciselne razeni pro cisla obsazene v retezcich
    // vraci <0 ('s1' < 's2'), ==0 ('s1' == 's2'), >0 ('s1' > 's2')
    function RegSetStrCmp( s1 : PChar; s2 : PChar ) : Integer; virtual; stdcall; abstract;

    // porovnani (case-sensitive) dvou retezcu 's1' a 's2' (o delkach 'l1' a 'l2'), je-li
    // SALCFG_SORTUSESLOCALE TRUE, pouziva razeni podle regionalniho nastaveni Windows,
    // jinak porovnava stejne jako standardni C knihovni funkce strcmp, je-li
    // SALCFG_SORTDETECTNUMBERS TRUE, pouziva ciselne razeni pro cisla obsazene v retezcich;
    // v 'numericalyEqual' (neni-li NULL) vraci TRUE, pokud jsou retezce numericky shodne
    // (napr. "a01" a "a1"), je automaticky TRUE pokud jsou retezce shodne
    // vraci <0 ('s1' < 's2'), ==0 ('s1' == 's2'), >0 ('s1' > 's2')
    function RegSetStrCmpEx( s1 : PChar; l1 : Integer; s2 : PChar; l2 : Integer;
                             numericalyEqual : PBOOL ) : Integer; virtual; stdcall; abstract;

    // vraci cestu v panelu; 'panel' je jeden z PANEL_XXX; 'buffer' je buffer pro cestu (muze
    // byt i NULL); 'bufferSize' je velikost bufferu 'buffer' (je-li 'buffer' NULL, musi zde
    // byt nula); 'type' neni-li NULL ukazuje na promennou, do ktere se ulozi typ cesty
    // (viz PATH_TYPE_XXX); pokud jde o archiv a 'archiveOrFS' neni NULL a 'buffer' neni NULL,
    // vraci 'archiveOrFS' nastavene do 'buffer' na pozici za soubor archivu;
    // pokud jde o file-system a 'archiveOrFS' neni NULL a 'buffer' neni NULL, vraci
    // 'archiveOrFS' nastavene do 'buffer' na ':' za nazvem file-systemu (za ':' je user-part
    // cesty file-systemu); je-li 'convertFSPathToExternal' TRUE a v panelu je cesta na FS,
    // najde se plugin jehoz cesta je (podle fs-name) a zavola se jeho
    // CPluginInterfaceForFSAbstract::ConvertPathToExternal(); vraci uspech (neni-li
    // 'bufferSize'==0, povazuje se za neuspech i to, kdyz se cesta nevejde do bufferu
    // 'buffer')
    // omezeni: hlavni thread
    function GetPanelPath( panel : Integer; buffer : PChar; bufferSize : Integer; _type : PInteger;
                           archiveOrFS : PChar; convertFSPathToExternal : BOOL = False ) : BOOL; virtual; stdcall; abstract;

    // vraci posledni navstivenou windowsovou cestu v panelu, hodi se pro navraty z FS (prijemnejsi nez
    // jit hned na fixed-drive); 'panel' je jeden z PANEL_XXX; 'buffer' je buffer pro cestu;
    // 'bufferSize' je velikost bufferu 'buffer'; vraci uspech
    // omezeni: hlavni thread
    function GetLastWindowsPanelPath( panel : Integer; buffer : PChar; bufferSize : Integer ) : BOOL; virtual; stdcall; abstract;

    // vraci jmeno FS pridelene "dozivotne" pluginu Salamanderem (podle navrhu z SetBasicPluginData);
    // 'buf' je buffer o velikosti alespon MAX_PATH znaku; 'fsNameIndex' je index fs-name (index je
    // nula pro fs-name zadane v CSalamanderPluginEntryAbstract::SetBasicPluginData2, u ostatnich
    // fs-name index vraci CSalamanderPluginEntryAbstract::AddFSName)
    // omezeni: hlavni thread (jinak muze dojit ke zmenam v konfiguraci pluginu behem volani),
    // v entry-pointu lze volat az po akt. verzi "SetBasicPluginData", drive nemusi byt zname
    procedure GetPluginFSName( buf : PChar; fsNameIndex : Integer ); virtual; stdcall; abstract;

    // vraci interface pluginoveho file-systemu (FS) otevreneho v panelu 'panel' (jeden z PANEL_XXX);
    // pokud v panelu neni otevren FS nebo jde o FS jineho pluginu (nepatri volajicimu pluginu), vraci
    // metoda NULL (nelze pracovat s objektem jineho pluginu, jeho struktura je neznama)
    // omezeni: hlavni thread
    function GetPanelPluginFS( panel : Integer ) : TPluginFSInterfaceAbstract; virtual; stdcall; abstract;

    // vraci plugin data interface listingu panelu (muze byt i NULL), 'panel' je jeden z PANEL_XXX;
    // pokud plugin data interface existuje, ale nepatri tomuto (volajicimu) pluginu, vraci
    // metoda NULL (nelze pracovat s objektem jineho pluginu, jeho struktura je neznama)
    // omezeni: hlavni thread
    function GetPanelPluginData( panel : Integer ) : TPluginDataInterfaceAbstract; virtual; stdcall; abstract;

    // vraci fokusenou polozku panelu (soubor/adresar/updir("..")), 'panel' je jeden z PANEL_XXX,
    // vraci NULL (zadna polozka v panelu) nebo data fokusene polozky; neni-li 'isDir' NULL,
    // vraci v nem FALSE pokud jde o soubor (jinak jde o adresar nebo updir)
    // POZOR: vracena data polozky jsou pouze pro cteni
    // omezeni: hlavni thread
    function GetPanelFocusedItem( panel : Integer; isDir : PBOOL ) : PFileData; virtual; stdcall; abstract;

    // vraci postupne polozky panelu (nejprve adresare, pak soubory), 'panel' je jeden z PANEL_XXX,
    // 'index' je vstupne/vystupni promenna, ukazuje na int, ve kterem je pri prvnim volani 0,
    // hodnotu pro dalsi volani si funkce ulozi pri navratu (pouziti: na zacatku vynulovat, pak
    // nemenit), vraci NULL (jiz zadna polozka) nebo data dalsi (pripadne prvni) polozky;
    // neni-li 'isDir' NULL, vraci v nem FALSE pokud jde o soubor (jinak jde o adresar nebo updir)
    // POZOR: vracena data polozky jsou pouze pro cteni
    // omezeni: hlavni thread
    function GetPanelItem( panel : Integer; index : PInteger; isDir : PBOOL ) : PFileData; virtual; stdcall; abstract;

    // vraci postupne oznacene polozky panelu (nejprve adresare, pak soubory), 'panel' je jeden z
    // PANEL_XXX, 'index' je vstupne/vystupni promenna, ukazuje na int, ve kterem je pri prvnim volani 0,
    // hodnotu pro dalsi volani si funkce ulozi pri navratu (pouziti: na zacatku vynulovat, pak
    // nemenit), vraci NULL (jiz zadna polozka) nebo data dalsi (pripadne prvni) polozky;
    // neni-li 'isDir' NULL, vraci v nem FALSE pokud jde o soubor (jinak jde o adresar nebo updir)
    // POZOR: vracena data polozky jsou pouze pro cteni
    // omezeni: hlavni thread
    function GetPanelSelectedItem( panel : Integer; index : PInteger; isDir : PBOOL ) : PFileData; virtual; stdcall; abstract;

    // zjisti kolik souboru a adresaru je oznacenych v panelu; 'panel' je jeden z PANEL_XXX;
    // neni-li 'selectedFiles' NULL, vraci v nem pocet oznacenych souboru; neni-li 'selectedDirs'
    // NULL, vraci v nem pocet oznacenych adresaru; vraci TRUE pokud je oznaceny aspon jeden
    // soubor nebo adresar nebo pokud je fokus na souboru nebo adresari (je-li s cim
    // pracovat - fokus neni na up-diru)
    // omezeni: hlavni thread (jinak se muze obsah panelu menit)
    function GetPanelSelection( panel : Integer; selectedFiles : PInteger; selectedDirs : PInteger ) : BOOL; virtual; stdcall; abstract;

    // vraci top-index listboxu v panelu; 'panel' je jeden z PANEL_XXX
    // omezeni: hlavni thread (jinak se muze obsah panelu menit)
    function GetPanelTopIndex( panel : Integer ) : Integer; virtual; stdcall; abstract;

    // informuje hlavni okno Salamandera, ze se deaktivuje okno viewru, pokud se bude
    // bezprostredne aktivovat hlavni okno a v panelech budou neautomaticky refreshovane
    // disky nedojde k jejich refreshi (viewry nemeni obsah disku), nepovine (dojde
    // k mozna zbytecnemu refrehi)
    // mozne volat z libovolneho threadu
    procedure SkipOneActivateRefresh; virtual; stdcall; abstract;

    // oznacuje/odznacuje polozku panelu, 'file' je ukazatel na menenou polozku ziskany predchozim
    // "get-item" volanim (metody GetPanelFocusedItem, GetPanelItem a GetPanelSelectedItem);
    // je nutne, aby od "get-item" volani nebyl opusten plugin a volani probehlo v hlavnim
    // threadu (aby nedoslo k obnove panelu - zneplatneni ukazatele); 'panel' musi byt shodny
    // s parametrem 'panel' prislusneho "get-item" volani; je-li 'select' TRUE dojde k oznaceni,
    // jinak dojde k odznaceni; po poslednim volani je nutne pouzit RepaintChangedItems('panel') pro
    // prekresleni panelu
    // omezeni: hlavni thread
    procedure SelectPanelItem( panel : Integer; _file : PFileData; select : BOOL ); virtual; stdcall; abstract;

    // provede prekresleni polozek panelu u kterych doslo ke zmenam (oznaceni); 'panel' je
    // jeden z PANEL_XXX
    // omezeni: hlavni thread
    procedure RepaintChangedItems( panel : Integer ); virtual; stdcall; abstract;

    // oznacuje/odznacuje vsechny polozky v panelu, 'panel' je jeden z PANEL_XXX; je-li 'select'
    // TRUE dojde k oznaceni, jinak dojde k odznaceni; je-li 'repaint' TRUE prekresli se vsechny
    // zmenene polozky v panelu, jinak k prekresleni nedojde (mozne volat pozdeji RepaintChangedItems)
    // omezeni: hlavni thread
    procedure SelectAllPanelItems( panel : Integer; select : BOOL; repaint : BOOL ); virtual; stdcall; abstract;

    // nastavi fokus v panelu, 'file' je ukazatel na fokusenou polozku ziskany predchozim
    // "get-item" volanim (metody GetPanelFocusedItem, GetPanelItem a GetPanelSelectedItem);
    // je nutne, aby od "get-item" volani nebyl opusten plugin a volani probehlo v hlavnim
    // threadu (aby nedoslo k obnove panelu - zneplatneni ukazatele); 'panel' musi byt shodny
    // s parametrem 'panel' prislusneho "get-item" volani; je-li 'partVis' TRUE a polozka bude
    // viditelna jen castecne, nedojde k odrolovani panelu pri fokusu, pri FALSE odroluje panel
    // tak, aby byla videt cela polozka
    // omezeni: hlavni thread
    procedure SetPanelFocusedItem( panel : Integer; _file : PFileData; partVis : BOOL ); virtual; stdcall; abstract;

    // zjisti jestli se v panelu pouziva filtr a pokud se pouziva, ziska retezec s maskami
    // tohoto filtru; 'panel' oznacuje panel, o ktery se zajimame (jedna z PANEL_XXX);
    // 'masks' je buffer pro masky filtru o velikosti minimalne 'masksBufSize' bytu (doporucena
    // velikost je MAX_GROUPMASK); vraci TRUE pokud se filtr pouziva a buffer 'masks' je
    // dost velky; vraci FALSE pokud se filtr nepouziva nebo se retezec masek nevesel
    // do 'masks'
    // omezeni: hlavni thread
    function GetFilterFromPanel( panel : Integer; masks : PChar; masksBufSize : Integer ) : BOOL; virtual; stdcall; abstract;

    // vraci pozici zdrojoveho panelu (je vlevo nebo vpravo?), vraci PANEL_LEFT nebo PANEL_RIGHT
    // omezeni: hlavni thread
    function GetSourcePanel : Integer; virtual; stdcall; abstract;

    // zjistuje, ve kterem panelu je otevreny 'pluginFS'; pokud neni ani v jednom panelu,
    // vraci FALSE; pokud vrati TRUE, je cislo panelu v 'panel' (PANEL_LEFT nebo PANEL_RIGHT)
    // omezeni: hlavni thread (jinak se muze obsah panelu menit)
    function GetPanelWithPluginFS( pluginFS : TPluginFSInterfaceAbstract; var panel : Integer ) : BOOL; virtual; stdcall; abstract;

    // aktivuje druhy panel (ala klavesa TAB); panely oznacene pres PANEL_SOURCE a PANEL_TARGET
    // se tim prirozene prohazuji
    // omezeni: hlavni thread
    procedure ChangePanel; virtual; stdcall; abstract;

    // prevod cisla na "prehlednejsi" retezec (po trech cislicich mezera), retezec vraci v
    // 'buffer' (min. velikost 50 bytu), vraci 'buffer'
    // mozne volat z libovolneho threadu
    function NumberToStr( buffer : PChar; var number : TLargeInteger ) : PChar; virtual; stdcall; abstract;

    // tisk velikosti mista na disku do 'buf' (min. velikost bufferu je 100 bytu),
    // mode==0 "1.23 MB", mode==1 "1 230 000 bytes, 1.23 MB", mode==2 "1 230 000 bytes",
    // mode==3 "12 KB" (vzdy v celych kilo bajtech), mode==4 (jako mode==0, ale vzdy
    // aspon 3 platne cislice, napr. "2.00 MB")
    // vraci 'buf'
    // mozne volat z libovolneho threadu
    function PrintDiskSize( buf : PChar; var size : TLargeInteger; mode : Integer ) : PChar; virtual; stdcall; abstract;

    // prevadi pocet sekund na retezec ("5 sec", "1 hr 34 min", atp.); 'buf' je
    // buffer pro vysledny text, musi byt velky aspon 100 znaku; 'secs' je pocet sekund;
    // vraci 'buf'
    // mozne volat z libovolneho threadu
    function PrintTimeLeft( buf : PChar; var secs : TLargeInteger ) : PChar; virtual; stdcall; abstract;

    // porovna root normalni (c:\path) i UNC (\\server\share\path) cesty, vraci TRUE pokud je root shodny
    // mozne volat z libovolneho threadu
    function HasTheSameRootPath( path1 : PChar; path2 : PChar ) : BOOL; virtual; stdcall; abstract;

    // Vrati pocet znaku spolecne cesty. Na normalni ceste musi byt root ukonceny zpetnym
    // lomitkem, jinak funkce vrati 0. ("C:\"+"C:"->0, "C:\A\B"+"C:\"->3, "C:\A\B\"+"C:\A"->4,
    // "C:\AA\BB"+"C:\AA\CC"->5)
    // Pracuje pro normalni i UNC cesty.
    function CommonPrefixLength( path1 : PChar; path2 : PChar ) : Integer; virtual; stdcall; abstract;

    // Vraci TRUE, pokud je cesta 'prefix' zakladem cesty 'path'. Jinak vraci FALSE.
    // "C:\aa","C:\Aa\BB"->TRUE
    // "C:\aa","C:\aaa"->FALSE
    // "C:\aa\","C:\Aa"->TRUE
    // "\\server\share","\\server\share\aaa"->TRUE
    // Pracuje pro normalni i UNC cesty.
    function PathIsPrefix( prefix : PChar; path : PChar ) : BOOL; virtual; stdcall; abstract;

    // porovna dve normalni (c:\path) i UNC (\\server\share\path) cesty, ignoruje mala/velka pismena,
    // ignoruje take jeden backslash na zacatku a konci cest, vraci TRUE pokud jsou cesty stejne
    // mozne volat z libovolneho threadu
    function IsTheSamePath( path1 : PChar; path2 : PChar ) : BOOL; virtual; stdcall; abstract;

    // ziska root cestu z normalni (c:\path) i UNC (\\server\share\path) cesty 'path', v 'root' vraci
    // cestu ve formatu 'c:\' nebo '\\server\share\' (min. velikost 'root' bufferu je MAX_PATH),
    // vraci pocet znaku root cesty (bez null-terminatoru)
    // mozne volat z libovolneho threadu
    function GetRootPath( root : PChar; path : PChar ) : Integer; virtual; stdcall; abstract;

    // zkracuje normalni (c:\path) i UNC (\\server\share\path) cestu o posledni adresar
    // (zariznuti na poslednim backslashi - v oriznute ceste zustava na konci backslash
    // jen u 'c:\'), 'path' je in/out buffer (min. velikost strlen(path)+2 bytu),
    // v 'cutDir' (neni-li NULL) se vraci ukazatel (do bufferu 'path' za 1. null-terminator)
    // na posledni adresar (odriznutou cast), tato metoda nahrazuje PathRemoveFileSpec,
    // vraci TRUE pokud doslo ke zkraceni (neslo o root cestu)
    // mozne volat z libovolneho threadu
    function CutDirectory( path : PChar; cutDir : PChar = nil ) : BOOL; virtual; stdcall; abstract;

    // pracuje s normalnimi (c:\path) i UNC (\\server\share\path) cestami,
    // spoji 'path' a 'name' do 'path', zajisti spojeni backslashem, 'path' je buffer alespon
    // 'pathSize' znaku, vraci TRUE pokud se 'name' veslo za 'path'; je-li 'path' nebo 'name'
    // prazdne, spojovaci (pocatecni/ukoncovaci) backslash nebude (napr. "c:\" + "" -> "c:")
    // mozne volat z libovolneho threadu
    function SalPathAppend( path : PChar; name : PChar; pathSize : Integer ) : BOOL; virtual; stdcall; abstract;

    // pracuje s normalnimi (c:\path) i UNC (\\server\share\path) cestami,
    // pokud jeste 'path' nekonci na backslash, prida ho na konec 'path'; 'path' je buffer
    // alespon 'pathSize' znaku; vraci TRUE pokud se backslash vesel za 'path'; je-li 'path'
    // prazdne, backslash se neprida
    // mozne volat z libovolneho threadu
    function SalPathAddBackslash( path : PChar; pathSize : Integer ) : BOOL; virtual; stdcall; abstract;

    // pracuje s normalnimi (c:\path) i UNC (\\server\share\path) cestami,
    // pokud je v 'path' na konci backslash, odstrani ho
    // mozne volat z libovolneho threadu
    procedure SalPathRemoveBackslash( path : PChar ); virtual; stdcall; abstract;

    // pracuje s normalnimi (c:\path) i UNC (\\server\share\path) cestami,
    // z plneho jmena udela jmeno ("c:\path\file" -> "file")
    // mozne volat z libovolneho threadu
    procedure SalPathStripPath( path : PChar ); virtual; stdcall; abstract;

    // pracuje s normalnimi (c:\path) i UNC (\\server\share\path) cestami,
    // pokud je ve jmene pripona, odstrani ji
    // mozne volat z libovolneho threadu
    procedure SalPathRemoveExtension( path : PChar ); virtual; stdcall; abstract;

    // pracuje s normalnimi (c:\path) i UNC (\\server\share\path) cestami,
    // pokud ve jmenu 'path' jeste neni pripona, prida priponu 'extension' (napr. ".txt"),
    // 'path' je buffer alespon 'pathSize' znaku, vraci FALSE pokud buffer 'path' nestaci
    // pro vyslednou cestu
    // mozne volat z libovolneho threadu
    function SalPathAddExtension( path : PChar; extension : PChar; pathSize : Integer ) : BOOL; virtual; stdcall; abstract;

    // pracuje s normalnimi (c:\path) i UNC (\\server\share\path) cestami,
    // zmeni/prida priponu 'extension' (napr. ".txt") ve jmenu 'path', 'path' je buffer
    // alespon 'pathSize' znaku, vraci FALSE pokud buffer 'path' nestaci pro vyslednou cestu
    // mozne volat z libovolneho threadu
    function SalPathRenameExtension( path : PChar; extension : PChar; pathSize : Integer ) : BOOL; virtual; stdcall; abstract;

    // pracuje s normalnimi (c:\path) i UNC (\\server\share\path) cestami,
    // vraci ukazatel do 'path' na jmeno souboru/adresare (backslash na konci 'path' ignoruje),
    // pokud jmeno neobsahuje jine backslashe nez na konci retezce, vraci 'path'
    // mozne volat z libovolneho threadu
    function SalPathFindFileName( path : PChar ) : PChar; virtual; stdcall; abstract;

    // upravuje relativni nebo absolutni normalni (c:\path) i UNC (\\server\share\path) cestu
    // na absolutni bez '.', '..' a koncoveho backslashe (krom typu "c:\"); je-li 'curDir' NULL,
    // relativni cesty typu "\path" a "path" vraci chybu (neurcitelne), jinak je 'curDir' platna
    // upravena aktualni cesta (UNC i normalni); aktualni cesty ostatnich disku (mimo
    // 'curDir' + jen normalni, ne UNC) jsou v Salamandrovskem poli DefaultDir (pred pouzitim
    // je dobre zavolat metodu SalUpdateDefaultDir); 'name' - in/out buffer cesty alespon MAX_PATH
    // znaku; neni-li 'nextFocus' NULL a zadana relativni cesta neobsahuje backslash, provede se
    // strcpy(nextFocus, name); vraci TRUE - jmeno 'name' je pripraveno k pouziti, jinak neni-li
    // 'errTextID' NULL obsahuje chybu (konstanty GFN_XXX - text se da ziskat pres GetGFNErrorText)
    // POZOR: pred pouzitim je dobre zavolat metodu SalUpdateDefaultDir
    // omezeni: hlavni thread (jinak muze dojit ke zmenam DefaultDir v hl. threadu)
    function SalGetFullName( name : PChar; errTextID : PInteger = nil; curDir : PChar = nil;
                             nextFocus : PChar = nil ) : BOOL; virtual; stdcall; abstract;

    // obnovi Salamandrovske pole DefaultDir podle cest v panelech, je-li 'activePrefered' TRUE,
    // bude mit prednost cesta v aktivnim panelu (zapise se pozdeji do DefaultDir), jinak ma
    // prednost cesta v neaktivnim panelu
    // omezeni: hlavni thread (jinak muze dojit ke zmenam DefaultDir v hl. threadu)
    procedure SalUpdateDefaultDir( activePrefered : BOOL ); virtual; stdcall; abstract;

    // vraci textovou reprezentaci chybove konstanty GFN_XXX; vraci 'buf' (aby slo dat GetGFNErrorText
    // primo jako parametr funkce)
    // mozne volat z libovolneho threadu
    function GetGFNErrorText( GFN : Integer; buf : PChar; bufSize : Integer ) : PChar; virtual; stdcall; abstract;

    // vraci textovou reprezentaci systemove chyby (ERROR_XXX); vraci 'buf' (aby slo dat GetErrorText
    // primo jako parametr funkce)
    // mozne volat z libovolneho threadu
    function GetErrorText( err : Integer; buf : PChar; bufSize : Integer ) : PChar; virtual; stdcall; abstract;

    // vraci vnitrni barvu Salamandera, 'color' je konstanta barvy (viz SALCOL_XXX)
    // mozne volat z libovolneho threadu
    function GetCurrentColor( color : Integer ) : TColorRef; virtual; stdcall; abstract;

    // zajisti aktivaci hlavniho okna Salamandera + focus souboru/adresare 'name' na ceste
    // 'path' v panelu 'panel'; pripadne zmeni cestu v panelu (je-li nutne); 'panel' je jeden
    // z PANEL_XXX; 'path' je libovolna cesta (windowsova (diskova), na FS nebo do archivu);
    // omezeni: hlavni thread + mimo metody CPluginFSInterfaceAbstract a CPluginDataInterfaceAbstract
    // (hrozi napr. zavreni FS otevreneho v panelu - metode by mohl prestat existovat 'this')
    procedure FocusNameInPanel( panel : Integer; path : PChar; name : PChar ); virtual; stdcall; abstract;

    // zmena cesty v panelu - vstupem muze byt absolutni i relativni UNC (\\server\share\path)
    // i normalni (c:\path) cesta a to jak windowsova (diskova), tak do archivu nebo cesta
    // na FS (absolutni/relativni si resi primo plugin); pokud je vstupem cesta k souboru,
    // dojde k fokusu tohoto souboru; neni-li suggestedTopIndex -1, bude nastaven top-index
    // v panelu; neni-li suggestedFocusName NULL, zkusi se nalezt (ignore-case) a vyfokusit
    // polozka stejneho jmena; neni-li 'failReason' NULL, nastavuje se na jednu z konstant
    // CHPPFR_XXX (informuje o vysledku metody); je-li 'convertFSPathToInternal' TRUE a jde
    // o cestu na FS, najde se plugin jehoz cesta je (podle fs-name) a zavola se jeho
    // CPluginInterfaceForFSAbstract::ConvertPathToInternal(); vraci TRUE pokud se podarilo
    // vylistovat pozadovanou cestu;
    // POZNAMKA: pri zadani cesty na FS dojde k pokusu o otevreni cesty v tomto poradi: ve FS
    // v panelu, v odpojenem FS, nebo v novem FS (u FS z panelu a odpojenych FS se zjistuje
    // jestli odpovida plugin-fs-name a jestli metoda FS IsOurPath vraci pro zadanou cestu TRUE);
    // omezeni: hlavni thread + mimo metody CPluginFSInterfaceAbstract a CPluginDataInterfaceAbstract
    // (hrozi napr. zavreni FS otevreneho v panelu - metode by mohl prestat existovat 'this')
    function ChangePanelPath( panel : Integer; path : PChar; failReason : PInteger = nil;
                              suggestedTopIndex : Integer = -1; suggestedFocusName : PChar = nil;
                              convertFSPathToInternal : BOOL = True ) : BOOL; virtual; stdcall; abstract;

    // zmena cesty v panelu na relativni nebo absolutni UNC (\\server\share\path) nebo normalni (c:\path)
    // cestu, pokud neni nova cesta dostupna, zkousi uspet se zkracenymi cestami; pokud jde o zmenu
    // cesty v ramci jednoho disku (vcetne archivu na tomto disku) a na disku se nepodari nalezt
    // pristupnou cestu, zmeni cestu na root prvniho lokalniho fixed drivu (velka sance uspechu,
    // panel nezustane prazdny); pri prekladu relativni cesty na absolutni je uprednostnovana cesta
    // v panelu 'panel' (jen je-li to cesta na disk (i k archivu), jinak se nepouziva); 'panel' je
    // jeden z PANEL_XXX; 'path' je nova cesta; neni-li 'suggestedTopIndex' -1, bude nastaven jako
    // top-index v panelu (jen pro novou cestu, na zkracene (zmenene) ceste se nenastavuje); neni-li
    // 'suggestedFocusName' NULL, zkusi se nalezt (ignore-case) a vyfokusit polozka stejneho jmena
    // (jen pro novou cestu, na zkracene (zmenene) ceste se neprovadi); neni-li 'failReason' NULL,
    // nastavuje se na jednu z konstant CHPPFR_XXX (informuje o vysledku metody); vraci TRUE pokud
    // se podarilo vylistovat pozadovanou cestu (nezkracenou/nezmenenou)
    // omezeni: hlavni thread + mimo metody CPluginFSInterfaceAbstract a CPluginDataInterfaceAbstract
    // (hrozi napr. zavreni FS otevreneho v panelu - metode by mohl prestat existovat 'this')
    function ChangePanelPathToDisk( panel : Integer; path : PChar; failReason : PInteger = nil;
                                    suggestedTopIndex : Integer = -1;
                                    suggestedFocusName : PChar = nil ) : BOOL; virtual; stdcall; abstract;

    // zmena cesty v panelu do archivu, 'archiv' je relativni nebo absolutni UNC
    // (\\server\share\path\file) nebo normalni (c:\path\file) jmeno archivu, 'archivePath' je cesta
    // uvnitr archivu, pokud neni nova cesta v archivu dostupna, zkousi uspet se zkracenymi cestami;
    // pri prekladu relativni cesty na absolutni je uprednostnovana cesta v panelu 'panel'
    // (jen je-li to cesta na disk (i k archivu), jinak se nepouziva); 'panel' je jeden z PANEL_XXX;
    // neni-li 'suggestedTopIndex' -1, bude nastaven jako top-index v panelu (jen pro novou
    // cestu, na zkracene (zmenene) ceste se nenastavuje); neni-li 'suggestedFocusName' NULL,
    // zkusi se nalezt (ignore-case) a vyfokusit polozka stejneho jmena (jen pro novou cestu,
    // na zkracene (zmenene) ceste se neprovadi); je-li 'forceUpdate' TRUE a provadi se zmena cesty
    // uvnitr archivu 'archive' (archiv uz je v panelu otevreny), provadi se test zmeny souboru
    // archivu (kontrola size & time) a v pripade zmeny se archiv zavre (hrozi update editovanych
    // souboru) a znovu vylistuje nebo pokud soubor prestal existovat, provede se zmena cesty na disk
    // (zavreni archivu; pokud cesta na disk neni pristupna, zmeni cestu na root prvniho lokalniho
    // fixed drivu); je-li 'forceUpdate' FALSE, provadi se zmeny cesty uvnitr archivu 'archive' bez
    // kontroly souboru archivu; neni-li 'failReason' NULL, nastavuje se na jednu z konstant
    // CHPPFR_XXX (informuje o vysledku metody); vraci TRUE pokud se podarilo vylistovat
    // pozadovanou cestu (nezkracenou/nezmenenou)
    // omezeni: hlavni thread + mimo metody CPluginFSInterfaceAbstract a CPluginDataInterfaceAbstract
    // (hrozi napr. zavreni FS otevreneho v panelu - metode by mohl prestat existovat 'this')
    function ChangePanelPathToArchive( panel : Integer; archive : PChar; archivePath : PChar;
                                       failReason : PInteger = nil; suggestedTopIndex : Integer = -1;
                                       suggestedFocusName : PChar = nil;
                                       forceUpdate : BOOL = False ) : BOOL; virtual; stdcall; abstract;

    // zmena cesty v panelu do pluginoveho FS, 'fsName' je jmeno FS (viz GetPluginFSName; nemusi
    // byt nutne z tohoto pluginu), 'fsUserPart' je cesta v ramci FS; pokud neni nova cesta v FS
    // dostupna, zkousi uspet se zkracenymi cestami (opakovane volani ChangePath a ListCurrentPath,
    // viz CPluginFSInterfaceAbstract); pokud jde o zmenu cesty v ramci aktualniho FS (viz
    // CPluginFSInterfaceAbstract::IsOurPath) a nepodari se od nove cesty nalezt pristupnou cestu,
    // zkusi najit pristupnou cestu jeste od puvodni (aktualni) cesty, a pokud zklame i ta,
    // zmeni cestu na root prvniho lokalniho fixed drivu (velka sance uspechu, panel nezustane prazdny);
    // 'panel' je jeden z PANEL_XXX; neni-li 'suggestedTopIndex' -1, bude nastaven jako top-index
    // v panelu (jen pro novou cestu, na zkracene (zmenene) ceste se nenastavuje); neni-li
    // 'suggestedFocusName' NULL, zkusi se nalezt (ignore-case) a vyfokusit polozka stejneho jmena
    // (jen pro novou cestu, na zkracene (zmenene) ceste se neprovadi); je-li 'forceUpdate' TRUE,
    // neoptimalizuje se (cesta se normalne vylistuje) pripad zmeny cesty na aktualni cestu v panelu
    // (bud nova cesta odpovida aktualni ceste rovnou nebo na ni byla hned prvnim ChangePath
    // zkracena); je-li 'convertPathToInternal' TRUE, najde se podle 'fsName' plugin a zavola se
    // jeho metoda CPluginInterfaceForFSAbstract::ConvertPathToInternal() pro 'fsUserPart';
    // neni-li 'failReason' NULL, nastavuje se na jednu z konstant CHPPFR_XXX (informuje
    // o vysledku metody); vraci TRUE pokud se podarilo vylistovat pozadovanou cestu
    // (nezkracenou/nezmenenou)
    // POZNAMKA: pokud potrebujete, aby se FS cesta zkusila otevrit i v odpojenych FS, pouzijte metodu
    // ChangePanelPath (ChangePanelPathToPluginFS odpojene FS ignoruje - pracuje jen s FS otevrenym
    // v panelu nebo otevira nove FS);
    // omezeni: hlavni thread + mimo metody CPluginFSInterfaceAbstract a CPluginDataInterfaceAbstract
    // (hrozi napr. zavreni FS otevreneho v panelu - metode by mohl prestat existovat 'this')
    function ChangePanelPathToPluginFS( panel : Integer; fsName : PChar; fsUserPart : PChar;
                                        failReason : PInteger = nil; suggestedTopIndex : Integer = -1;
                                        suggestedFocusName : PChar = nil; forceUpdate : BOOL = False;
                                        convertPathToInternal : BOOL = False ) : BOOL; virtual; stdcall; abstract;

    // zmena cesty v panelu do odpojeneho pluginoveho FS (viz FSE_DETACHED/FSE_ATTACHED),
    // 'detachedFS' je odpojeny pluginovy FS; pokud neni aktualni cesta v odpojenem FS dostupna,
    // zkousi uspet se zkracenymi cestami (opakovane volani ChangePath a ListCurrentPath, viz
    // CPluginFSInterfaceAbstract); 'panel' je jeden z PANEL_XXX; neni-li 'suggestedTopIndex' -1,
    // bude nastaven jako top-index v panelu (jen pro novou cestu, na zkracene (zmenene) ceste
    // se nenastavuje); neni-li 'suggestedFocusName' NULL, zkusi se nalezt (ignore-case) a vyfokusit
    // polozka stejneho jmena (jen pro novou cestu, na zkracene (zmenene) ceste se neprovadi);
    // neni-li 'failReason' NULL, nastavuje se na jednu z konstant CHPPFR_XXX (informuje o vysledku
    // metody); vraci TRUE pokud se podarilo vylistovat pozadovanou cestu (nezkracenou/nezmenenou)
    // omezeni: hlavni thread + mimo metody CPluginFSInterfaceAbstract a CPluginDataInterfaceAbstract
    // (hrozi napr. zavreni FS otevreneho v panelu - metode by mohl prestat existovat 'this')
    function ChangePanelPathToDetachedFS( panel : Integer; detachedFS : TPluginFSInterfaceAbstract;
                                          failReason : PInteger = nil; suggestedTopIndex : Integer = -1;
                                          suggestedFocusName : PChar = nil ) : BOOL; virtual; stdcall; abstract;

    // zmena cesty v panelu na root prvniho lokalniho fixed drivu, jde o temer jistou zmenu
    // aktualni cesty v panelu; 'panel' je jeden z PANEL_XXX; neni-li 'failReason' NULL,
    // nastavuje se na jednu z konstant CHPPFR_XXX (informuje o vysledku metody); vraci
    // TRUE pokud se podarilo vylistovat root prvniho lokalniho fixed drivu
    // omezeni: hlavni thread + mimo metody CPluginFSInterfaceAbstract a CPluginDataInterfaceAbstract
    // (hrozi napr. zavreni FS otevreneho v panelu - metode by mohl prestat existovat 'this')
    function ChangePanelPathToFixedDrive( panel : Integer; failReason : PInteger = nil ) : BOOL; virtual; stdcall; abstract;

    // provede refresh cesty v panelu (znovu nacte listing a prenese oznaceni, ikony, fokus, atd.
    // do noveho obsahu panelu); diskove a FS cesty se vzdycky nacitaji znovu, cesty do archivu
    // se nacitaji jen pokud doslo ke zmene souboru archivu (kontrola size & time) nebo pokud
    // je 'forceRefresh' TRUE; thumbnaily na diskovych cestach se nacitaji znovu jen pri zmene
    // velikosti souboru nebo zmene datumu/casu posledniho zapisu do souboru nebo pokud je
    // 'forceRefresh' TRUE; 'panel' je jeden z PANEL_XXX; je-li 'focusFirstNewItem' TRUE a
    // v panelu pribyla jen jedina polozka, dojde k fokusu teto nove polozky (pouziva se napr.
    // pro fokus nove vytvoreneho souboru/adresare)
    // omezeni: hlavni thread a navic jen mimo metody CPluginFSInterfaceAbstract a
    // CPluginDataInterfaceAbstract (hrozi napr. zavreni FS otevreneho v panelu - metode by
    // mohl prestat existovat 'this')
    procedure RefreshPanelPath( panel : Integer; forceRefresh : BOOL = False;
                                focusFirstNewItem : BOOL = False ); virtual; stdcall; abstract;

    // postne panelu zpravu o tom, ze by se mel provest refresh cesty (znovu nacte listing a
    // prenese oznaceni, ikony, fokus, atd. do noveho obsahu panelu); refresh se provede az
    // dojde k aktivaci hlavniho okna Salamandera (az skonci suspend-mode); diskove
    // a FS cesty se vzdycky nacitaji znovu, cesty do archivu se nacitaji jen pokud doslo ke
    // zmene souboru archivu (kontrola size & time); 'panel' je jeden z PANEL_XXX; je-li
    // 'focusFirstNewItem' TRUE a v panelu pribyla jen jedina polozka, dojde k fokusu teto
    // nove polozky (pouziva se napr. pro fokus nove vytvoreneho souboru/adresare)
    // mozne volat z libovolneho threadu (pokud hlavni thread nespousti kod uvnitr pluginu,
    // probehne refresh co nejdrive, jinak refresh pocka minimalne do okamziku, kdy hlavni
    // thread opusti plugin)
    procedure PostRefreshPanelPath( panel : Integer; focusFirstNewItem : BOOL = False ); virtual; stdcall; abstract;

    // postne panelu s aktivnim FS 'modifiedFS' zpravu o tom, ze by se mel
    // provest refresh cesty (znovu nacte listing a prenese oznaceni, ikony, fokus, atd. do
    // noveho obsahu panelu); refresh se provede az dojde k aktivaci hlavniho okna Salamandera
    // (az skonci suspend-mode); FS cesta se vzdycky nacte znovu; pokud 'modifiedFS' neni v zadnem
    // panelu, neprovede se nic; je-li 'focusFirstNewItem' TRUE a v panelu pribyla jen jedina
    // polozka, dojde k fokusu teto nove polozky (pouziva se napr. pro fokus nove vytvoreneho
    // souboru/adresare)
    // mozne volat z libovolneho threadu (pokud hlavni thread nespousti kod uvnitr pluginu,
    // probehne refresh co nejdrive, jinak refresh pocka minimalne do okamziku, kdy hlavni
    // thread opusti plugin)
    procedure PostRefreshPanelFS( modifiedFS : TPluginFSInterfaceAbstract;
                                  focusFirstNewItem : BOOL = False ); virtual; stdcall; abstract;

    // zavre odpojeny pluginovy FS (pokud se necha, viz CPluginFSInterfaceAbstract::TryCloseOrDetach),
    // 'detachedFS' je odpojeny pluginovy FS; vraci TRUE pri uspechu (FALSE znamena, ze se odpojeny
    // pluginovy FS nezavrel); 'parent' je parent pripadnych messageboxu (zatim je muze otevrit jen
    // CPluginFSInterfaceAbstract::ReleaseObject)
    // Poznamka: plugin FS otevreny v panelu se zavre napr. pomoci ChangePanelPathToFixedDrive
    // omezeni: hlavni thread + mimo metody CPluginFSInterfaceAbstract (snazime se o zavreni
    // odpojeneho FS - metode by mohl prestat existovat 'this')
    function CloseDetachedFS( parent : HWND; detachedFS : TPluginFSInterfaceAbstract ) : BOOL; virtual; stdcall; abstract;

    // zdvojuje '&' - hodi se pro cesty zobrazovane v menu ('&&' se zobrazi jako '&');
    // 'buffer' je vstupne/vystupni retezec, 'bufferSize' je velikost 'buffer' v bytech;
    // vraci TRUE pokud zdvojenim nedoslo ke ztrate znaku z konce retezce (buffer byl dost
    // veliky)
    // mozne volat z libovolneho threadu
    function DuplicateAmpersands( buffer : PChar; bufferSize : Integer ) : BOOL; virtual; stdcall; abstract;

    // odstrani '&' z textu; najde-li dvojici "&&", nahradi ji jednim znakem '&'
    // mozne volat z libovolneho threadu
    procedure RemoveAmpersands( text : PChar ); virtual; stdcall; abstract;

    // ValidateVarString a ExpandVarString:
    // metody pro overovani a expanzi retezcu s promennymi ve tvaru "$(var_name)", "$(var_name:num)"
    // (num je sirka promenne, jde o ciselnou hodnotu od 1 do 9999), "$(var_name:max)" ("max" je
    // symbol, ktery oznacuje, ze sirka promenne se ridi hodnotou v poli 'maxVarWidths', podrobnosti
    // u ExpandVarString) a "$[env_var]" (expanduje hodnotu promenne prostredi); pouziva se pokud si
    // uzivatel muze zadat format retezce (jako napr. v info-line) priklad retezce s promennymi:
    // "$(files) files and $(dirs) directories" - promenne 'files' a 'dirs';
    // zdrojovy kod pro pouziti v info-line (bez promennych ve tvaru "$(varname:max)") je v DEMOPLUG
    //
    // kontroluje syntaxi 'varText' (retezce s promennymi), vraci FALSE pokud najde chybu, jeji
    // pozici umisti do 'errorPos1' (offset zacatku chybne casti) a 'errorPos2' (offset konce chybne
    // casti); 'variables' je pole struktur CSalamanderVarStrEntry, ktere je ukonceno strukturou s
    // Name==NULL; 'msgParent' je parent message-boxu s chybami, je-li NULL, chyby se nevypisuji
    function ValidateVarString( msgParent : HWND; varText : PChar; var errorPos1 : Integer;
                                var errorPos2 : Integer; variables : PSalamanderVarStrEntry ) : BOOL; virtual; stdcall; abstract;
    //
    // naplni 'buffer' vysledkem expanze 'varText' (retezce s promennymi), vraci FALSE pokud je
    // 'buffer' maly (predpoklada overeni retezce s promennymi pres ValidateVarString, jinak
    // vraci FALSE i pri chybe syntaxe) nebo uzivatel kliknul na Cancel pri chybe environment-variable
    // (nenalezena nebo prilis velka); 'bufferLen' je velikost bufferu 'buffer';
    // 'variables' je pole struktur CSalamanderVarStrEntry, ktere je ukonceno strukturou
    // s Name==NULL; 'param' je ukazatel, ktery se predava do CSalamanderVarStrEntry::Execute
    // pri expanzi nalezene promenne; 'msgParent' je parent message-boxu s chybami, je-li NULL,
    // chyby se nevypisuji; je-li 'ignoreEnvVarNotFoundOrTooLong' TRUE, ignoruji se chyby
    // environment-variable (nenalezena nebo prilis velka), je-li FALSE, zobrazi se messagebox
    // s chybou; neni-li 'varPlacements' NULL, ukazuje na pole DWORDu o '*varPlacementsCount' polozkach,
    // ktere bude naplneno DWORDy slozenymi vzdy z pozice promenne ve vystupnim bufferu (spodni WORD)
    // a poctu znaku promenne (horni WORD); neni-li 'varPlacementsCount' NULL, vraci se v nem pocet
    // naplnenych polozek v poli 'varPlacements' (jde vlastne o pocet promennych ve vstupnim
    // retezci);
    // pokud se tato metoda pouziva jen k expanzi retezce pro jednu hodnotu 'param', meli by
    // byt nastaveny 'detectMaxVarWidths' na FALSE, 'maxVarWidths' na NULL a 'maxVarWidthsCount'
    // na 0; pokud se ovsem pouziva tato metoda pro expanzi retezce opakovane pro urcitou
    // mnozinu hodnot 'param' (napr. u Make File List je to expanze radky postupne pro vsechny
    // oznacene soubory a adresare), ma smysl pouzivat i promenne ve tvaru "$(varname:max)",
    // u techto promennych se sirka urci jako nejvetsi sirka expandovane promenne v ramci cele
    // mnoziny hodnot; omereni nejvetsi sirky expandovane promenne se provadi v prvnim cyklu
    // (pro vsechny hodnoty mnoziny) volani ExpandVarString, v prvnim cyklu ma parametr
    // 'detectMaxVarWidths' hodnotu TRUE a pole 'maxVarWidths' o 'maxVarWidthsCount' polozkach
    // je predem nulovane (slouzi pro ulozeni maxim mezi jednotlivymi volanimi ExpandVarString);
    // samotna expanze pak probiha v druhem cyklu (pro vsechny hodnoty mnoziny) volani
    // ExpandVarString, v druhem cyklu ma parametr 'detectMaxVarWidths' hodnotu FALSE a pole
    // 'maxVarWidths' o 'maxVarWidthsCount' polozkach obsahuje predpocitane nejvetsi sirky
    // (z prvniho cyklu)
    function ExpandVarString( msgParent : HWND; varText : PChar; buffer : PChar;
                              bufferLen : Integer; variables : PSalamanderVarStrEntry;
                              param : Pointer; ignoreEnvVarNotFoundOrTooLong : BOOL = False;
                              varPlacements : PDWORD = nil; varPlacementsCount : PInteger = nil;
                              detectMaxVarWidths : BOOL = False; maxVarWidths : PInteger = nil;
                              maxVarWidthsCount : Integer = 0 ) : BOOL; virtual; stdcall; abstract;

    // nastavi flag load-on-salamander-start (loadit plugin pri startu Salamandera?) pluginu;
    // 'start' je nova hodnota flagu; vraci starou hodnotu flagu; pokud se nikdy nevolala
    // SetFlagLoadOnSalamanderStart, je flag nastaven na FALSE (plugin se neloadi pri startu, ale
    // az v pripade potreby)
    // omezeni: hlavni thread (jinak muze dojit ke zmenam v konfiguraci pluginu behem volani)
    function SetFlagLoadOnSalamanderStart( start : BOOL ) : BOOL; virtual; stdcall; abstract;

    // nastavi volajicimu pluginu priznak, ze se ma pri nejblizsi mozne prilezitosti unloadnout
    // (jakmile budou provedeny vsechny postnute prikazy menu (viz PostMenuExtCommand), nebudou
    // v message-queue hl. threadu zadne message a Salamander nebude "busy");
    // POZOR: pokud se vola z jineho nez hlavniho threadu, muze dojit k zadosti o unload (probiha
    // v hlavnim threadu) dokonce drive nez skonci PostUnloadThisPlugin (dalsi informace o
    // unloadu - viz CPluginInterfaceAbstract::Release)
    // mozne volat z libovolneho threadu (ale az po ukonceni entry-pointu pluginu, dokud se
    // spousti entry-point, je mozne metodu volat jen z hlavniho threadu)
    procedure PostUnloadThisPlugin; virtual; stdcall; abstract;

    // vraci postupne moduly Salamandera (spustitelny soubor a .spl soubory instalovanych
    // pluginu, vse vcetne verzi); 'index' je vstupne/vystupni promenna, ukazuje na int,
    // ve kterem je pri prvnim volani 0, hodnotu pro dalsi volani si funkce ulozi pri navratu
    // (pouziti: na zacatku vynulovat, pak nemenit); 'module' je buffer pro jmeno modulu
    // (min. velikost MAX_PATH znaku); 'version' je buffer pro verzi modulu (min. velikost
    // MAX_PATH znaku); vraci FALSE pokud 'module' + 'version' neni naplnene a jiz neni
    // zadny dalsi modul, vraci TRUE pokud 'module' + 'version' obsahuje dalsi modul
    // omezeni: hlavni thread (jinak muze dojit ke zmenam v konfiguraci pluginu behem
    // volani - add/remove)
    function EnumInstalledModules( index : PInteger; module : PChar; version : PChar ) : BOOL; virtual; stdcall; abstract;

    // vola 'loadOrSaveFunc' pro load nebo save konfigurace; je-li 'load' TRUE, jde o load
    // konfigurace, pokud plugin podporuje "load/save configuration" a v dobe volani existuje
    // soukromy klic pluginu v registry, vola se 'loadOrSaveFunc' pro tento klic, jinak
    // se vola load defaultni konfigurace (parametr 'regKey' funkce 'loadOrSaveFunc' je NULL);
    // je-li 'load' FALSE, jde o save konfigurace, vola se jen tehdy, pokud plugin podporuje
    // "load/save configuration"; 'param' je uzivatelsky parametr a je predavan do 'loadOrSaveFunc'
    // omezeni: hlavni thread, v entry-pointu lze volat az po akt. verzi "SetBasicPluginData",
    // drive nemusi byt znamo jestli existuje podpora pro "load/save configuration" a jmeno
    // soukromeho klice v registry
    procedure CallLoadOrSaveConfiguration( load : BOOL; loadOrSaveFunc : TSalLoadOrSaveConfigurationProc;
                                           param : Pointer ); virtual; stdcall; abstract;

    // ulozi 'text' o delce 'textLen' (-1 znamena "pouzij strlen") na clipboard a to jak multibyte,
    // tak Unicodove (jinak napr. Notepad nezvlada cestinu), v pripade uspechu muze (je-li 'echo' TRUE)
    // zobrazit hlaseni "Text was successfully copied to clipboard." (parent messageboxu bude
    // 'echoParent'); vraci TRUE pri uspechu
    // mozne volat z libovolneho threadu
    function CopyTextToClipboard( text : PChar; textLen : Integer; showEcho : BOOL;
                                  echoParent : HWND ) : BOOL; virtual; stdcall; abstract;

    // ulozi unicodovy 'text' o delce 'textLen' (-1 znamena "pouzij wcslen") na clipboard a to jak
    // unicodove, tak multibyte (jinak napr. MSVC6.0 nezvlada cestinu), v pripade uspechu muze (je-li
    // 'echo' TRUE) zobrazit hlaseni "Text was successfully copied to clipboard." (parent messageboxu
    // bude 'echoParent'); vraci TRUE pri uspechu
    // mozne volat z libovolneho threadu
    function CopyTextToClipboardW( text : PWChar; textLen : Integer; showEcho : BOOL;
                                   echoParent : HWND ) : BOOL; virtual; stdcall; abstract;

    // spusteni prikazu menu s identifikacnim cislem 'id' v hlavnim threadu (volani
    // CPluginInterfaceForMenuExtAbstract::ExecuteMenuItem(NULL, main-window-hwnd, 'id', 0),
    // navratova hodnota se ignoruje); je-li 'waitForSalIdle' FALSE, pouzije se ke spusteni
    // zprava postnuta hlavnimu oknu (tuto zpravu doruci kazda bezici message-loopa v hl.
    // threadu - i modalni dialogy/messageboxy, i ty otevrene pluginem), takze hrozi
    // vicenasobny vstup do pluginu; je-li 'waitForSalIdle' TRUE, je 'id' omezeno na interval
    // <0, 999999> a prikaz se spusti jakmile nebudou v message-queue hl. threadu zadne zpravy
    // a Salamander nebude "busy" (nebude otevreny zadny modalni dialog a nebude se zpracovavat
    // zadna zprava);
    // POZOR: pokud se vola z jineho nez hlavniho threadu, muze dojit ke spusteni prikazu menu
    // (probiha v hlavnim threadu) dokonce drive nez skonci PostMenuExtCommand
    // mozne volat z libovolneho threadu a je-li 'waitForSalIdle' FALSE, je nutne pockat az po volani
    // CPluginInterfaceAbstract::GetInterfaceForMenuExt (vola se po entry-pointu z hlavniho threadu)
    procedure PostMenuExtCommand( id : Integer; waitForSalIdle : BOOL ); virtual; stdcall; abstract;

    // zjisti jestli je velka sance (jiste se to urcit neda), ze Salamander v pristich par
    // okamzicich nebude "busy" (nebude otevreny zadny modalni dialog a nebude se zpracovavat
    // zadna zprava) - v tomto pripade vraci TRUE (jinak FALSE); neni-li 'lastIdleTime' NULL,
    // vraci se v nem GetTickCount() z okamziku posledniho prechodu z "idle" do "busy" stavu;
    // da se pouzit napr. jako predpoved pro doruceni prikazu postnuteho pomoci
    // CSalamanderGeneralAbstract::PostMenuExtCommand s parametrem 'waitForSalIdle'==TRUE;
    // je mozne volat z libovolneho threadu
    function SalamanderIsNotBusy( lastIdleTime : PDWORD ) : BOOL; virtual; stdcall; abstract;

    // nastavi zpravu, kterou ma vypsat Bug Report dialog pokud dojde k padu uvnitr pluginu
    // (uvnitr pluginu = alespon jedna call-stack-message ulozena z pluginu) a umozni predefinovat
    // standardni bug-reportovou e-mailovou adresu (support@altap.cz); 'message' je nova zprava
    // (NULL znamena "zadna zprava"); 'email' je nova e-mailova adresa (NULL znamena "pouzij
    // standardni"; max. delka e-mailu je 100 znaku); tuto metodu je mozne volat opakovane, puvodni
    // zprava a e-mail se prepisuji; Salamander si pro dalsi spusteni nepamatuje ani zpravu ani
    // e-mail, takze je nutne tuto metodu vzdy pri loadu pluginu (nejlepe v entry-pointu) znovu
    // zavolat
    // omezeni: hlavni thread (jinak muze dojit ke zmenam v konfiguraci pluginu behem volani)
    procedure SetPluginBugReportInfo( message : PChar; email : PChar ); virtual; stdcall; abstract;

    // zjisti jestli je plugin nainstalovan (ovsem nezjisti, jestli se da naloadit - jestli ho
    // user napr. nesmazal jen z disku); 'pluginSPL' identifikuje plugin - jde o pozadovanou
    // koncovou cast plne cesty .SPL souboru pluginu (napr. "ieviewer\\ieviewer.spl" identifikuje
    // IEViewer dodavany se Salamanderem); vraci TRUE pokud je plugin nainstalovan
    // omezeni: hlavni thread (jinak muze dojit ke zmenam v konfiguraci pluginu behem volani)
    function IsPluginInstalled( pluginSPL : PChar ) : BOOL; virtual; stdcall; abstract;

    // otevre soubor ve vieweru implementovanem v pluginu nebo internim text/hex vieweru;
    // je-li 'pluginSPL' NULL, ma byt pouzit interni text/hex viewer, jinak identifikuje plugin
    // vieweru - jde o pozadovanou koncovou cast plne cesty .SPL souboru pluginu (napr.
    // "ieviewer\\ieviewer.spl" identifikuje IEViewer dodavany se Salamanderem); 'pluginData'
    // je struktura dat obsahujici jmeno viewovaneho souboru a nepovinne obsahuje take rozsirene
    // parametry vieweru (viz popis CSalamanderPluginViewerData); je-li 'useCache' FALSE jsou
    // 'rootTmpPath' a 'fileNameInCache' ignorovany a dojde jen k otevreni souboru ve viewru;
    // je-li 'useCache' TRUE, bude soubor nejprve presunut do diskove cache pod jmenem souboru
    // 'fileNameInCache' (jmeno je bez cesty), pak otevren ve vieweru a po uzavreni vieweru bude
    // z diskove cache odstranen, v tomto pripade je pozadovano, aby soubor 'pluginData->FileName'
    // byl umisten na stejnem disku jako diskova cache (je-li 'rootTmpPath' NULL, je diskova cache
    // ve Windows TEMP adresari, jinak je cesta do disk cache v 'rootTmpPath'; pro prejmenovani
    // do disk cache pres Win32 API funkci MoveFile), idealni je pouziti SalGetTempFileName
    // s parametrem 'path' rovnym 'rootTmpPath'; vraci TRUE pri uspesnem otevreni souboru ve
    // vieweru; vraci FALSE pokud dojde k chybe pri otevirani (konkretni duvod je ulozen v
    // 'error' - 0 - uspech, 1 - nelze naloadit plugin, 2 - ViewFile z pluginu vratil
    // chybu, 3 - nelze presunout soubor do disk cache), je-li 'useCache' TRUE, dojde
    // k odstraneni souboru z disku (jako po uzavreni vieweru)
    // omezeni: hlavni thread (jinak muze dojit ke zmenam v konfiguraci pluginu behem volani),
    // navic neni mozne volat z entry-pointu (load pluginu neni reentrantni)
    function ViewFileInPluginViewer( pluginSPL : PChar; pluginData : TSalamanderPluginViewerData;
                                     useCache : BOOL; rootTmpPath : PChar;
                                     fileNameInCache : PChar; var error : Integer ) : BOOL; virtual; stdcall; abstract;

    // co nejdrive informuje Salamandera, pak vsechny nactene (loaded) pluginy a pak vsechny otevrene
    // FS (v panelech i odpojene) o zmene na ceste 'path' (diskova nebo FS cesta); je dulezite pro
    // cesty na nichz nelze monitorovat zmeny automaticky (viz FindFirstChangeNotification) nebo
    // si toto monitorovani (auto-refresh) uzivatel vypnul, u FS si monitorovani zmen zajistuje
    // sam plugin; informovani o zmenach probehne co nejdrive (pokud hlavni thread nespousti kod
    // uvnitr pluginu, probehne refresh po doruceni zpravy hlavnimu oknu a pripadne po opetovnem
    // povoleni refreshovani (po zavreni dialogu, atp.), jinak refresh pocka minimalne do okamziku,
    // kdy hlavni thread opusti plugin); 'includingSubdirs' je TRUE v pripade, ze se zmena muze
    // projevit i v podadresarich 'path';
    // POZOR: pokud se vola z jineho nez hlavniho threadu, muze dojit k informovani o zmenach
    // (probiha v hlavnim threadu) dokonce drive nez skonci PostChangeOnPathNotification
    // mozne volat z libovolneho threadu
    procedure PostChangeOnPathNotification( path : PChar; includingSubdirs : BOOL ); virtual; stdcall; abstract;

    // zkusi pristup na windowsovou cestu 'path' (normal nebo UNC), probiha ve vedlejsim threadu, takze
    // umoznuje prerusit zkousku klavesou ESC (po jiste dobe vybali okenko s hlasenim o ESC)
    // 'echo' TRUE znamena povoleny vypis chybove hlasky (pokud cesta nebude pristupna);
    // 'err' ruzne od ERROR_SUCCESS v kombinaci s 'echo' TRUE pouze zobrazi chybu (na cestu
    // se jiz nepristupuje); 'parent' je parent messageboxu; vraci ERROR_SUCCESS v pripade,
    // ze je cesta v poradku, jinak vraci standardni windowsovy kod chyby nebo ERROR_USER_TERMINATED
    // v pripade, ze uzivatel pouzil klavesu ESC k preruseni testu
    // omezeni: hlavni thread (opakovane volani neni mozne a hl. thread tuto metodu pouziva)
    function SalCheckPath( echo : BOOL; path : PChar; err : DWORD; parent : HWND ) : DWORD; virtual; stdcall; abstract;

    // zkusi jestli je windowsovou cesta 'path' pristupna, prip. obnovi sitova spojeni (pokud jde
    // o normalni cestu, zkusi ozivit zapamatovane sit. spojeni, pokud jde o UNC cestu, umozni login
    // s novym uzivatelskym jmenem a heslem); vraci TRUE pokud je cesta pristupna; 'parent' je parent
    // messageboxu a dialogu; 'tryNet' je TRUE pokud ma smysl zkouset obnovit sitova spojeni
    // (pri FALSE degraduje na SalCheckPath; je zde jen pro moznost optimalizace)
    // omezeni: hlavni thread (opakovane volani neni mozne a hl. thread tuto metodu pouziva)
    function SalCheckAndRestorePath( parent : HWND; path : PChar; tryNet : BOOL ) : BOOL; virtual; stdcall; abstract;

    // slozitejsi varianta metody SalCheckAndRestorePath; zkusi jestli je windowsovou cesta 'path'
    // pristupna, prip. ji zkrati; je-li 'tryNet' TRUE, zkusi i obnovit sitove spojeni a nastavi
    // 'tryNet' na FALSE (pokud jde o normalni cestu, zkusi ozivit zapamatovane sit. spojeni, pokud
    // jde o UNC cestu, umozni login s novym uzivatelskym jmenem a heslem); je-li 'donotReconnect'
    // TRUE, zjisti se pouze chyba, obnova spojeni uz se neprovede; vraci 'err' (windowsovy kod chyby
    // aktualni cesty), 'lastErr' (kod chyby vedouci ke zkraceni cesty), 'pathInvalid' (TRUE pokud
    // se zkusila obnova sit. spojeni bez uspechu), 'cut' (TRUE pokud je vysledna cesta zkracena);
    // 'parent' je parent messageboxu; vraci TRUE pokud je vysledna cesta 'path' pristupna
    // omezeni: hlavni thread (opakovane volani neni mozne a hl. thread tuto metodu pouziva)
    function SalCheckAndRestorePathWithCut( parent : HWND; path : PChar; var tryNet : BOOL;
                                            var err : DWORD; var lastErr : DWORD;
                                            var pathInvalid : BOOL; var cut : BOOL;
                                            donotReconnect : BOOL ) : BOOL; virtual; stdcall; abstract;

    // rozpozna o jaky typ cesty (FS/windowsova/archiv) jde a postara se o rozdeleni na
    // jeji casti (u FS jde o fs-name a fs-user-part, u archivu jde o path-to-archive a
    // path-in-archive, u windowsovych cest jde o existujici cast a zbytek cesty), u FS cest
    // se nic nekontroluje, u windowsovych (normal + UNC) cest se kontroluje kam az cesta existuje
    // (prip. obnovi sitove spojeni), u archivu se kontroluje existence souboru archivu
    // (rozliseni archivu dle pripony);
    // 'path' je plna nebo relativni cesta (buffer min. MAX_PATH znaku; u relativnich cest se
    // uvazuje aktualni cesta 'curPath' (neni-li NULL) jako zaklad pro vyhodnoceni plne cesty;
    // 'curPathIsDiskOrArchive' je TRUE pokud je 'curPath' windowsova nebo archivova cesta;
    // pokud je aktualni cesta archivova, obsahuje 'curArchivePath' jmeno archivu, jinak je NULL),
    // do 'path' se ulozi vysledna plna cesta (musi byt min. MAX_PATH znaku); vraci TRUE pri
    // uspesnem rozpoznani, pak 'type' je typ cesty (viz PATH_TYPE_XXX) a 'secondPart' je nastavene:
    // - do 'path' na pozici za existujici cestu (za '\\' nebo na konci retezce; existuje-li
    //   v ceste soubor, ukazuje za cestu k tomuto souboru) (typ cesty windows)
    // - za soubor archivu (typ cesty archiv)
    // - za ':' za nazvem file-systemu - user-part cesty file-systemu (typ cesty FS);
    // pokud vraci TRUE je jeste 'isDir' nastavena na:
    // - TRUE pokud existujici cast cesty je adresar, FALSE == soubor (typ cesty windows)
    // - FALSE pro cesty typu archiv a FS;
    // pokud vrati FALSE, userovi byla vypsana chyba (az na jednu vyjimku - viz popis SPP_INCOMLETEPATH),
    // ktera pri rozpoznavani nastala (neni-li 'error' NULL, vraci se v nem jedna z konstant SPP_XXX);
    // 'errorTitle' je titulek messageboxu s chybou; pokud je 'nextFocus' != NULL a windowsova/archivova
    // cesta neobsahuje '\\' nebo na '\\' jen konci, nakopiruje se cesta do 'nextFocus' (viz SalGetFullName);
    // POZOR: pouziva SalGetFullName, proto je dobre napred zavolat metodu
    //        CSalamanderGeneralAbstract::SalUpdateDefaultDir)
    // omezeni: hlavni thread (opakovane volani neni mozne a hl. thread tuto metodu pouziva)
    function SalParsePath( parent : HWND; path : PChar; var _type : Integer; var isDir : BOOL;
                           var secondPart : PChar; errorTitle : PChar; nextFocus : PChar;
                           curPathIsDiskOrArchive : BOOL; curPath : PChar;
                           curArchivePath : PChar; error : PInteger ) : BOOL; virtual; stdcall; abstract;

    // ziska z windowsove cilove cesty existujici cast a operacni masku; pripadnou neexistujici cast
    // umozni vytvorit; pri uspechu vraci TRUE a existujici windowsovou cilovou cestu (v 'path')
    // a nalezenou operacni masku (v 'mask' - ukazuje do bufferu 'path', ale cesta a maska jsou oddelene
    // nulou; pokud v ceste neni maska, automaticky vytvori masku "*.*"); 'parent' - parent pripadnych
    // messageboxu; 'title' + 'errorTitle' jsou titulky messageboxu s informaci + chybou; 'selCount' je
    // pocet oznacenych souboru a adresaru; 'path' je na vstupu cilova cesta ke zpracovani, na vystupu
    // (alespon 2 * MAX_PATH znaku) existujici cilova cesta; 'secondPart' ukazuje do 'path' na pozici
    // za existujici cestu (za '\\' nebo na konec retezce; existuje-li v ceste soubor, ukazuje za cestu
    // k tomuto souboru); 'pathIsDir' je TRUE/FALSE pokud existujici cast cesty je adresar/soubor;
    // 'backslashAtEnd' je TRUE pokud byl pred provedenim "parse" na konci 'path' backslash (napr.
    // SalParsePath takovy backslash rusi); 'dirName' + 'curDiskPath' nejsou NULL pokud je oznaceny
    // max. jeden soubor/adresar (jeho jmeno bez cesty je v 'dirName'; pokud neni nic oznacene, bere
    // se focus) a aktualni cesta je windowsova (cesta je v 'curDiskPath'); 'mask' je na vystupu
    // ukazatel na operacni masku do bufferu 'path'; pokud je v ceste chyba, vraci metoda FALSE,
    // problem uz byl uzivateli ohlasen
    // mozne volat z libovolneho threadu
    function SalSplitWindowsPath( parent : HWND; title : PChar; errorTitle : PChar;
                                  selCount : Integer; path : PChar; secondPart : PChar;
                                  pathIsDir : BOOL; backslashAtEnd : BOOL;
                                  dirName : PChar; curDiskPath : PChar; var mask : PChar ) : BOOL; virtual; stdcall; abstract;

    // ziska z cilove cesty existujici cast a operacni masku; pripadnou neexistujici cast rozpozna; pri
    // uspechu vraci TRUE, relativni cestu k vytvoreni (v 'newDirs'), existujici cilovou cestu (v 'path';
    // existujici jen za predpokladu vytvoreni relativni cesty 'newDirs') a nalezenou operacni masku
    // (v 'mask' - ukazuje do bufferu 'path', ale cesta a maska jsou oddelene nulou; pokud v ceste neni
    // maska, automaticky vytvori masku "*.*"); 'parent' - parent pripadnych messageboxu;
    // 'title' + 'errorTitle' jsou titulky messageboxu s informaci + chybou; 'selCount' je pocet oznacenych
    // souboru a adresaru; 'path' je na vstupu cilova cesta ke zpracovani, na vystupu (alespon 2 * MAX_PATH
    // znaku) existujici cilova cesta (vzdy konci backslashem); 'afterRoot' ukazuje do 'path' za root cesty
    // (za '\\' nebo na konec retezce); 'secondPart' ukazuje do 'path' na pozici za existujici cestu (za
    // '\\' nebo na konec retezce; existuje-li v ceste soubor, ukazuje za cestu k tomuto souboru);
    // 'pathIsDir' je TRUE/FALSE pokud existujici cast cesty je adresar/soubor; 'backslashAtEnd' je
    // TRUE pokud byl pred provedenim "parse" na konci 'path' backslash (napr. SalParsePath takovy
    // backslash rusi); 'dirName' + 'curPath' nejsou NULL pokud je oznaceny max. jeden soubor/adresar
    // (jeho jmeno bez cesty je v 'dirName'; jeho cesta je v 'curPath'; pokud neni nic oznacene, bere
    // se focus); 'mask' je na vystupu ukazatel na operacni masku do bufferu 'path'; neni-li 'newDirs' NULL,
    // pak jde o buffer (o velikosti alespon MAX_PATH) pro relativni cestu (vzhledem k existujici ceste
    // v 'path'), kterou je nutne vytvorit (uzivatel s vytvorenim souhlasi, byl pouzit stejny dotaz jako
    // u kopirovani z disku na disk; prazdny retezec = nic nevytvaret); je-li 'newDirs' NULL a je-li
    // potreba vytvorit nejakou relativni cestu, je jen vypsana chyba; 'isTheSamePathF' je funkce pro
    // porovnani dvou cest (potrebna jen pokud 'curPath' neni NULL), je-li NULL pouzije se IsTheSamePath;
    // pokud je v ceste chyba, vraci metoda FALSE, problem uz byl uzivateli ohlasen
    // mozne volat z libovolneho threadu
    function SalSplitGeneralPath( parent : HWND; title : PChar; errorTitle : PChar;
                                  selCount : Integer; path : PChar; afterRoot : PChar;
                                  secondPart : PChar; pathIsDir : BOOL; backslashAtEnd : BOOL;
                                  dirName : PChar; curPath : PChar; var mask : PChar;
                                  newDirs : PChar; isTheSamePathF : TSGP_IsTheSamePathFunc ) : BOOL; virtual; stdcall; abstract;

    // odstrani ".." (vynecha ".." spolu s jednim podadresarem vlevo) a "." (vynecha jen ".")
    // z cesty; podminkou je backslash jako oddelovac podadresaru; 'afterRoot' ukazuje za root
    // zpracovavane cesty (zmeny cesty se deji jen za 'afterRoot'); vraci TRUE pokud se upravy
    // podarily, FALSE pokud se nedaji odstranit ".." (vlevo uz je root)
    // mozne volat z libovolneho threadu
    function SalRemovePointsFromPath( afterRoot : PChar ) : BOOL; virtual; stdcall; abstract;

    // vraci parametr z konfigurace Salamandera; 'paramID' identifikuje o jaky parametr jde
    // (viz konstanty SALCFG_XXX); 'buffer' ukazuje na buffer, do ktereho se nakopiruji
    // data parametru, velikost bufferu je 'bufferSize'; neni-li 'type' NULL, vraci se
    // v nem jedna z konstant SALCFGTYPE_XXX nebo SALCFGTYPE_NOTFOUND (pokud parametr s
    // 'paramID' nebyl nalezen); vraci TRUE pokud je 'paramID' platne a zaroven pokud se
    // hodnota parametru konfigurace vejde do bufferu 'buffer'
    // poznamka: zmeny v konfiguraci Salamandera jsou hlaseny pomoci udalosti
    //           PLUGINEVENT_CONFIGURATIONCHANGED (viz metoda CPluginInterfaceAbstract::Event)
    // omezeni: hlavni thread, zmeny konfigurace se deji jen v hl. threadu (neobsahuje jinou
    //          synchronizaci)
    function GetConfigParameter( paramID : Integer; buffer : Pointer; bufferSize : Integer;
                                 _type : PInteger ) : BOOL; virtual; stdcall; abstract;

    // meni velikost pismen ve jmene souboru (jmeno je bez cesty); 'tgtName' je buffer pro vysledek
    // (velikost je min. pro ulozeni retezce 'srcName'); 'srcName' je jmeno souboru (zapisuje se do nej,
    // ale pred navratem z metody je vzdy obnoveno); 'format' je format vysledku (1 - velka pocatecni
    // pismena slov, 2 - komplet mala pismena, 3 - komplet velka pismena, 4 - beze zmen, 5 - pokud
    // je DOS jmeno (8.3) -> velka pocatecni pismena slov, 6 - soubor malymi, adresar velkymi pismeny,
    // 7 - velka pocatecni pismena ve jmene a mala pismena v pripone);
    // 'changedParts' urcuje jake casti jmena se maji menit (0 - meni jmeno i priponu, 1 - meni jen
    // jmeno (mozne jen s format == 1, 2, 3, 4), 2 - meni jen priponu (mozne jen s
    // format == 1, 2, 3, 4)); 'isDir' je TRUE pokud jde o jmeno adresare
    // mozne volat z libovolneho threadu
    procedure AlterFileName( tgtName : PChar; srcName : PChar; format : Integer;
                             changedParts : Integer; isDir : BOOL ); virtual; stdcall; abstract;

    // zobrazi/schova zpravu v okenku ve vlastnim threadu (neodcerpa message-queue); zobrazi
    // najednou jen jednu zpravu, opakovane volani ohlasi chybu do TRACE (neni fatalni);
    // POZN.: pouziva se v SalCheckPath a dalsich rutinach, muze tedy dojit ke kolizi mezi
    //        pozadavky na otevreni okenek (neni fatalni, jen se nezobrazi)
    // vse je mozne volat z libovolneho threadu (ale okenko se musi obsluhovat jen
    // z jednoho threadu - nelze jej ukazat z jednoho threadu a schovat z druheho)
    //
    // otevreni okenka s textem 'message' se zpodenim 'delay' (v ms), jen pokud je 'hForegroundWnd' NULL
    // nebo identifikuje okno na popredi (foreground)
    // 'message' muze byt vicerakova; jednotlive radky se oddeluji znakem '\n'
    // 'caption' muze byt NULL: pouzije se pak "Altap Salamander"
    // 'showCloseButton' udava, zda bude okenko obsahovat tlacitko Close; ekvivalent ke klavese Escape
    procedure CreateSafeWaitWindow( message : PChar; caption : PChar; delay : Integer;
                                    showCloseButton : BOOL; hForegroundWnd : HWND ); virtual; stdcall; abstract;
    // zavreni okenka
    procedure DestroySafeWaitWindow; virtual; stdcall; abstract;
    // schovani/zobrazeni okenka (je-li otevrene); volat jako reakci na WM_ACTIVATE z okna hForegroundWnd:
    //    case WM_ACTIVATE:
    //    {
    //      ShowSafeWaitWindow(LOWORD(wParam) != WA_INACTIVE);
    //      break;
    //    }
    // Pokud je thread (ze ktereho bylo okenko vytvoreno) zamestnan, nedochazi
    // k distribuci zprav, takze nedojde ani k doruceni WM_ACTIVATE pri kliknuti
    // na jinou aplikaci. Zpravy se doruci az ve chvili zobrazeni messageboxu,
    // coz presne potrebujeme: docasne se nechame schovat a pozdeji (po zavreni
    // messageboxu a aktivaci okna hForegroundWnd) zase zobrazit.
    procedure ShowSafeWaitWindow( show : BOOL ); virtual; stdcall; abstract;
    // po zavolani CreateSafeWaitWindow nebo ShowSafeWaitWindow vraci funkce FALSE az do doby,
    // kdy user kliknul mysi na Close tlacitko (pokud je zobrazeno); pak vraci TRUE
    function GetSafeWaitWindowClosePressed : BOOL; virtual; stdcall; abstract;
    // slouzi pro dodatecnou zmenu textu v okenku
    // POZOR: nedochazi k novemu layoutovani okna a pokud dojde k vetsimu natazeni
    // textu, bude orezan; pouzivat napriklad pro countdown: 60s, 55s, 50s, ...
    procedure SetSafeWaitWindowText( message : PChar ); virtual; stdcall; abstract;

    // najde v disk-cache existujici kopii souboru a zamkne ji (zakaze jeji zruseni); 'uniqueFileName'
    // je unikatni nazev originalniho souboru (podle tohoto nazvu se prohledava disk-cache; melo by
    // stacit plne jmeno souboru v salamanderovske forme - "fs-name:fs-user-part"; POZOR: nazev se
    // porovnava "case-sensitive", pokud plugin vyzaduje "case-insensitive", musi vsechny nazvy
    // prevadet napr. na mala pismena - viz CSalamanderGeneralAbstract::ToLowerCase); v 'tmpName'
    // se vraci ukazatel (platny az do zruseni kopie souboru v disk-cache) na plne jmeno kopie souboru,
    // ktera je umistena v docasnem adresari; 'fileLock' je zamek kopie souboru, jde o systemovy event
    // ve stavu nonsignaled, ktery po zpracovani kopie souboru prejde do stavu signaled (je nutne
    // pouzit metodu UnlockFileInCache; plugin dava signal, ze kopii v disk-cache jiz lze zrusit);
    // pokud nebyla kopie nalezena vraci FALSE a 'tmpName' NULL (jinak vraci TRUE)
    // mozne volat z libovolneho threadu
    function GetFileFromCache( uniqueFileName : PChar; var tmpName : PChar; fileLock : THandle ) : BOOL; virtual; stdcall; abstract;

    // odemkne zamek kopie souboru v disk-cache (nastavi 'fileLock' do stavu signaled, vyzve
    // disk-cache k provedeni kontroly zamku, a pak nastavi 'fileLock' zpet do stavu nonsignaled);
    // pokud slo o posledni zamek, muze dojit ke zruseni kopie, kdy dojde ke zruseni kopie zalezi
    // na velikosti disk-cache na disku; zamek muze byt pouzity i pro vice kopii souboru (zamek
    // musi byt typu "manual reset", jinak se po odemknuti prvni kopie zamek nastavi do stavu
    // nonsignaled a odemykani skonci), v tomto pripade probehne odemknuti u vsech kopii
    // mozne volat z libovolneho threadu
    procedure UnlockFileInCache( fileLock : THandle ); virtual; stdcall; abstract;

    // vlozi (presune) kopii souboru do disk-cache (vkladana kopie neni zamcena, takze kdykoliv muze dojit
    // k jejimu zruseni); 'uniqueFileName' je unikatni nazev originalniho souboru (podle tohoto
    // nazvu se prohledava disk-cache; melo by stacit plne jmeno souboru v salamanderovske
    // forme - "fs-name:fs-user-part"; POZOR: nazev se porovnava "case-sensitive", pokud plugin
    // vyzaduje "case-insensitive", musi vsechny nazvy prevadet napr. na mala pismena - viz
    // CSalamanderGeneralAbstract::ToLowerCase); 'nameInCache' je jmeno kopie souboru, ktera bude umistena
    // v docasnem adresari (ocekava se zde posledni cast jmena originalniho souboru, aby pozdeji
    // uzivateli pripominala originalni soubor); 'newFileName' je plne jmeno ukladane kopie souboru,
    // ktera bude presunuta do disk-cache pod jmenem 'nameInCache', musi byt umistena na stejnem disku
    // jako diskova cache (je-li 'rootTmpPath' NULL, je diskova cache ve Windows TEMP adresari, jinak
    // je cesta do disk-cache v 'rootTmpPath'; pro prejmenovani do disk cache pres Win32 API funkci
    // MoveFile); 'newFileName' je idealni ziskat volanim SalGetTempFileName s parametrem 'path' rovnym
    // 'rootTmpPath'); v 'newFileSize' je velikost ukladane kopie souboru; vraci TRUE pri uspechu
    // (soubor byl presunut do disk-cache - zmizel z puvodniho mista na disku), vraci FALSE pri
    // interni chybe nebo pokud soubor jiz je v disk-cache (neni-li 'alreadyExists' NULL, vraci
    // se v nem TRUE pokud soubor jiz je v disk-cache)
    // POZN.: pokud plugin vyuziva disk-cache, mel by alespon pri unloadu pluginu zavolat
    //        CSalamanderGeneralAbstract::RemoveFilesFromCache("fs-name:"), jinak budou
    //        jeho kopie souboru zbytecne prekazet v disk-cache
    // mozne volat z libovolneho threadu
    function MoveFileToCache( uniqueFileName : PChar; nameInCache : PChar; rootTmpPath : PChar;
                              newFileName : PChar; var newFileSize : TLargeInteger;
                              alreadyExists : PBOOL ) : BOOL; virtual; stdcall; abstract;

    // odstrani z disk-cache kopii souboru, jejiz unikatni nazev je 'uniqueFileName' (POZOR: nazev
    // se porovnava "case-sensitive", pokud plugin vyzaduje "case-insensitive", musi vsechny nazvy
    // prevadet napr. na mala pismena - viz CSalamanderGeneralAbstract::ToLowerCase); pokud se kopie
    // souboru jeste pouziva, odstrani se az to bude mozne (az se zavrou viewery), kazdopadne uz ji
    // disk-cache nikomu neposkytne jako platnou kopii souboru (je oznacena jako out-of-date)
    // mozne volat z libovolneho threadu
    procedure RemoveOneFileFromCache( uniqueFileName : PChar ); virtual; stdcall; abstract;

    // odstrani z disk-cache vsechny kopie souboru, jejichz unikatni nazvy zacinaji na 'fileNamesRoot'
    // (pouziva se pri uzavreni file-systemu, kdyz uz je dale nezadouci cachovat downloadene kopie
    // souboru; POZOR: nazvy se porovnavaji "case-sensitive", pokud plugin vyzaduje "case-insensitive",
    // musi vsechny nazvy prevadet napr. na mala pismena - viz CSalamanderGeneralAbstract::ToLowerCase);
    // pokud se kopie souboru jeste pouzivaji, odstrani se az to bude mozne (az se odemknou
    // napr. po zavreni vieweru), kazdopadne uz je disk-cache nikomu neposkytne jako platne kopie
    // souboru (jsou oznacene jako out-of-date)
    // mozne volat z libovolneho threadu
    procedure RemoveFilesFromCache( fileNamesRoot : PChar ); virtual; stdcall; abstract;

    // vraci postupne konverzni tabulky (nactene ze souboru convert\XXX\convert.cfg
    // v instalaci Salamandera - XXX je prave pouzivany adresar konverznich tabulek);
    // 'parent' je parent messageboxu (je-li NULL, je parent hlavni okno);
    // 'index' je vstupne/vystupni promenna, ukazuje na int, ve kterem je pri prvnim volani 0,
    // hodnotu pro dalsi volani si funkce ulozi pri navratu (pouziti: na zacatku vynulovat, pak
    // nemenit); vraci FALSE, pokud jiz neexistuje zadna dalsi tabulka; pokud vraci TRUE, obsahuje
    // 'name' (neni-li NULL) odkaz na jmeno konverze (muze obsahovat '&' - podtrzeny znak v menu) nebo NULL
    // jde-li o separator a 'table' (neni-li NULL) odkaz na 256-bytovou konverzni tabulku nebo NULL
    // jde-li o separator; odkazy 'name' a 'table' jsou platne po celou dobu behu Salamandera (neni
    // nutne kopirovat obsah)
    // POZOR: ukazatel 'table' pouzivat timto zpusobem (nutne pretypovani na "unsigned"):
    //        *s = table[(unsigned char)*s]
    // mozne volat z libovolneho threadu
    function EnumConversionTables( parent : HWND; index : PInteger; name : PChar; table : PChar ) : BOOL; virtual; stdcall; abstract;

    // vraci konverzni tabulku 'table' (buffer min. 256 znaku) pro konverzi 'conversion' (jmeno
    // konverze viz soubor convert\XXX\convert.cfg v instalaci Salamandera, napr. "ISO-8859-2 - CP1250";
    // znaky <= ' ' a '-' a '&' ve jmene pri hledani nehraji roli; hleda se bez ohledu na velka a mala
    // pismena); 'parent' je parent messageboxu (je-li NULL, je parent hlavni okno); vraci TRUE
    // pokud byla konverze nalezena (jinak neni obsah 'table' platny);
    // POZOR: pouzivat timto zpusobem (nutne pretypovani na "unsigned"): *s = table[(unsigned char)*s]
    // mozne volat z libovolneho threadu
    function GetConversionTable( parent : HWND; table : PChar; conversion : PChar ) : BOOL; virtual; stdcall; abstract;

    // vraci jmeno kodove stranky pouzivane ve Windows v tomto regionu (cerpa v convert\XXX\convert.cfg
    // v instalaci Salamandera); jde o normalne zobrazitelne kodovani, proto se pouziva v pripade,
    // ze je potreba zobrazit text, ktery byl vytvoren v jine kodove strance (zde se zada jako
    // "cilove" kodovani pri hledani konverzni tabulky, viz metoda GetConversionTable);
    // 'parent' je parent messageboxu (je-li NULL, je parent hlavni okno); 'codePage' je buffer
    // (min. 101 znaku) pro jmeno kodove stranky (pokud neni v souboru convert\XXX\convert.cfg toto jmeno
    // definovane, vraci se v bufferu prazdny retezec)
    // mozne volat z libovolneho threadu
    procedure GetWindowsCodePage( parent : HWND; codePage : PChar ); virtual; stdcall; abstract;

    // zjisti z bufferu 'pattern' o delce 'patternLen' (napr. prvnich 10000 znaku) jestli jde
    // o text (existuje kodova stranka, ve ktere obsahuje jen povolene znaky - zobrazitelne
    // a ridici) a pokud jde o text, zjisti take jeho kodovou stranku (nejpravdepodobnejsi);
    // 'parent' je parent messageboxu (je-li NULL, je parent hlavni okno); je-li 'forceText'
    // TRUE, neprovadi se kontrola na nepovolene znaky (pouziva se, pokud 'pattern' obsahuje
    // text); neni-li 'isText' NULL, vraci se v nem TRUE pokud jde o text; neni-li 'codePage'
    // NULL, jde o buffer (min. 101 znaku) pro jmeno kodove stranky (nejpravdepodobnejsi)
    // mozne volat z libovolneho threadu
    procedure RecognizeFileType( parent : HWND; pattern : PChar; patternLen : Integer;
                                 forceText : BOOL; isText : PBOOL; codePage : PChar ); virtual; stdcall; abstract;

    // zjisti z bufferu 'text' o delce 'textLen' jestli jde o ANSI text (obsahuje (v ANSI
    // znakove sade) jen povolene znaky - zobrazitelne a ridici); rozhoduje bez kontextu
    // (nezalezi na poctu znaku ani jak jdou po sobe - testovany text je mozne rozdelit
    // na libovolne casti a testovat je postupne); vraci TRUE pokud jde o ANSI text (jinak
    // je obsah bufferu 'text' binarni)
    // mozne volat z libovolneho threadu
    function IsANSIText( text : PChar; textLen : Integer ) : BOOL; virtual; stdcall; abstract;

    // zavola funkci 'callback' s parametry 'param' a funkci pro ziskavani oznacenych
    // souboru/adresaru (viz definice typu SalPluginOperationFromDisk) z panelu 'panel'
    // (v panelu musi byt otevrena windowsova cesta); 'panel' je jeden z PANEL_XXX
    // omezeni: hlavni thread
    procedure CallPluginOperationFromDisk( panel : Integer; callback : TSalPluginOperationFromDiskProc;
                                           param : Pointer ); virtual; stdcall; abstract;

    // vraci standardni charset, ktery ma uzivatel nastaveny (soucast regionalniho
    // nastaveni); fonty je nutne konstruovat s timto charsetem, jinak nemusi byt
    // texty citelne (je-li text ve standardni kodove strance, viz Win32 API funkce
    // GetLocaleInfo(LOCALE_USER_DEFAULT, LOCALE_IDEFAULTANSICODEPAGE, ...))
    // mozne volat z libovolneho threadu
    function GetUserDefaultCharset : Byte; virtual; stdcall; abstract;

    // alokuje novy objekt Boyer-Moorova vyhledavaciho algoritmu; pri nedostatku pameti vraci NULL
    // mozne volat z libovolneho threadu
    function AllocSalamanderBMSearchData : TSalamanderBMSearchData; virtual; stdcall; abstract;

    // uvolni objekt Boyer-Moorova vyhledavaciho algoritmu (ziskany metodou AllocSalamanderBMSearchData)
    // mozne volat z libovolneho threadu
    procedure FreeSalamanderBMSearchData( data : TSalamanderBMSearchData ); virtual; stdcall; abstract;

    // alokuje novy objekt algoritmu pro vyhledavani pomoci regularnich vyrazu; pri nedostatku
    // pameti vraci NULL
    // mozne volat z libovolneho threadu
    function AllocSalamanderREGEXPSearchData : TSalamanderREGEXPSearchData; virtual; stdcall; abstract;

    // uvolni objekt algoritmu pro vyhledavani pomoci regularnich vyrazu (ziskany metodou
    // AllocSalamanderREGEXPSearchData)
    // mozne volat z libovolneho threadu
    procedure FreeSalamanderREGEXPSearchData( data : TSalamanderREGEXPSearchData ); virtual; stdcall; abstract;

    // vraci postupne prikazy Salamandera (postupuje v poradi definice konstant SALCMD_XXX);
    // 'index' je vstupne/vystupni promenna, ukazuje na int, ve kterem je pri prvnim volani 0,
    // hodnotu pro dalsi volani si funkce ulozi pri navratu (pouziti: na zacatku vynulovat, pak
    // nemenit); vraci FALSE, pokud jiz neexistuje zadny dalsi prikaz; pokud vraci TRUE, obsahuje
    // 'salCmd' (neni-li NULL) cislo prikazu Salamandera (viz konstanty SALCMD_XXX; cisla maji
    // rezervovany interval 0 az 499, pokud tedy maji v menu byt prikazy Salamandera spolecne s
    // jinymi prikazy, neni problem vytvorit vzajemne se neprekryvajici mnoziny hodnot prikazu
    // napr. posunutim vsech hodnot o zvolene cislo - priklad viz DEMOPLUGin -
    // CPluginFSInterface::ContextMenu), 'nameBuf' (buffer o velikosti 'nameBufSize' bytu)
    // obsahuje jmeno prikazu (jmeno je pripraveno pro pouziti v menu - ma zdvojene ampersandy,
    // podtrzene znaky oznacene ampersandy a za '\t' ma popisy klavesovych zkratek), 'enabled'
    // (neni-li NULL) obsahuje stav prikazu (TRUE/FALSE pokud je enabled/disabled), 'type'
    // (neni-li NULL) obsahuje typ prikazu (viz popis konstant sctyXXX)
    // mozne volat z libovolneho threadu
    function EnumSalamanderCommands( index : PInteger; salCmd : PInteger; nameBuf : PChar;
                                     nameBufSize : Integer; enabled : PBOOL;
                                     _type : PInteger ) : BOOL; virtual; stdcall; abstract;

    // vraci prikaz Salamandera s cislem 'salCmd' (viz konstanty SALCMD_XXX);
    // vraci FALSE, pokud takovy prikaz neexistuje; pokud vraci TRUE, obsahuje
    // 'nameBuf' (buffer o velikosti 'nameBufSize' bytu) jmeno prikazu (jmeno je pripraveno pro
    // pouziti v menu - ma zdvojene ampersandy, podtrzene znaky oznacene ampersandy a za '\t' ma
    // popisy klavesovych zkratek), 'enabled' (neni-li NULL) obsahuje stav prikazu (TRUE/FALSE
    // pokud je enabled/disabled), 'type' (neni-li NULL) obsahuje typ prikazu (viz popis konstant
    // sctyXXX)
    // mozne volat z libovolneho threadu
    function GetSalamanderCommand( salCmd : Integer; nameBuf : PChar; nameBufSize : Integer;
                                   enabled : PBOOL; _type : PInteger ) : BOOL; virtual; stdcall; abstract;

    // nastavi volajicimu pluginu priznak, ze se ma pri nejblizsi mozne prilezitosti spustit
    // prikaz Salamandera s cislem 'salCmd' (jakmile nebudou v message-queue hl. threadu zadne
    // message a Salamander nebude "busy" (nebude otevreny zadny modalni dialog a nebude se
    // zpracovavat zadna zprava));
    // POZOR: pokud se vola z jineho nez hlavniho threadu, muze dojit ke spusteni prikazu Salamandera
    // (probiha v hlavnim threadu) dokonce drive nez skonci PostSalamanderCommand
    // mozne volat z libovolneho threadu
    procedure PostSalamanderCommand( salCmd : Integer ); virtual; stdcall; abstract;

    // nastavi priznak "uzivatel pracoval s aktualni cestou" v panelu 'panel' (tento priznak
    // se vyuziva pri plneni seznamu pracovnich adresaru - List Of Working Directories (Alt+F12));
    // 'panel' je jeden z PANEL_XXX
    // omezeni: hlavni thread
    procedure SetUserWorkedOnPanelPath( panel : Integer ); virtual; stdcall; abstract;

    // v panelu 'panel' (jedna z konstant PANEL_XXX) ulozi vybrana (Selected) jmena
    // do zvlastniho pole, ze ktereho muze uzivatel vyber obnovit prikazem Edit/Restore Selection
    // vyuziva se pro prikazy, ktere zrusi aktualni vyber, aby mel uzivatel moznost
    // se k nemu vratit a provest jeste dalsi operaci
    // omezeni: hlavni thread
    procedure StoreSelectionOnPanelPath( panel : Integer ); virtual; stdcall; abstract;

    //
    // UpdateCrc32
    //   Updates CRC-32 (32-bit Cyclic Redundancy Check) with specified array of bytes.
    //
    // Parameters
    //   'buffer'
    //      [in] Pointer to the starting address of the block of memory to update 'crcVal' with.
    //
    //   'count'
    //      [in] Size, in bytes, of the block of memory to update 'crcVal' with.
    //
    //   'crcVal'
    //      [in] Initial crc value. Set this value to zero to calculate CRC-32 of the 'buffer'.
    //
    // Return Values
    //   Returns updated CRC-32 value.
    //
    // Remarks
    //   Method can be called from any thread.
    //
    function UpdateCrc32( buffer : Pointer; count : DWORD; crcVal : DWORD ) : DWORD; virtual; stdcall; abstract;

    // alokuje novy objekt pro vypocet MD5; pri nedostatku pameti vraci NULL
    // mozne volat z libovolneho threadu
    function AllocSalamanderMD5 : TSalamanderMD5; virtual; stdcall; abstract;

    // uvolni objekt pro vypocet MD5 (ziskany metodou AllocSalamanderMD5)
    // mozne volat z libovolneho threadu
    procedure FreeSalamanderMD5( md5 : TSalamanderMD5 ); virtual; stdcall; abstract;

    // V textu nalezne pary '<' '>', vyradi je z bufferu a prida odkazy na
    // jejich obsah do 'varPlacements'. 'varPlacements' je pole DWORDu o '*varPlacementsCount'
    // polozkach, DWORDy jsou slozene vzdy z pozice odkazu ve vystupnim bufferu (spodni WORD)
    // a poctu znaku odkazu (horni WORD). Retezce "\<", "\>", "\\" jsou chapany
    // jako escape sekvence a budou nahrazeny znaky '<', '>' a '\\'.
    // Vraci TRUE v pripade uspechu, jinak FALSE; vzdy nastavi 'varPlacementsCount' na
    // pocet zpracovanych promennych.
    // mozne volat z libovolneho threadu
    function LookForSubTexts( text : PChar; varPlacements : PDWORD; varPlacementsCount : PInteger ) : BOOL; virtual; stdcall; abstract;

    // cekani (maximalne 0.2 sekundy) na pusteni klavesy ESC; pouziva se pokud plugin obsahuje
    // akce, ktere se prerusuji klavesou ESC (monitorovani klavesy ESC pres
    // GetAsyncKeyState(VK_ESCAPE)) - zabranuje tomu, aby se po stisknuti ESC v dialogu/messageboxu
    // hned prerusila i nasledujici akce monitorujici klavesu ESC
    // mozne volat z libovolneho threadu
    procedure WaitForESCRelease; virtual; stdcall; abstract;

    //
    // GetMouseWheelScrollLines
    //   An OS independent method to retrieve the mouse wheel message.
    //
    // Return Values
    //   If the message is successfully retrieved, the return value is a message identifier
    //   of message that is sent to the focus window when the mouse wheel is rotated.
    //
    //   If the function fails, the return value is zero.
    //
    // Remarks
    //   Method can be called from any thread.
    //
    function GetMouseWheelMsg : DWORD; virtual; stdcall; abstract;

    //
    // GetMouseWheelScrollLines
    //   An OS independent method to retrieve the number of wheel scroll lines.
    //
    // Return Values
    //   Number of scroll lines where WHEEL_PAGESCROLL (0xffffffff) indicates to scroll a page at a time.
    //
    // Remarks
    //   Method can be called from any thread.
    //
    function GetMouseWheelScrollLines : DWORD; virtual; stdcall; abstract;

    //
    // GetTopVisibleParent
    //   Retrieves the visible root window by walking the chain of parent windows
    //   returned by GetParent.
    //
    // Parameters
    //   'hParent'
    //      [in] Handle to the window whose parent window handle is to be retrieved.
    //
    // Return Values
    //   The return value is the handle to the top Popup or Overlapped visible parent window.
    //
    // Remarks
    //   Method can be called from any thread.
    //
    function GetTopVisibleParent( hParent : HWND ) : HWND; virtual; stdcall; abstract;

    //
    // MultiMonGetDefaultWindowPos
    //   Retrieves the default position of the upper-left corner for a newly created window
    //   on the display monitor that has the largest area of intersection with the bounding
    //   rectangle of a specified window.
    //
    // Parameters
    //   'hByWnd'
    //      [in] Handle to the window of interest.
    //
    //   'p'
    //      [out] Pointer to a POINT structure that receives the virtual-screen coordinates
    //      of the upper-left corner for the window that would be created with CreateWindow
    //      with CW_USEDEFAULT in the 'x' parameter. Note that if the monitor is not the
    //      primary display monitor, some of the point's coordinates may be negative values.
    //
    // Return Values
    //   If the default window position lies on the primary monitor or some error occured,
    //   the return value is FALSE and you should use CreateWindow with CW_USEDEFAULT in
    //   the 'x' parameter.
    //
    //   Otherwise the return value is TRUE and coordinates from 'p' structure should be used
    //   in the CreateWindow 'x' and 'y' parameters.
    //
    // Remarks
    //   Method can be called from any thread.
    //
    function MultiMonGetDefaultWindowPos( hByWnd : HWND; p : PPoint ) : BOOL; virtual; stdcall; abstract;

    //
    // MultiMonGetClipRectByRect
    //   Retrieves the bounding rectangle of the display monitor that has the largest
    //   area of intersection with a specified rectangle.
    //
    // Parameters
    //   'rect'
    //      [in] Pointer to a RECT structure that specifies the rectangle of interest
    //      in virtual-screen coordinates.
    //
    //   'workClipRect'
    //      [out] A RECT structure that specifies the work area rectangle of the
    //      display monitor, expressed in virtual-screen coordinates. Note that
    //      if the monitor is not the primary display monitor, some of the rectangle's
    //      coordinates may be negative values.
    //
    //   'monitorClipRect'
    //      [out] A RECT structure that specifies the display monitor rectangle,
    //      expressed in virtual-screen coordinates. Note that if the monitor is
    //      not the primary display monitor, some of the rectangle's coordinates
    //      may be negative values. This parameter can be NULL.
    //
    // Remarks
    //   Method can be called from any thread.
    //
    procedure MultiMonGetClipRectByRect( rect : PRect; workClipRect : PRect; monitorClipRect : PRect ); virtual; stdcall; abstract;

    //
    // MultiMonGetClipRectByWindow
    //   Retrieves the bounding rectangle of the display monitor that has the largest
    //   area of intersection with the bounding rectangle of a specified window.
    //
    // Parameters
    //   'hByWnd'
    //      [in] Handle to the window of the interest. If this parameter is NULL,
    //      or window is not visible or is iconic, monitor with the currently active window
    //      from the same application will be used. Otherwise primary monitor will be used.
    //
    //   'workClipRect'
    //      [out] A RECT structure that specifies the work area rectangle of the
    //      display monitor, expressed in virtual-screen coordinates. Note that
    //      if the monitor is not the primary display monitor, some of the rectangle's
    //      coordinates may be negative values.
    //
    //   'monitorClipRect'
    //      [out] A RECT structure that specifies the display monitor rectangle,
    //      expressed in virtual-screen coordinates. Note that if the monitor is
    //      not the primary display monitor, some of the rectangle's coordinates
    //      may be negative values. This parameter can be NULL.
    //
    // Remarks
    //   Method can be called from any thread.
    //
    procedure MultiMonGetClipRectByWindow( hByWnd : HWND; workClipRect : PRect; monitorClipRect : PRect ); virtual; stdcall; abstract;

    //
    // MultiMonCenterWindow
    //   Centers the window against a specified window or monitor.
    //
    // Parameters
    //   'hWindow'
    //      [in] Handle to the window whose parent window handle is to be retrieved.
    //
    //   'hByWnd'
    //      [in] Handle to the window against which to center. If this parameter is NULL,
    //      or window is not visible or is iconic, the method will center 'hWindow' against
    //      the working area of the monitor. Monitor with the currently active window
    //      from the same application will be used. Otherwise primary monitor will be used.
    //
    //   'findTopWindow'
    //      [in] If this parameter is TRUE, non child visible window will be used by walking
    //      the chain of parent windows of 'hByWnd' as the window against which to center.
    //
    //      If this parameter is FALSE, 'hByWnd' will be to the window against which to center.
    //
    // Remarks
    //   If centered window gets over working area of the monitor, the method positions
    //   the window to be whole visible.
    //
    //   Method can be called from any thread.
    //
    procedure MultiMonCenterWindow( hWindow : HWND; hByWnd : HWND; findTopWindow : BOOL ); virtual; stdcall; abstract;

    //
    // MultiMonEnsureRectVisible
    //   Ensures that specified rectangle is either entirely or partially visible,
    //   adjusting the coordinates if necessary. All monitors are considered.
    //
    // Parameters
    //   'rect'
    //      [in/out] Pointer to the RECT structure that contain the coordinated to be
    //      adjusted. The rectangle is presumed to be in virtual-screen coordinates.
    //
    //   'partialOK'
    //      [in] Value specifying whether the rectangle must be entirely visible.
    //      If this parameter is TRUE, no moving occurs if the item is at least
    //      partially visible.
    //
    // Return Values
    //   If the rectangle is adjusted, the return value is TRUE.
    //
    //   If the rectangle is not adjusted, the return value is FALSE.
    //
    // Remarks
    //   Method can be called from any thread.
    //
    function MultiMonEnsureRectVisible( rect : PRect; partialOK : BOOL ) : BOOL; virtual; stdcall; abstract;

    //
    // InstallWordBreakProc
    //   Installs special word break procedure to the specified window. This procedure
    //   is inteded for easier cursor movevement in the single line edit controls.
    //   Delimiters '\\', '/', ' ', ';', ',', and '.' are used as cursor stops when user
    //   navigates using Ctrl+Left or Ctrl+Right keys.
    //   You can use Ctrl+Backspace to delete one word.
    //
    // Parameters
    //   'hWindow'
    //      [in] Handle to the window or control where word break proc is to be isntalled.
    //      Window may be either edit or combo box with edit control.
    //
    // Return Values
    //   The return value is TRUE if the word break proc is installed. It is FALSE if the
    //   window is neither edit nor combo box with edit control, some error occured, or
    //   this special word break proc is not supported on your OS.
    //
    // Remarks
    //   You needn't uninstall word break procedure before window is destroyed.
    //
    //   Method can be called from any thread.
    //
    function InstallWordBreakProc( hWindow : HWND ) : BOOL; virtual; stdcall; abstract;

    // vraci TRUE, pokud byla tato instance Altap Salamandera prvni spustena (v dobe
    // startu instance se hledaji dalsi bezici instance verze 1.6 beta 2 nebo novejsi)
    // mozne volat z libovolneho threadu
    function IsFirstInstance16beta2OrLater : BOOL; virtual; stdcall; abstract;

    // podpora pro tvorbu parametrizovanych textu (reseni jednotnych a mnoznych cisel
    // v textech); 'format' je formatovaci retezec pro vysledny text - popis jeho formatu
    // nasleduje; vysledny text se vraci v bufferu 'buffer' o velikosti 'bufferSize' bytu;
    // 'parametersArray' je pole parametru textu, 'parametersCount' je pocet techto parametru;
    // vraci delku vysledneho textu
    //
    // popis formatovaciho retezce:
    //   - na zacatku kazdeho formatovaciho retezce je signatura "{!}"
    //   - uznavaji se nasledujici escape sekvence pro potlaceni vyznamu specialnich
    //     znaku ve formatovacim retezci (znak backslashe v tomto popisu
    //     neni zdvojen): "\\" = "\", "\{" = "{", "\}" = "}" a "\|" = "|"
    //     (pozor na zdvojovani backslashe pri zapisu do C++ retezce: "\\{" = "{"  /*jen kvuli MSVC editoru:}}*/)
    //   - text, ktery neni ve slozenych zavorkach se do vysledneho retezce
    //     prenasi beze zmen (az na escape sekvence)
    //   - parametrizovany text je ve slozenych zavorkach
    //   - postupne kazdemu parametrizovanemu textu odpovida jeden
    //     parametr z 'parametersArray' - jde o 64-bitovy unsigned int
    //   - parametrizovany text obsahuje ruzne vysledne texty pro ruzne intervaly
    //     hodnot parametru
    //   - jednotlive vysledne texty a meze intervalu se oddeluji znakem "|"
    //   - parametrizovany text "{}" se pouziva pro preskoceni jednoho parametru
    //     z pole 'parametersArray' (nevytvari zadny vystupni text)
    //
    // priklady formatovacich retezcu:
    //   - "{!}director{y|1|ies}" pro hodnotu parametru od 0 do 1 (vcetne) bude
    //     "directory" a pro hodnotu od 2 do "nekonecna" (2^64-1) bude "directories"
    //   - "{!}soubo{r|0|r|1|ry|4|r}" pro hodnotu parametru 0 bude "soubor",
    //     pro 1 bude "soubor", pro 2 az 4 (vcetne) bude "soubory" a od 5
    //     do "nekonecna" bude "soubor"
    //
    // mozne volat z libovolneho threadu
    function ExpandPluralString( buffer : PChar; bufferSize : Integer; format : PChar;
                                 parametersCount : Integer; parametersArray : PLargeInteger ) : Integer; virtual; stdcall; abstract;

    // v aktualni jazykove verzi Salamandera pripravi retezec "XXX (selected/hidden)
    // files and YYY (selected/hidden) directories"; je-li XXX (hodnota parametru 'files')
    // nebo YYY (hodnota parametru 'dirs') nula, prislusna cast retezce se vypousti (oba
    // parametry zaroven nulove se neuvazuji); pouziti "selected" a "hidden" zavisi
    // na rezimu 'mode' - viz popis konstant epfdmXXX; vysledny text
    // se vraci v bufferu 'buffer' o velikosti 'bufferSize' bytu; vraci delku vysledneho
    // textu; 'forDlgCaption' je TRUE/FALSE pokud je/neni text urceny pro titulek dialogu
    // (v anglictine nutna velka pocatecni pismena)
    // mozne volat z libovolneho threadu
    function ExpandPluralFilesDirs( buffer : PChar; bufferSize : Integer; files : Integer;
                                    dirs : Integer; mode : Integer; forDlgCaption : BOOL ) : Integer; virtual; stdcall; abstract;

    // v aktualni jazykove verzi Salamandera pripravi retezec "BBB bytes in XXX selected
    // files and YYY selected directories"; BBB je hodnota parametru 'selectedBytes';
    // je-li XXX (hodnota parametru 'files') nebo YYY (hodnota parametru 'dirs') nula,
    // prislusna cast retezce se vypousti (oba parametry zaroven nulove se neuvazuji);
    // je-li 'useSubTexts' TRUE, uzavorkuje se BBB do '<' a '>', aby se s BBB dalo
    // dale pracovat na info-line (viz metoda CSalamanderGeneralAbstract::LookForSubTexts a
    // CPluginDataInterfaceAbstract::GetInfoLineContent); vysledny text se vraci v bufferu
    // 'buffer' o velikosti 'bufferSize' bytu; vraci delku vysledneho textu
    // mozne volat z libovolneho threadu
    function ExpandPluralBytesFilesDirs( buffer : PChar; bufferSize : Integer;
                                         var selectedBytes : TLargeInteger; files : Integer;
                                         dirs : Integer; useSubTexts : BOOL ) : Integer; virtual; stdcall; abstract;


    // vraci retezec popisujici s cim se pracuje (napr. "file "test.txt"" nebo "directory "test""
    // nebo "3 files and 1 directory"); 'sourceDescr' je buffer pro vysledek o velikosti
    // minimalne 'sourceDescrSize'; 'panel' popisuje zdrojovy panel operace (jedna z PANEL_XXX nebo -1
    // pokud operace nema zdrojovy panel (napr. CPluginFSInterfaceAbstract::CopyOrMoveFromDiskToFS));
    // 'selectedFiles'+'selectedDirs' - pokud ma operace zdrojovy panel, je zde pocet oznacenych
    // souboru a adresaru ve zdrojovem panelu, pokud jsou obe hodnoty nulove, pracuje se se
    // souborem/adresarem pod kurzorem (fokusem); 'selectedFiles'+'selectedDirs' - pokud operace nema
    // zdrojovy panel, je zde pocet souboru/adresaru, se kterymi operace pracuje;
    // 'fileOrDirName'+'isDir' - pouziva se jen pokud operace nema zdrojovy panel a pokud
    // 'selectedFiles + selectedDirs == 1', je zde jmeno souboru/adresare a jestli jde o soubor
    // nebo adresar ('isDir' je FALSE nebo TRUE); 'forDlgCaption' je TRUE/FALSE pokud je/neni
    // text urceny pro titulek dialogu (v anglictine nutna velka pocatecni pismena)
    // omezeni: hlavni thread (muze pracovat s panelem)
    procedure GetCommonFSOperSourceDescr( sourceDescr : PChar; sourceDescrSize : Integer;
                                          panel : Integer; selectedFiles : Integer;
                                          selectedDirs : Integer; fileOrDirName : PChar;
                                          isDir : BOOL; forDlgCaption : BOOL ); virtual; stdcall; abstract;

    // nakopiruje retezec 'srcStr' za retezec 'dstStr' (za jeho koncovou nulu);
    // 'dstStr' je buffer o velikosti 'dstBufSize' (musi byt nejmene rovno 2);
    // pokud se do bufferu oba retezce nevejdou, jsou zkraceny (vzdy tak, aby se
    // veslo co nejvice znaku z obou retezu)
    // mozne volat z libovolneho threadu
    procedure AddStrToStr( dstStr : PChar; dstBufSize : Integer; srcStr : PChar ); virtual; stdcall; abstract;

    // zjisti jestli je mozne retezec 'fileNameComponent' pouzit jako komponentu
    // jmena na Windows filesystemu (resi retezce delsi nez MAX_PATH-4 (4 = "C:\"
    // + null-terminator), prazdny retezec, retezce znaku '.', retezce white-spaces,
    // znaky "*?\\/<>|\":" a jednoducha jmena typu "prn" a "prn  .txt")
    // mozne volat z libovolneho threadu
    function SalIsValidFileNameComponent( fileNameComponent : PChar ) : BOOL; virtual; stdcall; abstract;

    // pretvori retezec 'fileNameComponent' tak, aby mohl byt pouzity jako komponenta
    // jmena na Windows filesystemu (resi retezce delsi nez MAX_PATH-4 (4 = "C:\"
    // + null-terminator), resi prazdny retezec, retezce znaku '.', retezce
    // white-spaces, znaky "*?\\/<>|\":" nahradi '_' + jednoducha jmena typu "prn"
    // a "prn  .txt" doplni o '_' na konci nazvu); 'fileNameComponent' musi jit
    // rozsirit alespon o jeden znak (maximalne se vsak z 'fileNameComponent'
    // pouzije MAX_PATH bytu)
    // mozne volat z libovolneho threadu
    procedure SalMakeValidFileNameComponent( fileNameComponent : PChar ); virtual; stdcall; abstract;

    // vraci TRUE pokud je zdroj enumerace panel, v 'panel' pak vraci PANEL_LEFT nebo
    // PANEL_RIGHT; pokud nebyl zdroj enumerace nalezen nebo jde o Find okno, vraci FALSE;
    // 'srcUID' je unikatni identifikator zdroje (predava se jako parametr pri otevirani
    // vieweru)
    function IsFileEnumSourcePanel( srcUID : Integer; panel : PInteger ) : BOOL; virtual; stdcall; abstract;

    // vraci dalsi jmeno souboru pro viewer ze zdroje (levy/pravy panel nebo Findy);
    // 'srcUID' je unikatni identifikator zdroje (predava se jako parametr pri otevirani
    // vieweru); 'lastFileIndex' (nesmi byt NULL) je IN/OUT parametr, ktery by plugin mel
    // menit jen pokud chce vratit jmeno prvniho souboru, v tomto pripade nastavit 'lastFileIndex'
    // na -1; pocatecni hodnota 'lastFileIndex' se predava jako parametr pri otevirani
    // vieweru; 'lastFileName' je plne jmeno aktualniho souboru (prazdny retezec pokud neni
    // zname, napr. je-li 'lastFileIndex' -1); je-li 'preferSelected' TRUE a aspon jedno
    // jmeno oznacene, budou se vracet oznacena jmena; je-li 'onlyAssociatedExtensions'
    // TRUE, vraci jen soubory s priponou asociovanou s viewerem tohoto pluginu (F3 na tomto
    // souboru by se pokusilo otevrit viewer tohoto pluginu + ignoruje pripadne zastineni
    // viewerem jineho pluginu); 'fileName' je buffer pro ziskane jmeno (velikost alespon
    // MAX_PATH); vraci TRUE pokud se podari jmeno ziskat; vraci FALSE pri chybe: zadne
    // dalsi jmeno souboru ve zdroji neni (neni-li 'noMoreFiles' NULL, vraci se v nem TRUE),
    // zdroj je zaneprazdnen (nezpracovava zpravy; neni-li 'srcBusy' NULL, vraci se v nem
    // TRUE), jinak zdroj prestal existovat (zmena cesty v panelu, atp.)
    // mozne volat z libovolneho threadu
    function GetNextFileNameForViewer( srcUID : Integer; lastFileIndex : PInteger;
                                       lastFileName : PChar; preferSelected : BOOL;
                                       onlyAssociatedExtensions : BOOL; fileName : PChar;
                                       noMoreFiles : PBOOL; srcBusy : PBOOL ) : BOOL; virtual; stdcall; abstract;

    // vraci predchozi jmeno souboru pro viewer ze zdroje (levy/pravy panel nebo Findy);
    // 'srcUID' je unikatni identifikator zdroje (predava se jako parametr pri otevirani
    // vieweru); 'lastFileIndex' (nesmi byt NULL) je IN/OUT parametr, ktery by plugin mel
    // menit jen pokud chce vratit jmeno posledniho souboru, v tomto pripade nastavit 'lastFileIndex'
    // na -1; pocatecni hodnota 'lastFileIndex' se predava jako parametr pri otevirani
    // vieweru; 'lastFileName' je plne jmeno aktualniho souboru (prazdny retezec pokud neni
    // zname, napr. je-li 'lastFileIndex' -1); je-li 'preferSelected' TRUE a aspon jedno
    // jmeno oznacene, budou se vracet oznacena jmena; je-li 'onlyAssociatedExtensions' TRUE,
    // vraci jen soubory s priponou asociovanou s viewerem tohoto pluginu (F3 na tomto
    // souboru by se pokusilo otevrit viewer tohoto pluginu + ignoruje pripadne zastineni
    // viewerem jineho pluginu); 'fileName' je buffer pro ziskane jmeno (velikost alespon
    // MAX_PATH); vraci TRUE pokud se podari jmeno ziskat; vraci FALSE pri chybe: zadne
    // predchozi jmeno souboru ve zdroji neni (neni-li 'noMoreFiles' NULL, vraci se v nem
    // TRUE), zdroj je zaneprazdnen (nezpracovava zpravy; neni-li 'srcBusy' NULL, vraci
    // se v nem TRUE), jinak zdroj prestal existovat (zmena cesty v panelu, atp.)
    // mozne volat z libovolneho threadu
    function GetPreviousFileNameForViewer( srcUID : Integer; lastFileIndex : PInteger;
                                           lastFileName : PChar; preferSelected : BOOL;
                                           onlyAssociatedExtensions : BOOL; fileName : PChar;
                                           noMoreFiles : PBOOL; srcBusy : PBOOL ) : BOOL; virtual; stdcall; abstract;

    // zjisti, jestli je aktualni soubor z vieweru oznaceny (selected) ve zdroji (levy/pravy
    // panel nebo Findy); 'srcUID' je unikatni identifikator zdroje (predava se jako parametr
    // pri otevirani vieweru); 'lastFileIndex' je parametr, ktery by plugin nemel menit,
    // pocatecni hodnota 'lastFileIndex' se predava jako parametr pri otevirani vieweru;
    // 'lastFileName' je plne jmeno aktualniho souboru; vraci TRUE pokud se podarilo zjistit,
    // jestli je aktialni soubor oznaceny, vysledek je v 'isFileSelected' (nesmi byt NULL);
    // vraci FALSE pri chybe: zdroj prestal existovat (zmena cesty v panelu, atp.) nebo soubor
    // 'lastFileName' uz ve zdroji neni (pri techto dvou chybach, neni-li 'srcBusy' NULL,
    // vraci se v nem FALSE), zdroj je zaneprazdnen (nezpracovava zpravy; pri teto chybe,
    // neni-li 'srcBusy' NULL, vraci se v nem TRUE)
    // mozne volat z libovolneho threadu
    function IsFileNameForViewerSelected( srcUID : Integer; lastFileIndex : Integer;
                                          lastFileName : PChar; isFileSelected : PBOOL;
                                          srcBusy : PBOOL ) : BOOL; virtual; stdcall; abstract;

    // nastavi oznaceni (selectionu) na aktualnim souboru z vieweru ve zdroji (levy/pravy
    // panel nebo Findy); 'srcUID' je unikatni identifikator zdroje (predava se jako parametr
    // pri otevirani vieweru); 'lastFileIndex' je parametr, ktery by plugin nemel menit,
    // pocatecni hodnota 'lastFileIndex' se predava jako parametr pri otevirani vieweru;
    // 'lastFileName' je plne jmeno aktualniho souboru; 'select' je TRUE/FALSE pokud se ma
    // aktualni soubor oznacit/odznacit; vraci TRUE pri uspechu; vraci FALSE pri chybe:
    // zdroj prestal existovat (zmena cesty v panelu, atp.) nebo soubor 'lastFileName' uz
    // ve zdroji neni (pri techto dvou chybach, neni-li 'srcBusy' NULL, vraci se v nem FALSE),
    // zdroj je zaneprazdnen (nezpracovava zpravy; pri teto chybe, neni-li 'srcBusy' NULL,
    // vraci se v nem TRUE)
    // mozne volat z libovolneho threadu
    function SetSelectionOnFileNameForViewer( srcUID : Integer; lastFileIndex : Integer;
                                              lastFileName : PChar; select : BOOL;
                                              srcBusy : PBOOL ) : BOOL; virtual; stdcall; abstract;

    // vraci odkaz na sdilenou historii (posledne pouzite hodnoty) zvoleneho comboboxu;
    // jde o pole alokovanych retezcu; pole ma pevny pocet retezcu, ten se vraci
    // v 'historyItemsCount' (nesmi byt NULL); odkaz na pole se vraci v 'historyArr'
    // (nesmi byt NULL); 'historyID' (jedna v SALHIST_XXX) urcuje, na kterou sdilenou historii se ma vracet
    // odkaz
    // omezeni: hlavni thread (sdilene historie se v jinem threadu nedaji pouzivat, pristup
    // do nich neni nijak synchronizovany)
    function GetStdHistoryValues( historyID : Integer; historyArr : PChar; historyItemsCount : PInteger ) : BOOL; virtual; stdcall; abstract;

    // prida do sdilene historie ('historyArr'+'historyItemsCount') naalokovanou kopii
    // nove hodnoty 'value'; je-li 'caseSensitiveValue' TRUE, hleda se hodnota (retezec)
    // v poli historie pomoci case-sensitive porovnani (FALSE = case-insensitive porovnani),
    // nalezena hodnota se pouze presouva na prvni misto v poli historie
    // omezeni: hlavni thread (sdilene historie se v jinem threadu nedaji pouzivat, pristup
    // do nich neni nijak synchronizovany)
    // POZNAMKA: pokud se pouziva pro jine nez sdilene historie, je mozne volat v libovolnem threadu
    procedure AddValueToStdHistoryValues( historyArr : PChar; historyItemsCount : Integer;
                                          value : PChar; caseSensitiveValue : BOOL ); virtual; stdcall; abstract;

    // prida do comboboxu ('combo') texty ze sdilene historie ('historyArr'+'historyItemsCount');
    // pred pridavanim provede reset obsahu comboboxu (viz CB_RESETCONTENT)
    // omezeni: hlavni thread (sdilene historie se v jinem threadu nedaji pouzivat, pristup
    // do nich neni nijak synchronizovany)
    // POZNAMKA: pokud se pouziva pro jine nez sdilene historie, je mozne volat v libovolnem threadu
    procedure LoadComboFromStdHistoryValues( combo : HWND; historyArr : PChar; historyItemsCount : Integer ); virtual; stdcall; abstract;

    // zjisti barevnou hloubku aktualniho zobrazeni a je-li vice nez 8-bit (256 barev), vraci TRUE
    // mozne volat z libovolneho threadu
    function CanUse256ColorsBitmap : BOOL; virtual; stdcall; abstract;

    // zkontroluje jestli je enablovany-root-parent okna 'parent' foreground window, pokud ne,
    // udela se FlashWindow(root-parent okna 'parent', TRUE) a vrati root-parent okna 'parent',
    // jinak se vraci NULL
    // POUZITI:
    //    HWND mainWnd = GetWndToFlash(parent);
    //    CDlg(parent).Execute();
    //    if (mainWnd != NULL) FlashWindow(mainWnd, FALSE);  // pod Win9x se musi flashovani odstranit rucne
    // mozne volat z libovolneho threadu
    function GetWndToFlash( parent : HWND ) : HWND; virtual; stdcall; abstract;

    // provede reaktivaci drop-targetu (po dropnuti pri drag&dropu) po otevreni naseho progress-
    // -okna (to se pri otevreni aktivuje, cimz deaktivuje drop-target); neni-li 'dropTarget'
    // NULL a zaroven nejde o panel v tomto Salamanderovi, provede aktivaci 'progressWnd' a nasledne
    // aktivaci nejvzdalenejsiho enablovaneho predka 'dropTarget' (tato kombinace nas zbavi aktivovaneho
    // stavu bez aktivni aplikace, ktery jinak obcas vznika)
    // mozne volat z libovolneho threadu
    procedure ActivateDropTarget( dropTarget : HWND; progressWnd : HWND ); virtual; stdcall; abstract;

    // naplanuje otevreni Pack dialogu s vybranym packerem z tohoho pluginu (viz
    // CSalamanderConnectAbstract::AddCustomPacker), pokud packer z tohoto pluginu
    // neexistuje (napr. protoze ho uzivatel smazal), vypise se userovi chybova
    // hlaska; dialog se otevre jakmile nebudou v message-queue hl. threadu zadne
    // zpravy a Salamander nebude "busy" (nebude otevreny zadny modalni dialog
    // a nebude se zpracovavat zadna zprava); opakovane volani teto metody pred
    // otevrenim Pack dialogu vede jen ke zmene parametru 'delFilesAfterPacking';
    // 'delFilesAfterPacking' ovlivnuje checkbox "Delete files after packing"
    // v Pack dialogu: 0=default, 1=zapnuty, 2=vypnuty
    // omezeni: hlavni thread
    procedure PostOpenPackDlgForThisPlugin( delFilesAfterPacking : Integer ); virtual; stdcall; abstract;

    // naplanuje otevreni Unpack dialogu s vybranym unpackerem z tohoho pluginu (viz
    // CSalamanderConnectAbstract::AddCustomUnpacker), pokud unpacker z tohoto pluginu
    // neexistuje (napr. protoze ho uzivatel smazal), vypise se userovi chybova
    // hlaska; dialog se otevre jakmile nebudou v message-queue hl. threadu zadne
    // zpravy a Salamander nebude "busy" (nebude otevreny zadny modalni dialog
    // a nebude se zpracovavat zadna zprava); opakovane volani teto metody pred
    // otevrenim Unpack dialogu vede jen ke zmene parametru 'unpackMask';
    // 'unpackMask' ovlivnuje masku "Unpack files": NULL=default, jinak text masky
    // omezeni: hlavni thread
    procedure PostOpenUnpackDlgForThisPlugin( unpackMask : PChar ); virtual; stdcall; abstract;

    // vytvoreni souboru se jmenem 'fileName' pres klasicke volani Win32 API
    // CreateFile (lpSecurityAttributes==NULL, dwCreationDisposition==CREATE_NEW,
    // hTemplateFile==NULL); tato metoda resi kolizi 'fileName' s dosovym nazvem
    // jiz existujiciho souboru/adresare (jen pokud nejde i o kolizi s dlouhym
    // jmenem souboru/adresare) - zajisti zmenu dosoveho jmena tak, aby se soubor se
    // jmenem 'fileName' mohl vytvorit (zpusob: docasne prejmenuje konfliktni
    // soubor/adresar na jine jmeno a po vytvoreni 'fileName' ho prejmenuje zpet);
    // vraci handle souboru nebo pri chybe INVALID_HANDLE_VALUE (vraci v 'err'
    // (neni-li NULL) kod Windows chyby)
    // mozne volat z libovolneho threadu
    function SalCreateFileEx( fileName : PChar; desiredAccess : DWORD; shareMode : DWORD;
                              flagsAndAttributes : DWORD; err : PDWORD ) : THandle; virtual; stdcall; abstract;

    // vytvoreni adresare se jmenem 'name' pres klasicke volani Win32 API
    // CreateDirectory(lpSecurityAttributes==NULL); tato metoda resi kolizi 'name'
    // s dosovym nazvem jiz existujiciho souboru/adresare (jen pokud nejde i o
    // kolizi s dlouhym jmenem souboru/adresare) - zajisti zmenu dosoveho jmena
    // tak, aby se adresar se jmenem 'name' mohl vytvorit (zpusob: docasne prejmenuje
    // konfliktni soubor/adresar na jine jmeno a po vytvoreni 'name' ho
    // prejmenuje zpet); vraci TRUE pri uspechu, pri chybe FALSE (vraci v 'err'
    // (neni-li NULL) kod Windows chyby)
    // mozne volat z libovolneho threadu
    function SalCreateDirectoryEx( name : PChar; err : PDWORD ) : BOOL; virtual; stdcall; abstract;

    // umozni odpojit/pripojit sledovani zmen (jen pro windows cesty a cesty do archivu)
    // na cestach prohlizenych v jednom z panelu; ucel: pokud vasemu kodu (formatovani
    // disku, shredovani disku, atp.) prekazi to, ze panel ma pro cestu otevreny handle
    // "ChangeNotification", touto metodou jej muzete docasne odpojit (po pripojeni se
    // vyvolava refresh pro cestu v panelu); 'panel' je jeden z PANEL_XXX; 'stopMonitoring'
    // je TRUE/FALSE (odpojeni/pripojeni)
    // omezeni: hlavni thread
    procedure PanelStopMonitoring( panel : Integer; stopMonitoring : BOOL ); virtual; stdcall; abstract;

    // alokuje novy objekt CSalamanderDirectory pro praci se soubory/adresari v archivu nebo
    // file-systemu; je-li 'isForFS' TRUE, je objekt prednastaven pro pouziti pro file-system,
    // jinak je objekt prednastaven pro pouziti pro archiv (defaultni flagy objektu se
    // lisi pro archiv a file-system, viz metoda CSalamanderDirectoryAbstract::SetFlags);
    // pri nedostatku pameti vraci NULL
    // mozne volat z libovolneho threadu
    function AllocSalamanderDirectory( isForFS : BOOL ) : TSalamanderDirectoryAbstract; virtual; stdcall; abstract;

    // uvolni objekt CSalamanderDirectory (ziskany metodou AllocSalamanderDirectory,
    // POZOR: nesmi se volat pro zadny jiny ukazatel CSalamanderDirectoryAbstract)
    // mozne volat z libovolneho threadu
    procedure FreeSalamanderDirectory( salDir : TSalamanderDirectoryAbstract ); virtual; stdcall; abstract;

    // podpora pro testovani registracnich klicu v pripade, ze neni mozne prilinkovat RSAKEY.LIB
    // (napr. kompilace pluginu v C++ Builder); popis viz shared\rsakey.h
    function RSAKeyValid( keyContent : PChar; keyContentSize : PDWORD ) : BOOL; virtual; stdcall; abstract;

    // prida novy timer pro objekt pluginoveho FS; az dojde k timeoutu timeru, zavola se metoda
    // CPluginFSInterfaceAbstract::Event() objektu pluginoveho FS 'timerOwner' s parametry
    // FSE_TIMER a 'timerParam'; 'timeout' je timeout timeru od jeho pridani (v milisekundach,
    // musi byt >= 0); timer se zrusi v okamziku sveho timeoutu (pred volanim
    // CPluginFSInterfaceAbstract::Event()) nebo pri zavreni objektu pluginoveho FS;
    // vraci TRUE, pokud byl timer uspesne pridan
    // omezeni: hlavni thread
    function AddPluginFSTimer( timeout : Integer; timerOwner : TPluginFSInterfaceAbstract;
                               timerParam : DWORD ) : BOOL; virtual; stdcall; abstract;

    // zrusi bud vsechny timery objektu pluginoveho FS 'timerOwner' (je-li 'allTimers' TRUE)
    // nebo jen vsechny timery s parametrem rovnym 'timerParam' (je-li 'allTimers' FALSE);
    // vraci pocet zrusenych timeru
    // omezeni: hlavni thread
    function KillPluginFSTimer( timerOwner : TPluginFSInterfaceAbstract; allTimers : BOOL;
                                timerParam : DWORD ) : Integer; virtual; stdcall; abstract;

    // zjistuje viditelnost polozky pro FS v Change Drive menu; vraci TRUE, pokud je
    // polozka viditelna, jinak vraci FALSE
    // omezeni: hlavni thread (jinak muze dojit ke zmenam v konfiguraci pluginu behem volani)
    function GetChangeDriveMenuItemVisibility : BOOL; virtual; stdcall; abstract;

    // nastavuje viditelnost polozky pro FS v Change Drive menu; pouzivat jen pri
    // instalaci pluginu (jinak hrozi prenastaveni uzivatelem zvolene viditelnosti);
    // 'visible' je TRUE v pripade, ze polozka ma byt viditelna
    // omezeni: hlavni thread (jinak muze dojit ke zmenam v konfiguraci pluginu behem volani)
    procedure SetChangeDriveMenuItemVisibility( visible : BOOL ); virtual; stdcall; abstract;

    // Nastavuje breakpoint na x-tou COM/OLE alokaci. Slouzi k dohledani COM/OLE leaku.
    // V release verzi Salamandera nedela nic. Debug verze Salamandera pri svem ukonceni
    // zobrazuje do Debug okna debuggeru a do Trace Serveru seznam COM/OLE leaku.
    // V hranatych zavorkach je poradi alokace, kterou predame jako 'alloc' do volani
    // OleSpySetBreak. Lze zavolat z libovolneho threadu.
    procedure OleSpySetBreak( alloc : Integer ); virtual; stdcall; abstract;

    // Vraci kopie ikon, ktere Salamander pouziva v panelech. 'icon' urcuje ikonu a jde
    // o jednu z hodnot SALICON_xxx. 'iconSize' urcuje jakou ma mit vracena ikona velikost
    // a jde o jednu z hodnot SALICONSIZE_xxx.
    // V pripade uspechu vraci handle vytvorene ikony. Destrukci ikony musi zajistit
    // plugin pomoci volani API DestroyIcon. V pripade neuspechu vraci NULL.
    // omezeni: hlavni thread
    function GetSalamanderIcon( icon : Integer; iconSize : Integer ) : HICON; virtual; stdcall; abstract;

    //*****************************************************************************
    //
    // GetFileIcon
    //   Function retrieves handle to large or small icon from the specified object,
    //   such as a file, a folder, a directory, or a drive root.
    //
    // Parameters
    //   'path'
    //      [in] Pointer to a null-terminated string that contains the path and file
    //      name. If the 'pathIsPIDL' parameter is TRUE, this parameter must be the
    //      address of an ITEMIDLIST (PIDL) structure that contains the list of item
    //      identifiers that uniquely identify the file within the Shell's namespace.
    //      The PIDL must be a fully qualified PIDL. Relative PIDLs are not allowed.
    //
    //   'pathIsPIDL'
    //      [in] Indicate that 'path' is the address of an ITEMIDLIST structure rather
    //      than a path name.
    //
    //   'hIcon'
    //      [out] Pointer to icon handle that receive handle to the icon extracted
    //      from the object.
    //
    //   'iconSize'
    //      [in] Required size of icon. SALICONSIZE_xxx
    //
    // Return Values
    //   Returns TRUE if successful, or FALSE otherwise.
    //
    // Remarks
    //   You are responsible for freeing returned icons with DestroyIcon when you
    //   no longer need them.
    //
    //   You must initialize COM with CoInitialize or OLEInitialize prior to
    //   calling GetFileIcon.
    //
    // lze volat z libovolneho threadu
    function GetFileIcon( path : PChar; pathIsPIDL : BOOL; hIcon : PHICON;
                          iconSize : Integer ) : BOOL; virtual; stdcall; abstract;

    // FileExists
    //   Function checks the existence of a file. It returns TRUE if the specified
    //   file exists. If the file does not exist, it returns 0. FileExists only checks
    //   the existence of files, directories are ignored.
    // lze volat z libovolneho threadu
    function FileExists( fileName : PChar ) : BOOL; virtual; stdcall; abstract;

    // provede zmenu cesty v panelu na posledni znamou diskovou cestu, pokud nni pristupna,
    // tak se provede zmena na prvni fixed-drive; pouziva se pro zavreni file-systemu
    // v panelu (disconnect); 'parent' je parent pripadnych messageboxu; 'panel' je jeden
    // z PANEL_XXX
    // omezeni: hlavni thread + mimo metody CPluginFSInterfaceAbstract a CPluginDataInterfaceAbstract
    // (hrozi napr. zavreni FS otevreneho v panelu - metode by mohl prestat existovat 'this')
    procedure DisconnectFSFromPanel( parent : HWND; panel : Integer ); virtual; stdcall; abstract;

    // vraci TRUE, pokud je nazev souboru 'name' asociovan v Archives Associations in Panels
    // k volajicimu pluginu
    // 'name' musi byt pouze nazev souboru, ne s plnou nebo relativni cestou
    // omezeni: hlavni thread
    function IsArchiveHandledByThisPlugin( name : PChar ) : BOOL; virtual; stdcall; abstract;

    // slouzi jako LR_xxx parametr pro API funkci LoadImage()
    // pokud uzivatel nema zapnute hi-color ikony v konfiguraci desktopu,
    // vraci LR_VGACOLOR, aby nedoslo k chybnemu nacteni vice barevne verze ikony
    // jinak vraci 0 (LR_DEFAULTCOLOR); vysledek funkce lze orovat s dalsimi LR_xxx flagy
    // lze volat z libovolneho threadu
    function GetIconLRFlags : DWORD; virtual; stdcall; abstract;

    // zjisti podle pripony souboru, jestli jde o link ("lnk", "pif" nebo "url"); 'fileExtension'
    // je pripona souboru (ukazatel za tecku), nesmi byt NULL; vraci 1 pokud jde o link, jinak
    // vraci 0; POZNAMKA: pouziva se pro plneni CFileData::IsLink
    // lze volat z libovolneho threadu
    function IsFileLink( fileExtension : PChar ) : Integer; virtual; stdcall; abstract;

    // vrati ILC_COLOR??? podle verze Windows - odladene pro pouziti imagelistu v listviewech
    // typicke pouziti: ImageList_Create(16, 16, ILC_MASK | GetImageListColorFlags(), ???, ???)
    // lze volat z libovolneho threadu
    function GetImageListColorFlags : DWORD; virtual; stdcall; abstract;

    // "bezpecna" verze GetOpenFileName()/GetSaveFileName() resi situaci, kdy podana cesta
    // v OPENFILENAME::lpstrFile neni platna (napriklad z:\); v tomto pripade std. API verze
    // funkce neotevre okenko a tise se vrati z FALSE a CommDlgExtendedError() vraci FNERR_INVALIDFILENAME.
    // Nasledujici dve funkce v tomto pripade zavolaji API jeste jednou, ale s "bezpecne"
    // existujici cestou (My Documents, pripadne Desktop).
    function SafeGetOpenFileName( lpofn : POpenFilename ) : BOOL; virtual; stdcall; abstract;
    function SafeGetSaveFileName( lpofn : POpenFilename ) : BOOL; virtual; stdcall; abstract;

    // plugin musi pred pouzitim OpenHtmlHelp() zadat Salamanderovi jmeno sveho .chm souboru
    // bez cesty (napr. "demoplug.chm")
    // lze volat z libovolneho threadu, ale je potreba vyloucit soucasne volani s OpenHtmlHelp()
    procedure SetHelpFileName( chmName : PChar ); virtual; stdcall; abstract;

    // otevre HTML help pluginu, jazyk helpu (adresar s .chm soubory) vybira takto:
    // -adresar ziskany z aktualniho .slg souboru Salamandera (viz SLGHelpDir v shared\versinfo.rc)
    // -HELP\ENGLISH\*.chm
    // -prvni nalezeny podadresar v podadresari HELP
    // plugin musi pred pouzitim OpenHtmlHelp() zavolat SetHelpFileName(); 'parent' je parent
    // messageboxu s chybou; 'command' je prikaz HTML helpu, viz HHCDisplayXXX; 'dwData' je parametr
    // prikazu HTML helpu, viz HHCDisplayXXX
    // lze volat z libovolneho threadu
    function OpenHtmlHelp( parent : HWND; command : THtmlHelpCommand; dwData : UINT;
                           quiet : BOOL ) : BOOL; virtual; stdcall; abstract;

    // nacte do alokovane pameti (volajici je odpovedny za dealokaci) obsah registracniho klice
    // (z archivu keys*.zip nebo ze souboru (vybalene klice verze 2.0)); 'dllInstance' je modul
    // pluginu nebo je NULL (pak jde primo o Salamandera); 'keyName' je relativni jmeno souboru
    // klice v archivu keys*.zip (napr. "plugins\\demoplug\\demoplug.key"), neni-li 'dllInstance'
    // NULL a neexistuje archiv keys*.zip, hleda se tento soubor u modulu pluginu (SPL souboru)
    // a to bez cesty v archivu (tedy napr. jen "demoplug.key"); v 'keyNotFound' vraci TRUE pokud
    // pozadovany klic nebyl nalezen; pri uspechu v 'keyContent' vraci alokovany obsah klice
    // (volajici je odpovedny za dealokaci); v 'keyContentSize' je velikost obsahu klice ('keyContent');
    // 'fullKeyName' je buffer (minimalne 2 * MAX_PATH znaku), ve kterem se vraci plna cesta
    // k souboru klice (cesta do keys*.zip archivu nebo na disk); vraci TRUE pri uspechu
    // lze volat z libovolneho threadu
    function ReadRegistrationKey( dllInstance : HINST; keyName : PChar; keyNotFound : PBOOL;
                                  keyContent : PChar; keyContentSize : PDWORD;
                                  fullKeyName : PChar ) : BOOL; virtual; stdcall; abstract;
  end;

// detekci zda kombinace ma tlacitko SKIP nebo YES nechavam zde ve forme inline, aby
// v pripade zavadeni novych kombinaci byla dobre na ocich a nezapomeli jsme ji doplnit
function ButtonsContainsSkip( btn : DWORD ) : BOOL;
function ButtonsContainsYes( btn : DWORD ) : BOOL;

implementation

// detekci zda kombinace ma tlacitko SKIP nebo YES nechavam zde ve forme inline, aby
// v pripade zavadeni novych kombinaci byla dobre na ocich a nezapomeli jsme ji doplnit
function ButtonsContainsSkip( btn : DWORD ) : BOOL;
begin
  Result:= ((btn and BUTTONS_MASK) = BUTTONS_SKIPCANCEL) or
           ((btn and BUTTONS_MASK) = BUTTONS_RETRYSKIPCANCEL) or
           ((btn and BUTTONS_MASK) = BUTTONS_YESALLSKIPCANCEL);
end;

function ButtonsContainsYes( btn : DWORD ) : BOOL;
begin
  Result:= ((btn and BUTTONS_MASK) = BUTTONS_YESALLSKIPCANCEL) or
           ((btn and BUTTONS_MASK) = BUTTONS_YESNOCANCEL) or
           ((btn and BUTTONS_MASK) = BUTTONS_YESALLCANCEL);
end;

end.

